/*
 *      Web server handler routines for DHCP Server stuffs
 *      Authors: Kaohj	<kaohj@realtek.com.tw>
 *
 */


/*-- System inlcude files --*/
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <dirent.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <signal.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <sys/wait.h>
#include <time.h>
#include <net/route.h>
#include <sys/ioctl.h>
#include <signal.h>
#include <sys/ioctl.h>
#include <net/if.h>

/*-- Local inlcude files --*/
#include "../webs.h"
#include "fmdefs.h"
#include "mib.h"
#include "utility.h"
#include "debug.h"
#include "../defs.h"


#ifdef __i386__
#define _LITTLE_ENDIAN_
#endif

#define _DHCPD_PID_FILE			T("/var/run/udhcpd.pid")
#define _DHCPD_LEASES_FILE		T("/var/udhcpd/udhcpd.leases")

extern int read_pid(char *filename);

/*-- Macro declarations --*/
#ifdef _LITTLE_ENDIAN_
#define ntohdw(v) ( ((v&0xff)<<24) | (((v>>8)&0xff)<<16) | (((v>>16)&0xff)<<8) | ((v>>24)&0xff) )

#else
#define ntohdw(v) (v)
#endif

///////////////////////////////////////////////////////////////////
#define goto_dhcp_check_err(errorStr) \
    do {lineno = __LINE__; strcpy(tmpBuf, T(errorStr)); goto check_err;} while(0)

int setLanIp(unsigned int uiIp, unsigned int uiMask)
{
	struct in_addr inOrigIp, inOrigMask;
	unsigned char ucLanIpChanged = 0, ucWLanState = 0;

	mib_get(MIB_ADSL_LAN_IP, (void *)&inOrigIp);
	if (inOrigIp.s_addr != uiIp)
	{
		ucLanIpChanged = 1;
		mib_set(MIB_ADSL_LAN_DHCP_GATEWAY, (void *)&uiIp);
        mib_set(MIB_ADSL_LAN_IP, (void *)&uiIp);
	}

	mib_get(MIB_ADSL_LAN_SUBNET, (void *)&inOrigMask);
	if (inOrigMask.s_addr != uiMask)
	{
		ucLanIpChanged = 1;
        mib_set(MIB_ADSL_LAN_SUBNET, (void *)&uiMask);
	}

    if (ucLanIpChanged)
    {
        mib_update(CURRENT_SETTING, CONFIG_MIB_ALL);
		restart_lanip();

		//setupIngressSetting();
    }
    
    return ucLanIpChanged;
}

int setDhcpMode(unsigned char ucMode, void *pvPara)
{
	unsigned char ucOrigMode, ucDhcpdChanged = 0, ucWLanState = 0;
	struct in_addr inOrigDhcpPoolStart, inOrigDhcpPoolEnd, inOrigRelayServer;
    unsigned int uiOrigLease;
    dhcpd_entry *pstDhcpdEntry;
    
    mib_get(MIB_DHCP_MODE, (void *)&ucOrigMode);
    if (ucOrigMode != ucMode)
    {
        ucDhcpdChanged = 1;
        mib_set(MIB_DHCP_MODE, (void *)&ucMode);
    }
    
    switch (ucMode)
    {
        case DHCP_LAN_SERVER:
            if (DHCP_LAN_SERVER == ucOrigMode)
            {
                pstDhcpdEntry = (dhcpd_entry *)pvPara;

        		mib_get(MIB_DHCP_POOL_START, (void *)&inOrigDhcpPoolStart);
        		if (inOrigDhcpPoolStart.s_addr != pstDhcpdEntry->dhcpRangeStart)
        		{
                    ucDhcpdChanged = 1;
                    mib_set(MIB_DHCP_POOL_START, (void *)&pstDhcpdEntry->dhcpRangeStart);
        		}

        		mib_get(MIB_DHCP_POOL_END, (void *)&inOrigDhcpPoolEnd);
        		if (inOrigDhcpPoolEnd.s_addr != pstDhcpdEntry->dhcpRangeEnd)
        		{
                    ucDhcpdChanged = 1;
                    mib_set(MIB_DHCP_POOL_END, (void *)&pstDhcpdEntry->dhcpRangeEnd);
        		}

        		mib_get(MIB_ADSL_LAN_DHCP_LEASE, (void *)&uiOrigLease);
        		if (uiOrigLease != pstDhcpdEntry->ulTime)
        		{
                    ucDhcpdChanged = 1;
                    mib_set(MIB_ADSL_LAN_DHCP_LEASE, (void *)&pstDhcpdEntry->ulTime);
        		}                
            }
            
            break;

        case DHCP_LAN_NONE:
            break;

        default: /* DHCP_LAN_RELAY */
			mib_get(MIB_ADSL_WAN_DHCPS, (void*)&inOrigRelayServer);
			if (inOrigRelayServer.s_addr != *(unsigned long *)pvPara)
			{
				ucDhcpdChanged = 1;
			    mib_set(MIB_ADSL_WAN_DHCPS, pvPara);
			}
            
            break;
    }

    if (ucDhcpdChanged)
    {
        mib_update(CURRENT_SETTING, CONFIG_MIB_ALL);
		restart_dhcp();
    }
    
    return ucDhcpdChanged;
}

//add by chenzhuoxin
void formDhcpd(webs_t wp, char_t *path, char_t *query)
{
	struct dhcpd_entry	entry;
	unsigned int	uIp = 0;            //	modem ip   
	unsigned int	uMask = 0;          //	modem netmask
	unsigned char	uDhcpType = 0;		//	0- dhcpserver;  1- dhcpserver ; 2-enable dhcprelay
	unsigned int	uServerIp = 0;		//	dhcp relay server ip
	char*			stemp = "";
	int				lineno = __LINE__;
	char            tmpBuf[128];
    void            *pvDhcpdPara = NULL; 
    unsigned char   ucDhcpdMode = DHCP_LAN_NONE;
    
    tmpBuf[0] = 0;
    
	_TRACE_CALL;

	//modem ip
	_GET_IP(uIp, _NEED);
	if(uIp == 0)
	{
        goto_dhcp_check_err(strWrongIP);
	}
    
	//netmask
	_GET_IP(uMask, _NEED);
	if(uMask == 0)
	{
        goto_dhcp_check_err(strWrongMask);
	}
    
	//service type
	_GET_INT(uDhcpType, _NEED);
	if(uDhcpType < 0 || uDhcpType > 2)
	{
        /* impossible */
        goto_dhcp_check_err(strSetDhcpModeerror);
	}

	if(uDhcpType == 1)	//1- dhcpserver
	{
		_ENTRY_IP(dhcpRangeStart, _NEED);
		if(entry.dhcpRangeStart == 0)
    	{
            goto_dhcp_check_err(strInvalidRange);
    	}
        
		_ENTRY_IP(dhcpRangeEnd, _NEED);
		if(entry.dhcpRangeEnd == 0)
    	{
            goto_dhcp_check_err(strInvalidRange);
    	}
        
        if (entry.dhcpRangeEnd < entry.dhcpRangeStart)
    	{
            goto_dhcp_check_err(strInvalidRange);
    	}
        
		_ENTRY_IP(ipMask, _NEED);
		if(entry.ipMask == 0)
    	{
            goto_dhcp_check_err(strWrongMask);
    	}
        
		_ENTRY_INT(ulTime, _NEED);
		if(entry.ulTime == 0)
    	{
            goto_dhcp_check_err(strSetLeaseTimeerror);
    	}

        ucDhcpdMode = DHCP_LAN_SERVER;
        pvDhcpdPara = (void *)&entry;
	}
	else if(uDhcpType == 2)	//1- enable dhcprelay
	{
		//	dhcp relay server ip
		_GET_IP(uServerIp, _NEED);
		if(uServerIp == 0)
    	{
            goto_dhcp_check_err(strInvalidGatewayerror);
    	}

        ucDhcpdMode = DHCP_LAN_RELAY;
        pvDhcpdPara = (void *)&uServerIp;
	}
	
	/*
	*Add code by Realtek
	*/
    setLanIp(uIp, uMask);
    setDhcpMode(ucDhcpdMode, pvDhcpdPara);
    
	_COND_REDIRECT;
	_TRACE_LEAVEL;
	return;

check_err:
	_TRACE_LEAVEL;
	ERR_MSG(tmpBuf);
    return;
}

//add by chenzhuoxin
int init_dhcpmain_page(int eid, webs_t wp, int argc, char_t **argv)
{
	struct dhcpd_entry	entry;
	unsigned int	uIp;           //modem ip   
	unsigned int	uMask;         // modem netmask
	unsigned char	uDhcpType = 1;				//0- ;  1- dhcpserver ; 2-enable dhcprelay
	unsigned int	uServerIp;     //dhcp rerty server ip
	int				lineno = __LINE__;
	
	_TRACE_CALL;

	/*
	*Add code by Realtek
	*/
	mib_get(MIB_ADSL_LAN_IP, (void *)&uIp);
	mib_get(MIB_ADSL_LAN_SUBNET, (void *)&uMask);
    
	mib_get(MIB_DHCP_MODE, (void *)&uDhcpType);
    switch (uDhcpType)
    {
        case DHCP_LAN_SERVER:
            uDhcpType = 1;
            break;

        case DHCP_LAN_RELAY:
            uDhcpType = 2;
            break;

        default:
            uDhcpType = 0;
            break;
    }

    memset(&entry, 0, sizeof(dhcpd_entry));
	mib_get(MIB_DHCP_POOL_START, (void *)&entry.dhcpRangeStart);
    mib_get(MIB_DHCP_POOL_END, (void *)&entry.dhcpRangeEnd);
    mib_get(MIB_ADSL_LAN_DHCP_LEASE, (void *)&entry.ulTime);
    entry.ipMask = uMask;
    
    mib_get(MIB_ADSL_WAN_DHCPS, (void*)&uServerIp);
	
	_PUT_IP(uIp);
	_PUT_IP(uMask);

	_PUT_INT(uDhcpType);
	
	_PUT_ENTRY_IP(dhcpRangeStart);
	_PUT_ENTRY_IP(dhcpRangeEnd);
	_PUT_ENTRY_IP(ipMask);
	_PUT_ENTRY_INT(ulTime);

	_PUT_IP(uServerIp);

check_err:
	_TRACE_LEAVEL;
	return 0;
}

#ifdef IP_BASED_CLIENT_TYPE    
int setClientIpRange(dhcp_client_entry *pstClientIpRange)
{
    dhcp_client_entry stOrigClientIpRange;

    memset(&stOrigClientIpRange, 0, sizeof(dhcp_client_entry));

	mib_get(CWMP_CT_PC_MINADDR, (void*)&stOrigClientIpRange.pcRangeStart);
	mib_get(CWMP_CT_PC_MAXADDR, (void*)&stOrigClientIpRange.pcRangeEnd);
	mib_get(CWMP_CT_CMR_MINADDR, (void*)&stOrigClientIpRange.cmrRangeStart);
	mib_get(CWMP_CT_CMR_MAXADDR, (void*)&stOrigClientIpRange.cmrRangeEnd);
	mib_get(CWMP_CT_STB_MINADDR, (void*)&stOrigClientIpRange.stbRangeStart);
	mib_get(CWMP_CT_STB_MAXADDR, (void*)&stOrigClientIpRange.stbRangeEnd);

    if (0 != memcmp(&stOrigClientIpRange, pstClientIpRange, sizeof(dhcp_client_entry)))
    {
    	mib_set(CWMP_CT_PC_MINADDR, (void*)&pstClientIpRange->pcRangeStart);
    	mib_set(CWMP_CT_PC_MAXADDR, (void*)&pstClientIpRange->pcRangeEnd);
    	mib_set(CWMP_CT_CMR_MINADDR, (void*)&pstClientIpRange->cmrRangeStart);
    	mib_set(CWMP_CT_CMR_MAXADDR, (void*)&pstClientIpRange->cmrRangeEnd);
    	mib_set(CWMP_CT_STB_MINADDR, (void*)&pstClientIpRange->stbRangeStart);
    	mib_set(CWMP_CT_STB_MAXADDR, (void*)&pstClientIpRange->stbRangeEnd);

        mib_update(CURRENT_SETTING, CONFIG_MIB_ALL);
        restart_dhcp();
        return 1;
    }
    else
    {
        return 0;
    }
}
#endif

//add by chenhzuoxin
void formIpRange(webs_t wp, char_t *path, char_t *query)
{
#ifdef IP_BASED_CLIENT_TYPE    
	struct dhcp_client_entry	entry;
	unsigned char	uDhcpType = 0;		//	0- dhcpserver;  1- dhcpserver ; 2-enable dhcprelay
	unsigned int	uServerIp = 0;		//	dhcp relay server ip
	char*			stemp = "";
	int				lineno = __LINE__;
	char            tmpBuf[128];
	unsigned int    uStart;
    unsigned int    uEnd;
	
	_TRACE_CALL;

	//pc
	_ENTRY_IP(pcRangeStart, _NEED);
	if(entry.pcRangeStart == 0)
	{
        goto_dhcp_check_err(strInvalidRangepc);
	}
    
	_ENTRY_IP(pcRangeEnd, _NEED);
	if(entry.pcRangeEnd == 0 || entry.pcRangeEnd < entry.pcRangeStart)
	{
        goto_dhcp_check_err(strInvalidRangepc);
	}
    
	//ͷ
	_ENTRY_IP(cmrRangeStart, _NEED);
	if(entry.cmrRangeStart == 0)
	{
        goto_dhcp_check_err(strInvalidRangecmr);
	}
    
	_ENTRY_IP(cmrRangeEnd, _NEED);
	if(entry.cmrRangeEnd == 0 || entry.cmrRangeEnd < entry.cmrRangeStart)
	{
        goto_dhcp_check_err(strInvalidRangecmr);
	}
    
	//
	_ENTRY_IP(stbRangeStart, _NEED);
	if(entry.stbRangeStart == 0)
	{
        goto_dhcp_check_err(strInvalidRangestb);
	}
    
	_ENTRY_IP(stbRangeEnd, _NEED);
	if(entry.stbRangeEnd == 0 || entry.stbRangeEnd < entry.stbRangeStart)
	{
        goto_dhcp_check_err(strInvalidRangestb);
	}
    
	/*
	*Add code by Realtek
	*/
	{
        unsigned int clientIpPool[3][2] = {{entry.pcRangeStart, entry.pcRangeEnd}, 
                                           {entry.cmrRangeStart, entry.cmrRangeEnd}, 
                                           {entry.stbRangeStart, entry.stbRangeEnd}};
        int             i, j;
    	
    	mib_get(MIB_DHCP_POOL_START, (void *)&uStart);
    	mib_get(MIB_DHCP_POOL_END, (void *)&uEnd);

        for (i = 0; i < 3; i++)
        {
            for (j = 0; j < 2; j++)
            {
                if ((clientIpPool[i][j] < uStart) || (clientIpPool[i][j] > uEnd))
                {
                    goto_dhcp_check_err(strInvalidTypeRange);
                }
            }
        }

        for (i = 0; i < 2; i++)
        {
            for (j = i + 1; j < 3; j++)
            {
                if ((clientIpPool[i][1] >= clientIpPool[j][0])
                    && (clientIpPool[j][1] >= clientIpPool[i][0])
                    )
                {
                    goto_dhcp_check_err(strOverlapRange);
                }
            }
        }
        
        setClientIpRange(&entry);
	}
    
	_COND_REDIRECT;
	_TRACE_LEAVEL;
	return;
    
check_err:
	_TRACE_LEAVEL;
	ERR_MSG(tmpBuf);
	return;
#endif
}

//dhcp_device init function
int init_dhcp_device_page(int eid, webs_t wp, int argc, char_t **argv)
{
	struct dhcp_client_entry entry;
	int	lineno = __LINE__;

	_TRACE_CALL;

	/*
	*Add code by Realtek
	*/
    memset(&entry, 0, sizeof(dhcp_client_entry));
	mib_get(CWMP_CT_PC_MINADDR, (void*)&entry.pcRangeStart);
	mib_get(CWMP_CT_PC_MAXADDR, (void*)&entry.pcRangeEnd);
	mib_get(CWMP_CT_CMR_MINADDR, (void*)&entry.cmrRangeStart);
	mib_get(CWMP_CT_CMR_MAXADDR, (void*)&entry.cmrRangeEnd);
	mib_get(CWMP_CT_STB_MINADDR, (void*)&entry.stbRangeStart);
	mib_get(CWMP_CT_STB_MAXADDR, (void*)&entry.stbRangeEnd);
    
	//pc
	_PUT_ENTRY_IP(pcRangeStart);
	_PUT_ENTRY_IP(pcRangeEnd);
	//ͷ
	_PUT_ENTRY_IP(cmrRangeStart);
	_PUT_ENTRY_IP(cmrRangeEnd);
	//
	_PUT_ENTRY_IP(stbRangeStart);
	_PUT_ENTRY_IP(stbRangeEnd);

check_err:
	_TRACE_LEAVEL;
	return 0;
}

//add by chenzhuoxin
void formMacBase(webs_t wp, char_t *path, char_t *query)
{
	MIB_CE_MAC_BASE_DHCP_T	entry;
	MIB_CE_MAC_BASE_DHCP_T  tmpMacIp;
	char tmpBuf[128];
	int mibtotal,i;
	char*			stemp = "";
	int				lineno = __LINE__;
	
	_TRACE_CALL;

	FETCH_INVALID_OPT(stemp, "action", _NEED);
	
	if(strcmp(stemp, "sv") == 0)	//add
	{
		_ENTRY_STR(macAddr, _NEED);
		_ENTRY_STR(ipAddr, _NEED);

		/************Place your code here, do what you want to do! ************/
		mibtotal = mib_chain_total(MIB_MAC_BASE_DHCP_TBL);
#define MAX_MAC_BASE_DHCP_ENTRY 10
        if (mibtotal >= MAX_MAC_BASE_DHCP_ENTRY)
        {
            goto_dhcp_check_err(strTableFull);
        }
        
		for (i = 0; i < mibtotal; i++)
		{
			mib_chain_get(MIB_MAC_BASE_DHCP_TBL, i, (void*)&tmpMacIp);
			if(!strcmp(entry.macAddr, tmpMacIp.macAddr) || !strcmp(entry.ipAddr, tmpMacIp.ipAddr))
            {
                goto_dhcp_check_err(strStaticipexist);
			}
		}

		mib_chain_add(MIB_MAC_BASE_DHCP_TBL, (unsigned char*)&entry);
		/************Place your code here, do what you want to do! ************/
	}
	else if(strcmp(stemp, "rm") == 0) //remove
	{
		_ENTRY_STR(macAddr, _NEED);
		_ENTRY_STR(ipAddr, _NEED);

		/************Place your code here, do what you want to do! ************/
	    mibtotal = mib_chain_total(MIB_MAC_BASE_DHCP_TBL);
		for (i = 0; i < mibtotal; i++)
		{
			mib_chain_get(MIB_MAC_BASE_DHCP_TBL, i, (void*)&tmpMacIp);
			if(!strcmp(entry.macAddr, tmpMacIp.macAddr) || !strcmp(entry.ipAddr, tmpMacIp.ipAddr))
            {
				break;
			}
		}

		mib_chain_delete(MIB_MAC_BASE_DHCP_TBL, i);
		/************Place your code here, do what you want to do! ************/
	}
	else
    {
		goto_dhcp_check_err("");
	}

    mib_update(CURRENT_SETTING, CONFIG_MIB_ALL);
    restart_dhcp();
    
	_COND_REDIRECT;
	_TRACE_LEAVEL;
	return;
    
check_err:
	_TRACE_LEAVEL;
	ERR_MSG(tmpBuf);
	return;
}

//add by chenzhuoxin
int showMACBaseTable(int eid, webs_t wp, int argc, char_t **argv)
{
	MIB_CE_MAC_BASE_DHCP_T	entry;
	int				cnt = 0;
	int				index = 0;
	int				lineno = __LINE__;
	
	_TRACE_CALL;

	/*test code*/
    /*
	cnt = 1;
	memset(&entry, 0, sizeof(entry));
	strcpy(entry.macAddr, "00-11-22-33-44-55");
	strcpy(entry.ipAddr, "192.168.1.2");
	*/
	/*test code*/

	/************Place your code here, do what you want to do! ************/
    cnt = mib_chain_total(MIB_MAC_BASE_DHCP_TBL);
	/************Place your code here, do what you want to do! ************/
	
	for(index = 0; index < cnt; index++)
	{
		/************Place your code here, do what you want to do! ************/
        mib_chain_get(MIB_MAC_BASE_DHCP_TBL, index, (void*)&entry);
		websWrite(wp, T("dmacips.push(new it_nr(\"%d\"" _PTS _PTS "));\n"), index, _PME(macAddr), _PME(ipAddr));
		/************Place your code here, do what you want to do! ************/

		/* websWrite(wp, T("dmacips.push(new it_nr(\"%d\"" _PTS _PTS "));\n"), index, _PME(macAddr), _PME(ipAddr)); */
	}

check_err:
	_TRACE_LEAVEL;
	return 0;
}

#if 0
/////////////////////////////////////////////////////////////////////////////
#ifdef EMBED
static int getOneDhcpClient(char **ppStart, unsigned long *size, char *ip, char *mac, char *liveTime)
{
	struct dhcpOfferedAddr {
        	u_int8_t chaddr[16];
        	u_int32_t yiaddr;       /* network order */        	
        	u_int32_t expires;      /* host order */        	
	};

	struct dhcpOfferedAddr entry;
#if defined(ZTE_531B_BRIDGE_SC)||defined(ZTE_GENERAL_ROUTER_SC)
	u_int32_t day,hour,minute,second,time,len=0;
#endif

	if ( *size < sizeof(entry) )
		return -1;

	entry = *((struct dhcpOfferedAddr *)*ppStart);
	*ppStart = *ppStart + sizeof(entry);
	*size = *size - sizeof(entry);

	if (entry.expires == 0) 		
		return 0;
//star: conflict ip addr will not be displayed on web
	if(entry.chaddr[0]==0&&entry.chaddr[1]==0&&entry.chaddr[2]==0&&entry.chaddr[3]==0&&entry.chaddr[4]==0&&entry.chaddr[5]==0)
		return 0;
	
	strcpy(ip, inet_ntoa(*((struct in_addr *)&entry.yiaddr)) );
	#if defined(ZTE_531B_BRIDGE_SC)||defined(ZTE_GENERAL_ROUTER_SC)
	snprintf(mac, 20, "%02x-%02x-%02x-%02x-%02x-%02x",
			entry.chaddr[0],entry.chaddr[1],entry.chaddr[2],entry.chaddr[3],
			entry.chaddr[4], entry.chaddr[5]);
	#else
	snprintf(mac, 20, "%02x:%02x:%02x:%02x:%02x:%02x",
			entry.chaddr[0],entry.chaddr[1],entry.chaddr[2],entry.chaddr[3],
			entry.chaddr[4], entry.chaddr[5]);
	#endif

#if defined(ZTE_531B_BRIDGE_SC)||defined(ZTE_GENERAL_ROUTER_SC)
	time = ntohl(entry.expires);
	if(time == -1){
		strcpy(liveTime, strShowDhcptime);
		return 1;
	}
	day = time/86400;
	hour = (time%86400)/3600;
	minute = ((time%86400)%3600)/60;
	second = ((time%86400)%3600)%60;

	if(day!=0)
		len=snprintf(liveTime, 20, "%lu%s", day,Tpublic_day2);
	if(hour!=0)
		len+=snprintf(liveTime+len, 20, "%lu%s", hour,Tpublic_hour2);
	if(minute!=0)
		len+=snprintf(liveTime+len, 20, "%lu%s", minute,Tpublic_minute);
	if(second!=0)
		snprintf(liveTime+len, 20, "%lu%s", second,Tpublic_second);
	if(day == 0 && hour == 0 && minute == 0 && second == 0)
		snprintf(liveTime+len, 20, "0%s",Tpublic_second);
#else
	snprintf(liveTime, 10, "%lu", (unsigned long)ntohl(entry.expires));
#endif

	return 1;
}
#endif
#endif
/////////////////////////////////////////////////////////////////////////////
void formReflashClientTbl(webs_t wp, char_t *path, char_t *query)
{
	char_t *submitUrl;

	submitUrl = websGetVar(wp, T("submit-url"), T(""));
	if (submitUrl[0])
		websRedirect(wp, submitUrl);
}

//////////////////////////////////////////////////////////////////////////////
int isDhcpClientExist(char *name)
{
/*
	char tmpBuf[100];
	struct in_addr intaddr;
	
	if ( getInAddr(name, IP_ADDR, (void *)&intaddr ) ) {
		snprintf(tmpBuf, 100, "%s/%s-%s.pid", _DHCPC_PID_PATH, _DHCPC_PROG_NAME, name);
		if ( getPid(tmpBuf) > 0)
			return 1;
	}
*/	
	return 0;
}


