/*
 *      Web server handler routines for get info and index (getinfo(), getindex())
 *      Authors: David Hsu	<davidhsu@realtek.com.tw>
 *      Authors: Dick Tam	<dicktam@realtek.com.tw>
 *
 */

#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <sys/sysinfo.h>
#ifdef EMBED
#include <linux/config.h>
#include <config/autoconf.h>
#else
#include "../../../uClibc/include/linux/autoconf.h"
#include "../../../config/autoconf.h"
#endif

#include "../webs.h"
#include "mib.h"
#include "adsl_drv.h"
#include "utility.h"
//added by xl_yue
#include "../defs.h"

// remote config status flag: 0: disabled, 1: enabled
int g_remoteConfig=0;
int g_remoteAccessPort=51003;

// Added by Mason Yu
extern char suName[MAX_NAME_LEN];
extern char usName[MAX_NAME_LEN];

#ifdef WLAN_SUPPORT
static void translate_control_code(char *buffer)
{
	char tmpBuf[200], *p1 = buffer, *p2 = tmpBuf;


	while (*p1) {
		if (*p1 == '"') {
			memcpy(p2, "&quot;", 6);
			p2 += 6;
		}
		else
			*p2++ = *p1;
		p1++;
	}
	*p2 = '\0';

	strcpy(buffer, tmpBuf);
}
#endif

// Kaohj
typedef enum {
	INFO_MIB,
	INFO_SYS
} INFO_T;

typedef struct {
	char *cmd;
	INFO_T type;
	int id;
} web_get_cmd;

typedef struct {
	char *cmd;
	int (*handler)(int , webs_t , int , char_t **, char *);
} web_custome_cmd;

web_get_cmd get_info_list[] = {
	{"lan-ip", INFO_MIB, MIB_ADSL_LAN_IP},
	{"lan-subnet", INFO_MIB, MIB_ADSL_LAN_SUBNET},
	{"lan-ip2", INFO_MIB, MIB_ADSL_LAN_IP2},
	{"lan-subnet2", INFO_MIB, MIB_ADSL_LAN_SUBNET2},
	{"lan-dhcpRangeStart", INFO_MIB, MIB_ADSL_LAN_CLIENT_START},
	{"lan-dhcpRangeEnd", INFO_MIB, MIB_ADSL_LAN_CLIENT_END},
	{"lan-dhcpLTime", INFO_MIB, MIB_ADSL_LAN_DHCP_LEASE},
	{"lan-dhcpDName", INFO_MIB, MIB_ADSL_LAN_DHCP_DOMAIN},
	{"elan-Mac", INFO_MIB, MIB_ELAN_MAC_ADDR},
	{"wlan-Mac", INFO_MIB, MIB_WLAN_MAC_ADDR},
	{"wan-dns1", INFO_MIB, MIB_ADSL_WAN_DNS1},
	{"wan-dns2", INFO_MIB, MIB_ADSL_WAN_DNS2},
	{"wan-dns3", INFO_MIB, MIB_ADSL_WAN_DNS3},
	{"wan-dhcps", INFO_MIB, MIB_ADSL_WAN_DHCPS},
	{"dmzHost", INFO_MIB, MIB_DMZ_IP},
	{"snmpSysDescr", INFO_MIB, MIB_SNMP_SYS_DESCR},
	{"snmpSysContact", INFO_MIB, MIB_SNMP_SYS_CONTACT},
	{"snmpSysName", INFO_MIB, MIB_SNMP_SYS_NAME},
	{"snmpSysLocation", INFO_MIB, MIB_SNMP_SYS_LOCATION},
	{"snmpSysObjectID", INFO_MIB, MIB_SNMP_SYS_OID},
	{"snmpTrapIpAddr", INFO_MIB, MIB_SNMP_TRAP_IP},
	{"snmpCommunityRO", INFO_MIB, MIB_SNMP_COMM_RO},
	{"snmpCommunityRW", INFO_MIB, MIB_SNMP_COMM_RW},
	{"name", INFO_MIB, MIB_SNMP_SYS_NAME},
#if defined(ZTE_GENERAL_ROUTER_SC) || defined(TIME_ZONE)
	{"ntpTimeZone", INFO_MIB, MIB_NTP_TIMEZONE},
	{"ntpServerIp1", INFO_MIB, MIB_NTP_SERVER_IP1},
	{"ntpServerIp2", INFO_MIB, MIB_NTP_SERVER_IP2},
#endif
#ifdef ZTE_GENERAL_ROUTER_SC
	{"ntpServerEbl", INFO_MIB, MIB_NTP_ENABLED},
	{"ntpServerId", INFO_MIB, MIB_NTP_SERVER_ID},
#endif
	{"uptime", INFO_SYS, SYS_UPTIME},
	{"date", INFO_SYS, SYS_DATE},
	{"year", INFO_SYS, SYS_YEAR},
	{"month", INFO_SYS, SYS_MONTH},
	{"day", INFO_SYS, SYS_DAY},
	{"hour", INFO_SYS, SYS_HOUR},
	{"minute", INFO_SYS, SYS_MINUTE},
	{"second", INFO_SYS, SYS_SECOND},
	{"fwVersion", INFO_SYS, SYS_FWVERSION},
	{"bridge-ageingTime", INFO_MIB, MIB_BRCTL_AGEINGTIME},
#ifdef CONFIG_USER_IGMPPROXY
	{"igmp-proxy-itf", INFO_MIB, MIB_IGMP_PROXY_ITF},
#endif
//#ifdef CONFIG_USER_UPNPD	
#if defined(CONFIG_USER_UPNPD)||defined(CONFIG_USER_MINIUPNPD)
	{"upnp-ext-itf", INFO_MIB, MIB_UPNP_EXT_ITF},
#endif	
#ifdef AUTO_PROVISIONING
	{"http-ip", INFO_MIB, MIB_HTTP_SERVER_IP},
#endif
#ifdef IP_PASSTHROUGH
	{"ippt-itf", INFO_MIB, MIB_IPPT_ITF},
	{"ippt-lease", INFO_MIB, MIB_IPPT_LEASE},
	{"ippt-lanacc", INFO_MIB, MIB_IPPT_LANACC},
#endif
#ifdef WLAN_SUPPORT
	{"ssid", INFO_MIB, MIB_WLAN_SSID},
	{"channel", INFO_MIB, MIB_WLAN_CHAN_NUM},
	{"fragThreshold", INFO_MIB, MIB_WLAN_FRAG_THRESHOLD},
	{"rtsThreshold", INFO_MIB, MIB_WLAN_RTS_THRESHOLD},
	{"beaconInterval", INFO_MIB, MIB_WLAN_BEACON_INTERVAL},
	{"wlanDisabled",INFO_MIB,MIB_WLAN_DISABLED},

	{"pskValue", INFO_SYS, SYS_WLAN_PSKVAL},
#ifdef WLAN_1x
	{"rsPort",INFO_MIB,MIB_WLAN_RS_PORT},
	{"rsIp",INFO_MIB,MIB_WLAN_RS_IP},
	{"rsPassword",INFO_MIB,MIB_WLAN_RS_PASSWORD},
	{"enable1X", INFO_MIB, MIB_WLAN_ENABLE_1X},
#endif
	{"wlanMode",INFO_MIB,MIB_WLAN_MODE},
	{"encrypt",INFO_MIB,MIB_WLAN_ENCRYPT},
	{"wpaAuth",INFO_MIB,MIB_WLAN_WPA_AUTH},
	{"networkType",INFO_MIB,MIB_WLAN_NETWORK_TYPE},

#ifdef CONFIG_WIFI_SIMPLE_CONFIG // WPS
	{"wscDisable",INFO_MIB,MIB_WSC_DISABLE},
	//{"wscConfig",INFO_MIB,MIB_WSC_CONFIGURED},
	{"wps_auth",INFO_MIB,MIB_WSC_AUTH},
	{"wps_enc",INFO_MIB,MIB_WSC_ENC},
	{"wscLoocalPin", INFO_MIB, MIB_WSC_PIN},

#endif

#ifdef WLAN_WDS
	{"wlanWdsEnabled",INFO_MIB,MIB_WLAN_WDS_ENABLED},
#endif
#endif
	//{ "dnsServer", INFO_SYS, SYS_DNS_SERVER},
	{"maxmsglen",INFO_MIB,MIB_MAXLOGLEN},
#ifdef _CWMP_MIB_
	{"acs-url", INFO_MIB, CWMP_ACS_URL},
	{"acs-username", INFO_MIB, CWMP_ACS_USERNAME},
	{"acs-password", INFO_MIB, CWMP_ACS_PASSWORD},
	{"inform-interval", INFO_MIB, CWMP_INFORM_INTERVAL},
	{"conreq-name", INFO_MIB, CWMP_CONREQ_USERNAME},
	{"conreq-pw", INFO_MIB, CWMP_CONREQ_PASSWORD},
	{"cert-pw", INFO_MIB, CWMP_CERT_PASSWORD},
#endif
#ifdef DOS_SUPPORT
	{"syssynFlood", INFO_MIB, MIB_DOS_SYSSYN_FLOOD},
	{"sysfinFlood", INFO_MIB, MIB_DOS_SYSFIN_FLOOD},
	{"sysudpFlood", INFO_MIB, MIB_DOS_SYSUDP_FLOOD},
	{"sysicmpFlood", INFO_MIB, MIB_DOS_SYSICMP_FLOOD},
	{"pipsynFlood", INFO_MIB, MIB_DOS_PIPSYN_FLOOD},
	{"pipfinFlood", INFO_MIB, MIB_DOS_PIPFIN_FLOOD},
	{"pipudpFlood", INFO_MIB, MIB_DOS_PIPUDP_FLOOD},
	{"pipicmpFlood", INFO_MIB, MIB_DOS_PIPICMP_FLOOD},
	{"blockTime", INFO_MIB, MIB_DOS_BLOCK_TIME},
#endif
	{"lan-dhcp-gateway", INFO_MIB, MIB_ADSL_LAN_DHCP_GATEWAY},
	{NULL, 0, 0}
};

#ifdef CONFIG_WIFI_SIMPLE_CONFIG//WPS
static void convert_bin_to_str(unsigned char *bin, int len, char *out)
{
	int i;
	char tmpbuf[10];

	out[0] = '\0';

	for (i=0; i<len; i++) {
		sprintf(tmpbuf, "%02x", bin[i]);
		strcat(out, tmpbuf);
	}
}


static int fnget_wpsKey(int eid, webs_t wp, int argc, char_t **argv, char *buffer) {
	int id, intVal;
	mib_get(MIB_WSC_ENC, (void *)&intVal);
	buffer[0]='\0';
	if (intVal == WSC_ENCRYPT_WEP) {
		unsigned char tmp[100];
		mib_get(MIB_WLAN_WEP, (void *)&intVal);
		mib_get(MIB_WLAN_WEP_DEFAULT_KEY, (void *)&id);
		if (intVal == 1) {
			if (id == 0)				
				id = MIB_WLAN_WEP64_KEY1;
			else if (id == 1)				
				id = MIB_WLAN_WEP64_KEY2;
			else if (id == 2)				
				id = MIB_WLAN_WEP64_KEY3;
			else	
				id = MIB_WLAN_WEP64_KEY4;				
			mib_get(id, (void *)tmp);				
			convert_bin_to_str(tmp, 5, buffer);
		}
		else {
			if (id == 0)				
				id = MIB_WLAN_WEP128_KEY1;
			else if (id == 1)				
				id = MIB_WLAN_WEP128_KEY2;
			else if (id == 2)				
				id = MIB_WLAN_WEP128_KEY3;
			else	
				id = MIB_WLAN_WEP128_KEY4;				
			mib_get(id, (void *)tmp);				
			convert_bin_to_str(tmp, 13, buffer);				
		}			
	}
	else {
		if (intVal==0 || intVal == WSC_ENCRYPT_NONE)
			strcpy(buffer, "N/A");
		else
			mib_get(MIB_WSC_PSK, (void *)buffer);
	}
   	return websWrite(wp, buffer);
}
#endif
web_custome_cmd get_info_custom_list[] = {
	#ifdef CONFIG_WIFI_SIMPLE_CONFIG//WPS
	{ "wps_key", fnget_wpsKey },
	#endif
	{ NULL, 0 }
};

int getInfo(int eid, webs_t wp, int argc, char_t **argv)
{
	char_t	*name;
	unsigned char buffer[256];
	int idx, ret;
	
   	if (ejArgs(argc, argv, T("%s"), &name) < 1) {
   		websError(wp, 400, T("Insufficient args\n"));
   		return -1;
   	}

	memset(buffer,0x00,64);
	
 	if ( !strncmp(name, T("adsl-drv-"), 9) ) {
 		getAdslDrvInfo(&name[9], buffer	, 64);
		return websWrite(wp, T("%s"), buffer);
	}
#ifdef ZTE_GENERAL_ROUTER_SC
	if ( !strcmp(name, T("dhcp-day")) ||
		!strcmp(name, T("dhcp-hour")) ||
		!strcmp(name, T("dhcp-min")) ) {
		int time;
		int day,hour,minute;

		mib_get(MIB_ADSL_LAN_DHCP_LEASE, (void*)&time);
		if(time == -1){
			ret = websWrite(wp, T("%s"), "-1");
		}
		else{
			if ( !strcmp(name, T("dhcp-day")) ) {
				day = time/86400;
				store_day = day;
				ret = websWrite(wp, T("%d"), day);
			} else if ( !strcmp(name, T("dhcp-hour")) ) {
				hour = (time%86400)/3600;
				store_hour = hour;
				ret = websWrite(wp, T("%d"), hour);
			} else {
				minute = ((time%86400)%3600)/60;
				store_minute = minute;
				ret = websWrite(wp, T("%d"), minute);
			}
		}
		goto NEXTSTEP;
	}
#endif
#if defined(ZTE_531B_BRIDGE_SC) || defined(ZTE_GENERAL_ROUTER_SC)
	if ( !strcmp(name, T("pvcExist")) ) {
		unsigned int totalEntry;
		int i, cnt=0, selected;
		unsigned int ifMap=0;	// high half for PPP bitmap, low half for vc bitmap
		MIB_CE_ATM_VC_T Entry;

		totalEntry = mib_chain_total(MIB_ATM_VC_TBL);

		selected = -1;
		for (i=0; i<totalEntry; i++) {
			if (!mib_chain_get(MIB_ATM_VC_TBL, i, (void *)&Entry)) {
				websError(wp, 400, T(Tget_chain_err));
				return;
			}
			if ( (selected == -1) && i==0 )
				selected = i;
			else {
				ifMap |= 1 << (Entry.ifIndex & 0x0f);	// vc map
				ifMap |= (1 << 16) << ((Entry.ifIndex >> 4) & 0x0f);	// PPP map
			}
			
			if (Entry.vpi == fcEntry.vpi && Entry.vci == fcEntry.vci && selected != i) 
				cnt++;
		}
#ifdef ZTE_531B_BRIDGE_SC
		if (cnt > 0)
			ret=websWrite(wp, T("%s"), "1");
		else
			ret=websWrite(wp, T("%s"), "0");
		goto NEXTSTEP;
#else
		if(cnt > 0) {
			//Make sure there is no mismatch mode
			for (i=0, cnt=0; i<totalEntry; i++) {
				if(i==selected)
					continue;
				if (!mib_chain_get(MIB_ATM_VC_TBL, i, (void *)&Entry)){
		  			websError(wp, 400, T(Tget_chain_err));
					return;
				}
				if (Entry.vpi == fcEntry.vpi && Entry.vci == fcEntry.vci){
					cnt++;
					if(Entry.cmode != fcEntry.cmode) {
						ret = websWrite(wp, T("%s"), "1");
						goto NEXTSTEP;
					}
				}
			}
			//Max. 5 PPPoE connections
			if(fcEntry.cmode == ADSL_PPPoE && cnt==MAX_POE_PER_VC) {	// Jenny, multisession PPPoE support
				ret = websWrite(wp, T("%s"), "2");
				goto NEXTSTEP;
			//Max. 1 connection except PPPoE
			} else if(fcEntry.cmode != ADSL_PPPoE && cnt>0 ) {
				ret = websWrite(wp, T("%s"), "1");
				goto NEXTSTEP;
			}
		}
#endif
		
	}
#endif

 	for (idx=0; get_info_custom_list[idx].cmd != NULL; idx++) {
 		if (!strcmp(name, get_info_custom_list[idx].cmd)) {
 			return get_info_custom_list[idx].handler(eid, wp, argc, argv, buffer);
 		}
 	}
	
	for (idx=0; get_info_list[idx].cmd != NULL; idx++) {
		if (!strcmp(name, get_info_list[idx].cmd)) {
			if (get_info_list[idx].type == INFO_MIB) {
				if (getMIB2Str(get_info_list[idx].id, buffer)) {
					fprintf(stderr, "failed to get %s\n", name);
					return -1;
				}
			}
			else {
				if (getSYS2Str(get_info_list[idx].id, buffer))
					return -1;
			}
	#ifdef ZTE_GENERAL_ROUTER_SC
			//jim :2007-06-21  if no IP set, 0.0.0.0 will be shown on webpages to tell user how to setting ip values, 
			//				according ZTE's advise.
			if((!strncmp(name, "wan-dns", 7))&&(*buffer==0))
			{
				ret=websWrite(wp, T("0.0.0.0"));
			}
			else
			{
				ret=websWrite(wp, T("%s"), buffer);
			}
	#else
			ret = websWrite(wp, T("%s"), buffer);
	#endif
			//fprintf(stderr, "%s = %s\n", name, buffer);
			//printf("%s = %s\n", name, buffer);
			break;
		}
	}

NEXTSTEP:
	return ret;
}

#if 0
int getInfo(int eid, webs_t wp, int argc, char_t **argv)
{
	char_t	*name;
	unsigned short vShort;
	unsigned char get_fail_buf[] = "Unknown";	
	unsigned char buffer[255];
	int updays, uphours, upminutes;
	struct sysinfo info;
	time_t tm;
	struct tm tm_time;
	
   	if (ejArgs(argc, argv, T("%s"), &name) < 1) {
   		websError(wp, 400, T("Insufficient args\n"));
   		return -1;
   	}

	memset(buffer,0x00,255);
	
	if ( !strcmp(name, T("lan-ip"))) {
		if(!mib_get( MIB_ADSL_LAN_IP,  (void *)buffer))
			return -1;

   		return websWrite(wp, T("%s"), inet_ntoa(*((struct in_addr *)buffer)));
	}
   	else if ( !strcmp(name, T("lan-subnet"))) {
		if(!mib_get( MIB_ADSL_LAN_SUBNET,  (void *)buffer))
			return -1;
		
   		return websWrite(wp, T("%s"), inet_ntoa(*((struct in_addr *)buffer)));
	}
   	else if ( !strcmp(name, T("lan-dhcpRangeStart"))) {
		if(!mib_get( MIB_ADSL_LAN_CLIENT_START,  (void *)buffer))
			return -1;
   		return websWrite(wp, T("%u"), *(unsigned char *)buffer);
	}
   	else if ( !strcmp(name, T("lan-dhcpRangeEnd"))) {
		if(!mib_get( MIB_ADSL_LAN_CLIENT_END,  (void *)buffer))
			return -1;
   		return websWrite(wp, T("%u"), *(unsigned char *)buffer);
	}
   	else if ( !strcmp(name, T("lan-dhcpLTime"))) {
		if(!mib_get( MIB_ADSL_LAN_DHCP_LEASE,  (void *)buffer))
			return -1;
   		return websWrite(wp, T("%u"), *(unsigned int *)buffer);
	}
   	else if ( !strcmp(name, T("lan-dhcpDName"))) {
		if(!mib_get( MIB_ADSL_LAN_DHCP_DOMAIN,  (void *)buffer))
			return -1;
   		return websWrite(wp, T("%s"), buffer);
	}
	else if ( !strcmp(name, T("elan-Mac"))) {
		if(!mib_get( MIB_ELAN_MAC_ADDR,  (void *)buffer))
			return -1;
		return websWrite(wp, T("%02x%02x%02x%02x%02x%02x"), buffer[0], buffer[1],
						buffer[2], buffer[3], buffer[4], buffer[5]);
	}
	else if ( !strcmp(name, T("wlan-Mac"))) {
		if(!mib_get( MIB_WLAN_MAC_ADDR,  (void *)buffer))
			return -1;
		return websWrite(wp, T("%02x%02x%02x%02x%02x%02x"), buffer[0], buffer[1],
						buffer[2], buffer[3], buffer[4], buffer[5]);
	}
  	else if ( !strcmp(name, T("wan-dns1"))) {
		if(!mib_get( MIB_ADSL_WAN_DNS1,  (void *)buffer))
			return -1;
		if (((struct in_addr *)buffer)->s_addr != 0)
   			return websWrite(wp, T("%s"), inet_ntoa(*((struct in_addr *)buffer)));
   		else
   			return websWrite(wp, T(""));
	}
	else if ( !strcmp(name, T("wan-dhcps"))) {
		if(!mib_get( MIB_ADSL_WAN_DHCPS,  (void *)buffer))
			return -1;
		if (((struct in_addr *)buffer)->s_addr != 0)
   			return websWrite(wp, T("%s"), inet_ntoa(*((struct in_addr *)buffer)));
   		else
   			return websWrite(wp, T(""));
	}	
  	else if ( !strcmp(name, T("wan-dns2"))) {
		if(!mib_get( MIB_ADSL_WAN_DNS2,  (void *)buffer))
			return -1;
		if (((struct in_addr *)buffer)->s_addr != 0)
	   		return websWrite(wp, T("%s"), inet_ntoa(*((struct in_addr *)buffer)));
   		else
   			return websWrite(wp, T(""));
	}
  	else if ( !strcmp(name, T("wan-dns3"))) {
		if(!mib_get( MIB_ADSL_WAN_DNS3,  (void *)buffer))
			return -1;
		if (((struct in_addr *)buffer)->s_addr != 0)
	   		return websWrite(wp, T("%s"), inet_ntoa(*((struct in_addr *)buffer)));
   		else
   			return websWrite(wp, T(""));
	}
	else if ( !strcmp(name, T("dmzHost"))) {
		if(!mib_get( MIB_DMZ_IP,  (void *)buffer))
			return -1;
   		return websWrite(wp, T("%s"), inet_ntoa(*((struct in_addr *)buffer)));
	}
 	else if ( !strcmp(name, T("snmpSysDescr")) ) {
		if(!mib_get( MIB_SNMP_SYS_DESCR,  (void *)buffer))
			return -1;
   		return websWrite(wp, T("\"%s\""), buffer);
	}
 	else if ( !strcmp(name, T("snmpSysContact")) ) {
		if(!mib_get( MIB_SNMP_SYS_CONTACT,  (void *)buffer))
			return -1;
   		return websWrite(wp, T("\"%s\""), buffer);
	}
 	else if ( !strcmp(name, T("snmpSysName")) ) {
		if(!mib_get( MIB_SNMP_SYS_NAME,  (void *)buffer))
			return -1;
   		return websWrite(wp, T("\"%s\""), buffer);
	}
 	else if ( !strcmp(name, T("snmpSysLocation")) ) {
		if(!mib_get( MIB_SNMP_SYS_LOCATION,  (void *)buffer))
			return -1;
   		return websWrite(wp, T("\"%s\""), buffer);
	}
 	else if ( !strcmp(name, T("snmpSysObjectID")) ) {
		if(!mib_get( MIB_SNMP_SYS_OID,  (void *)buffer))
			return -1;
   		return websWrite(wp, T("\"%s\""), buffer);
	}
 	else if ( !strcmp(name, T("snmpTrapIpAddr")) ) {
		if(!mib_get( MIB_SNMP_TRAP_IP,  (void *)buffer))
			return -1;
   		return websWrite(wp, T("%s"), inet_ntoa(*((struct in_addr *)buffer)));
	}
 	else if ( !strcmp(name, T("snmpCommunityRO")) ) {
		if(!mib_get( MIB_SNMP_COMM_RO,  (void *)buffer))
			return -1;
   		return websWrite(wp, T("\"%s\""), buffer);
	}
 	else if ( !strcmp(name, T("snmpCommunityRW")) ) {
		if(!mib_get( MIB_SNMP_COMM_RW,  (void *)buffer))
			return -1;
   		return websWrite(wp, T("\"%s\""), buffer);
	}
 	else if ( !strcmp(name, T("wanDhcp-current")) ) {
		return websWrite(wp, T("Under Construction"));
	}
 	else if ( !strcmp(name, T("name")) ) {
		if(!mib_get( MIB_SNMP_SYS_NAME,  (void *)buffer))
			return -1;
   		return websWrite(wp, buffer);
	}
 	else if ( !strcmp(name, T("uptime")) ) {
		sysinfo(&info);
		updays = (int) info.uptime / (60*60*24);
	 	if (updays)
	 		websWrite(wp, T("%d day%s, "), updays, (updays != 1) ? "s" : "");
		upminutes = (int) info.uptime / 60;
		uphours = (upminutes / 60) % 24;
		upminutes %= 60;
		if(uphours)
			return websWrite(wp, T("%2d:%02d"), uphours, upminutes);
		else
			return websWrite(wp, T("%d min"), upminutes);
	}
 	else if ( !strcmp(name, T("date")) ) {
 		time(&tm);
		memcpy(&tm_time, localtime(&tm), sizeof(tm_time));
		strftime(buffer, 200, "%a %b %e %H:%M:%S %Z %Y", &tm_time);
   		return websWrite(wp, buffer);
	}
 	else if ( !strcmp(name, T("fwVersion")) ) {
#ifdef EMBED
 		FILE *fp;
 		char strVer[128], tmp[128], *strBld;
 		strVer[0]=0;
		fp = fopen("/etc/version", "r");
 		if (fp!=NULL) {
 		    fgets(strVer, sizeof(strVer), fp);  //main version
 		    fclose(fp);
 		    strBld = strchr(strVer, ' ');
 		    *strBld=0;
 		    fp = fopen("/proc/version", "r");  
 		    if (fp!=NULL) {  //build number
 		    	int i;
 		        fgets(tmp, sizeof(tmp), fp);
 		        fclose(fp);
 		        strBld = strchr(tmp, '#');
 		        if (strBld!=NULL) {
 		            strBld++; i=0;
 		            while (strBld[i]!=' ') i++;
 		            strBld[i]=0;
 		        } else {
  		            strBld = tmp;
  		            tmp[0]=0;
  		        };
 		    }
 		};
		return websWrite(wp, T("%s.%s"), strVer, strBld);
#else
		return websWrite(wp, T("%s.%s"), "Realtek", "1");
#endif
	}
 	else if ( !strncmp(name, T("adsl-drv-"), 9) ) {
 		getAdslDrvInfo(&name[9], buffer	, 255);
		return websWrite(wp, T("%s"), buffer);
	}
   	else if ( !strcmp(name, T("bridge-ageingTime"))) {
		if(!mib_get( MIB_BRCTL_AGEINGTIME,  (void *)&vShort))
			return -1;
		sprintf(buffer, "%u", vShort);
   		return websWrite(wp, buffer);
	}
#ifdef CONFIG_USER_IGMPPROXY
   	else if ( !strcmp(name, T("igmp-proxy-itf"))) {
		if(!mib_get( MIB_IGMP_PROXY_ITF,  (void *)buffer))
			return -1;
   		return websWrite(wp, T("%u"), *(unsigned char *)buffer);
	}
#endif
   	else if ( !strcmp(name, T("ippt-itf"))) {
		if(!mib_get( MIB_IPPT_ITF,  (void *)buffer))
			return -1;
   		return websWrite(wp, T("%u"), *(unsigned char *)buffer);
	}
   	else if ( !strcmp(name, T("ippt-lease"))) {
		if(!mib_get( MIB_IPPT_LEASE,  (void *)buffer))
			return -1;
   		return websWrite(wp, T("%u"), *(unsigned int *)buffer);
	}
#ifdef WLAN_SUPPORT
   	else if ( !strcmp(name, T("ssid"))) {
		if ( !mib_get( MIB_WLAN_SSID,  (void *)buffer) )
			return -1;

		translate_control_code(buffer);
		
		return websWrite(wp, T("%s"), buffer);
	}
   	else if ( !strcmp(name, T("channel"))) {
		if ( !mib_get( MIB_WLAN_CHAN_NUM,  (void *)buffer) )
			return -1;
   		return websWrite(wp, T("%u"), *(unsigned char *)buffer);
	}
#ifdef WLAN_WPA
        else if ( !strcmp(name, T("wep"))) {
                ENCRYPT_T encrypt;
                if ( !mib_get( MIB_WLAN_ENCRYPT,  (void *)buffer) )
                        return -1;
                encrypt = (ENCRYPT_T)*buffer;
                if (encrypt == ENCRYPT_DISABLED)
                        strcpy( buffer, T("Disabled") );
                else if (encrypt == ENCRYPT_WPA)
                        strcpy( buffer, T("WPA") );
                else {
                        WEP_T wep;
                        if ( !mib_get( MIB_WLAN_WEP,  (void *)buffer) )
                                return -1;
                        wep = (WEP_T)*buffer; 
                        if ( wep == WEP_DISABLED )
                                strcpy( buffer, T("Disabled") );
                        else if ( wep == WEP64 )
                                strcpy( buffer, T("WEP 64bits") );
                        else if ( wep == WEP128)
                                strcpy( buffer, T("WEP 128bits") );
                }
                return websWrite(wp, buffer);
        }

#else
   	else if ( !strcmp(name, T("wep"))) {
   		WEP_T wep;
		if ( !mib_get( MIB_WLAN_WEP,  (void *)buffer) )
			return -1;
		
		wep = (WEP_T)*buffer; 
		if ( wep == WEP_DISABLED )
			strcpy( buffer, T("Disabled") );
		else if ( wep == WEP64 )
			strcpy( buffer, T("64bits") );
		else if ( wep == WEP128)
			strcpy( buffer, T("128bits") );
   		return websWrite(wp, buffer);
   	}
#endif

	/* Advance setting stuffs */
	else if ( !strcmp(name, T("fragThreshold"))) {
		if ( !mib_get( MIB_WLAN_FRAG_THRESHOLD, (void *)buffer) )
			return -1;
   		return websWrite(wp, T("%u"), *(unsigned short *)buffer);
	}
	else if ( !strcmp(name, T("rtsThreshold"))) {
		if ( !mib_get( MIB_WLAN_RTS_THRESHOLD, (void *)buffer) )
			return -1;
   		return websWrite(wp, T("%u"), *(unsigned short *)buffer);
	}
	else if ( !strcmp(name, T("beaconInterval"))) {
		if ( !mib_get( MIB_WLAN_BEACON_INTERVAL, (void *)buffer) )
			return -1;
   		return websWrite(wp, T("%u"), *(unsigned short *)buffer);
	}
	else if ( !strcmp(name, T("dtimPeriod"))) {
		if ( !mib_get( MIB_WLAN_DTIM_PERIOD, (void *)buffer) )
			return -1;
   		return websWrite(wp, T("%u"), *(unsigned char *)buffer);
	}

#ifdef WLAN_WPA
#if 0	// for RADIUS server
	else if ( !strcmp(name, T("rsIp"))) {
		if ( !mib_get( MIB_WLAN_RS_IP,  (void *)buffer) )
			return -1;
		if (!memcmp(buffer, "\x0\x0\x0\x0", 4))
			return websWrite(wp, T(""));
   		return websWrite(wp, T("%s"), inet_ntoa(*((struct in_addr *)buffer)) );
	}
	else if ( !strcmp(name, T("rsPort"))) {
		if ( !mib_get( MIB_WLAN_RS_PORT, (void *)buffer) )
			return -1;
   		return websWrite(wp, T("%u"), *(unsigned short *)buffer);
	}
 	else if ( !strcmp(name, T("rsPassword"))) {
		if ( !mib_get( MIB_WLAN_RS_PASSWORD,  (void *)buffer) )
			return -1;
  		return websWrite(wp, T("%s"), buffer);
	}
#endif
 	else if ( !strcmp(name, T("pskValue"))) {
		int i;
		buffer[0]='\0';
		if ( !mib_get(MIB_WLAN_WPA_PSK,  (void *)buffer) )
			return -1;
		for (i=0; i<strlen(buffer); i++)
			buffer[i]='*';
		buffer[i]='\0';
   		return websWrite(wp, buffer);
	}
#endif

#endif // of WLAN_SUPPORT

	return websWrite(wp, T("%s"), get_fail_buf );
}
#endif

int addMenuJavaScript( webs_t wp,int nums,int maxchildrensize)
{
   #if defined( ZTE_531B_BRIDGE_SC)||defined(ZTE_GENERAL_ROUTER_SC)
websWrite(wp,T("<script >\n"));
int i=0;
websWrite(wp,T("scores = new Array(%d);\n"),nums);
for(i=0;i<nums;i++ )
websWrite(wp,T("scores[%d]='Submenu%d';\n"),i,i);
websWrite(wp,T("btns = new Array(%d);\n"),nums);
for(i=0;i<nums;i++ )
websWrite(wp,T("btns[%d]='Btn%d';\n"),i,i);
websWrite(wp,T("function initIt()\n"
" {\n divColl = document.all.tags(\"div\");\n"
" for (i=0; i<divColl.length; i++)\n "
"  {\n            whichEl = divColl[i];\n"
"  if (whichEl.className == \"Child\")\n "
" whichEl.style.display = \"none\";\n}\n}\n"));
websWrite(wp,T("function closeMenu(el)\n"
"{\n "
"for(i=0;i<%d;i++)\n"
"	{\n	 for(j=0;j<%d;j++)"
"	     {\n	 if(scores[i]!=el)\n"
"		  {\n	  id=scores[i]+\"Child\"+j.toString();\n"
"		 if(document.getElementById(id)) \n"
"		             {\n	 document.getElementById(id).style.display = \"none\";\n"
"					 whichEl = eval(scores[i] + \"Child\");\n"
"					 whichEl.style.display = \"none\";  \n          "
"                     document.getElementById(btns[i]).src =\"menu-images/menu_folder_closed.gif\";\n"
"if(document.getElementById(\"pbtn\"+i.toString()).src)\n"
"					{\n		 if(document.getElementById(\"pbtn\"+i.toString()).src.indexOf(\"corner\")>0)\n"
"					                   document.getElementById(\"pbtn\"+i.toString()).src =\"menu-images/menu_corner_plus.gif\";\n"
"								else\n"
"								       document.getElementById(\"pbtn\"+i.toString()).src =\"menu-images/menu_tee_plus.gif\";\n"
"					}\n"
"					 }\n		}\n		}\n	}\n}\n"),nums, maxchildrensize);
 websWrite(wp,T("function expandMenu(el,imgs,menuChildTableId)\n "
"{\ncloseMenu(el);\n    whichEl1 = eval(el + \"Child\");\n"  
"               for(i=0;i<%d;i++)\n"),nums);
	websWrite(wp,T(" {\n   whichEl = eval(scores[i] + \"Child\");\n"
"             if(whichEl!=whichEl1)\n "                                   
"            {\n  whichEl.style.display = \"none\";"
"             document.getElementById(btns[i]).src =\"menu-images/menu_folder_closed.gif\";\n"                                        
"document.getElementById(menuChildTableId).style.display=\"none\";\n"
"                      }\n              }        \n"));
	websWrite(wp,T("whichEl1 = eval(el + \"Child\");\n"
"               if (whichEl1.style.display == \"none\")\n "
"               { \n           "
"                   whichEl1.style.display = \"\";\n"
"document.getElementById(menuChildTableId).style.display=\"\";\n"
"                   document.getElementById(imgs).src =\"menu-images/menu_folder_open.gif\";\n"
"               }\n else        {   \n whichEl1.style.display =\"none\";     \n    "
"document.getElementById(menuChildTableId).style.display=\"none\";\n"
"                  document.getElementById(imgs).src =\"menu-images/menu_folder_closed.gif\";\n"
"              }    \n}\n"));
websWrite(wp,T("function onlyexpandMenu(el,imgs,btn,nums,isEnd)\n"
"{\ncloseMenu(el);\nfor(i=0;i<nums;i++){\n"
"whichEl1 = eval(el + \"Child\"+i.toString());\n"
"               if (whichEl1.style.display == \"none\")\n "
"               { \n           "
"                   whichEl1.style.display = \"\";\n"
"                   document.getElementById(imgs).src =\"menu-images/menu_folder_open.gif\";\n"
"if(isEnd)document.getElementById(btn).src =\"menu-images/menu_corner_minus.gif\";\n"
"else  document.getElementById(btn).src =\"menu-images/menu_tee_minus.gif\";\n"
"               }\n else        {   \n whichEl1.style.display =\"none\";     \n    "
" document.getElementById(imgs).src =\"menu-images/menu_folder_closed.gif\";\n"
"if(isEnd)document.getElementById(btn).src =\"menu-images/menu_corner_plus.gif\";\n"
"else  document.getElementById(btn).src =\"menu-images/menu_tee_plus.gif\";\n"
"}\n}\n}\n</script>\n"));
websWrite(wp,T("<style type=\"text/css\">\n"
"\n.link {\n"
"	font-size:10pt;\ncolor: #FFFFFF;\ntext-decoration: none;\n}\n"
"</style>"));
#else
	websWrite(wp,T("<script type=\"text/javascript\" src=\"/admin/mtmcode.js\">\n"
	"</script>\n"
	"\n"
	"<script type=\"text/javascript\">\n"
	"    // Morten's JavaScript Tree Menu\n"
	"    // version 2.3.2-macfriendly, dated 2002-06-10\n"
	"    // http://www.treemenu.com/\n"
	"\n"
	"    // Copyright (c) 2001-2002, Morten Wang & contributors\n"
	"    // All rights reserved.\n"
	"\n"
	"    // This software is released under the BSD License which should accompany\n"));
	websWrite(wp,T("    // it in the file \"COPYING\".  If you do not have this file you can access\n"
	"    // the license through the WWW at http://www.treemenu.com/license.txt\n"
	"\n"
	"    // Nearly all user-configurable options are set to their default values.\n"
	"    // Have a look at the section \"Setting options\" in the installation guide\n"
	"    // for description of each option and their possible values.\n"));
	websWrite(wp,T("\n"
	"MTMDefaultTarget = \"view\";\n"
	"\n"
	"/******************************************************************************\n"
	" * User-configurable list of icons.                                            *\n"
	" ******************************************************************************/\n"));
	websWrite(wp,T("\n"
	"var MTMIconList = null;\n"
	"MTMIconList = new IconList();\n"
	"MTMIconList.addIcon(new MTMIcon(\"menu_link_external.gif\", \"http://\", \"pre\"));\n"
	"MTMIconList.addIcon(new MTMIcon(\"menu_link_pdf.gif\", \".pdf\", \"post\"));\n"));
	websWrite(wp,T("\n"
	"/******************************************************************************\n"
	" * User-configurable menu.                                                     *\n"
	" ******************************************************************************/\n"));
	websWrite(wp,T("\n"
	"var menu = null;\n"
	"\n"
	"menu = new MTMenu();\n"));
#endif
}

// Kaohj
int checkWrite(int eid, webs_t wp, int argc, char_t **argv)
{
	char_t *name;
	unsigned char vChar;
	unsigned short vUShort;
	unsigned int vUInt;
	
   	if (ejArgs(argc, argv, T("%s"), &name) < 1) {
   		websError(wp, 400, T("Insufficient args\n"));
   		return -1;
   	}
	if ( !strcmp(name, T("devType")) ) {
		if ( !mib_get( MIB_DEVICE_TYPE, (void *)&vChar) )
			return -1;
#ifdef EMBED
		if (0 == vChar)
			websWrite(wp, "disableTextField(document.adsl.adslConnectionMode);");
#endif
		return 0;
	}
#ifdef CONFIG_USER_ROUTED_ROUTED
	else if ( !strcmp(name, T("rip-on-0")) ) {
		if ( !mib_get( MIB_RIP_ENABLE, (void *)&vChar) )
			return -1;
		if (0 == vChar)
			websWrite(wp, "checked");
		return 0;
	}
	else if ( !strcmp(name, T("rip-on-1")) ) {
		if ( !mib_get( MIB_RIP_ENABLE, (void *)&vChar) )
			return -1;
		if (1 == vChar)
			websWrite(wp, "checked");
		return 0;
	}
	if ( !strcmp(name, T("rip-ver")) ) {
		if ( !mib_get( MIB_RIP_VERSION, (void *)&vChar) )
			return -1;
		if (0==vChar) {
			websWrite(wp, "<option selected value=0>v1</option>\n");
			websWrite(wp, "\t<option value=1>v2</option>");
		} else {
			websWrite(wp, "<option value=0>v1</option>\n");
			websWrite(wp, "\t<option selected value=1>v2</option>\n");
		}
		return 0;
	}
#endif
if (!strcmp (name, T ("SoftwareVersion")))
    {
     #if defined(ZTE_531B_BRIDGE_SC) || defined(ZTE_GENERAL_ROUTER_SC)
       websWrite (wp, T ("<tr bgcolor=\"#DDDDDD\">\n<td width=40%%><font size=2><b>汾</b></td>\n"));
       websWrite (wp,T(" <td width=60%%><font size=2>%s</td>\n</tr>\n"),ZTESOFTWAREVERSION);    
     #endif
    

      return 0;
    }

  if (!strcmp (name, T ("DspVersion")))
    {
      char buffer[256];
      getAdslDrvInfo ("version", buffer, 64);
      #if defined(ZTE_531B_BRIDGE_SC) || defined(ZTE_GENERAL_ROUTER_SC)
      websWrite (wp,  T("<tr bgcolor=\"#EEEEEE\">\n<td width=40%%><font size=2><b>DSP 汾</b></td>"
                  "<td width=60%%><font size=2>%s</td>\n </tr>"),
                 buffer);
     #else
	 websWrite (wp,  T("<tr bgcolor=\"#DDDDDD\">\n<td width=40%%><font size=2><b>DSP Version</b></td>"
                  "<td width=60%%><font size=2>%s</td>\n </tr>"),
                 buffer);
     #endif
     return 0;
    }

  if (!strcmp (name, T ("DnsServer")))
    {
  #ifdef ZTE_GENERAL_ROUTER_SC
      websWrite (wp, T("<tr bgcolor=\"#EEEEEE\">\n<td width=20%%><font size=2><b>DNS </b></td>"));  
  
      websWrite (wp, T ("<td width=80%% colspan=\"6\"><font size=2>"));
#else
   websWrite (wp,
                 T
                 ("<tr bgcolor=\"#EEEEEE\">\n<td width=20%%><font size=2><b>DNS Servers</b></td>"));
  websWrite (wp, T (" <td width=80%%><font size=2>"));
	#endif
      getNameServer (0, wp, 1, 0);
      websWrite (wp, T ("</td>\n</tr>"));
      return 0;
    }

  if (!strcmp (name, T ("DefaultGw")))
    {
   #ifdef ZTE_GENERAL_ROUTER_SC
      websWrite (wp,
                 T
                 ("<tr bgcolor=\"#DDDDDD\">\n <td width=20%% ><font size=2><b>ȱʡ·</b></td>"));
 #else
    websWrite (wp,
                 T
                 ("<tr bgcolor=\"#DDDDDD\">\n <td width=20%% ><font size=2><b>Default Gateway</b></td>"));
   #endif
   
      websWrite (wp, T ("<td width=80%% colspan=\"6\"><font size=2>"));
      getDefaultGW (0, wp, 1, 0);
      websWrite (wp, T ("</td> </tr>"));
 
      return 0;
    }

  if (!strcmp (name, T ("getDslSnr")))
    {
#if defined(ZTE_531B_BRIDGE_SC) || defined(ZTE_GENERAL_ROUTER_SC)
      websWrite (wp,
                 T
                 ("<tr bgcolor=\"#EEEEEE\">\n <td width=40%%><font size=2><b>ԣ</b></td>"));
#else
websWrite (wp,
                 T
                 ("<tr bgcolor=\"#DDDDDD\">\n <td width=40%%><font size=2><b>Upstream SNR</b></td>"));
#endif
      websWrite (wp, T ("<td width=60%%><font size=2>"));
      char buf[256] = { 0 };
      getAdslInfo (ADSL_GET_SNR_US, buf, sizeof (buf) / sizeof (char));
      websWrite (wp, T ("%sdB"), buf);
      websWrite (wp, T ("</td>\n </tr>"));
#if defined(ZTE_531B_BRIDGE_SC) || defined(ZTE_GENERAL_ROUTER_SC)
      websWrite (wp,
                 T
                 ("<tr bgcolor=\"#DDDDDD\">\n <td width=40%%><font size=2><b>ԣ</b></td>"));
#else
  websWrite (wp,
                 T
                 ("<tr bgcolor=\"#EEEEEE\">\n <td width=40%%><font size=2><b>Downstream SNR</b></td>"));
#endif
      websWrite (wp, T (" <td width=60%%><font size=2>"));
      memset (buf, 0, sizeof (buf) / sizeof (char));
      getAdslInfo (ADSL_GET_SNR_DS, buf, sizeof (buf) / sizeof (char));
      websWrite (wp, T ("%sdB"), buf);
      websWrite (wp, T ("</td>\n </tr>"));
      return 0;
    }

  if (!strcmp (name, T ("hskCount")))
    {
#if defined(ZTE_531B_BRIDGE_SC) || defined(ZTE_GENERAL_ROUTER_SC)
      websWrite (wp,
                 T
                 ("<tr bgcolor=\"#EEEEEE\">\n <td width=40%%><font size=2><b>ͳ</b></td>"));
#else
 websWrite (wp,
                 T
                 ("<tr bgcolor=\"#DDDDDD\">\n <td width=40%%><font size=2><b>reconnection Counts</b></td>"));
#endif
      websWrite (wp, T ("<td width=60%%><font size=2>"));
      int intVal;

      if (adsl_drv_get(RLCM_GET_REHS_COUNT, &intVal, sizeof(int)))
        {
          websWrite (wp, T ("%d"), (intVal==0)?0:intVal-1);
        }

      websWrite (wp, T ("</td>\n </tr>"));
      return 0;

    }

  if (!strcmp (name, T ("dslMode")))
    {

#if defined(ZTE_531B_BRIDGE_SC) || defined(ZTE_GENERAL_ROUTER_SC)
      websWrite (wp,
                 T
                 ("<tr bgcolor=\"#EEEEEE\">\n <td width=40%%><font size=2><b>DSLģʽ</b></td>"));
   #else
    websWrite (wp,
                 T
                 ("<tr bgcolor=\"#EEEEEE\">\n <td width=40%%><font size=2><b>mode</b></td>"));
   #endif
      websWrite (wp, T ("<td width=60%%><font size=2>"));
      short mode;
      char buf[100]={0};
      mib_get( MIB_ADSL_MODE, (void *)&mode);

      if (mode & ADSL_MODE_GLITE)
        strcat(buf,"G.Lite ");

      if (mode & ADSL_MODE_T1413)
        strcat(buf,"T1.413 ");

      if (mode & ADSL_MODE_GDMT)
        strcat(buf,"G.Dmt ");

      if (mode & ADSL_MODE_ADSL2)
        strcat(buf,"ADSL2 ");

      if (mode & ADSL_MODE_ADSL2P)
        strcat(buf,"ADSL2+ ");

      websWrite(wp,T("%s"),buf);

      websWrite (wp, T ("</td>\n </tr>"));
      return 0;

    }

  if (!strcmp (name, T ("showtime")))
    {
    #if defined(ZTE_531B_BRIDGE_SC) || defined(ZTE_GENERAL_ROUTER_SC)
      websWrite (wp,
                 T
                 ("<tr bgcolor=\"#EEEEEE\">\n <td width=40%%><font size=2><b>DSLʱ</b></td>"));
    #else
	websWrite (wp,
                 T
                 ("<tr bgcolor=\"#EEEEEE\">\n <td width=40%%><font size=2><b>Showtime:</b></td>"));
    #endif
      websWrite (wp, T ("<td width=60%%><font size=2>"));
      unsigned int intVal[3];
      char buffer[128]={0};

      if (adsl_drv_get(RLCM_GET_DSL_ORHERS, &intVal, 3*sizeof(int)))
        {
          if (intVal[0]>3600)
            {
              sprintf(buffer,"%dh:%dm",intVal[0]/3600,(intVal[0]%3600)/60);
              websWrite (wp, T ("%s"), buffer);
            }

          else if (intVal[0]>60)
            {
              websWrite(wp,T("%d mins"),intVal[0]/60);
            }

          else
            websWrite(wp,T("< 1 min"));
        }

      websWrite (wp, T ("</td>\n </tr>"));

      return 0;

    }

  if (!strcmp (name, T ("wlaninfo")))
    {
#ifdef WLAN_SUPPORT
      const char *bgColor[]=    {"#EEEEEE","#DDDDDD"     };
      int col_nums=0;
      const char *wlan_band[] =     {          0, "802.11b", "802.11g", "802.11 b+g", 0      };
      unsigned char vChar;
    #if defined(ZTE_531B_BRIDGE_SC) || defined(ZTE_GENERAL_ROUTER_SC)
      websWrite (wp,     T              ("<tr>\n <td width=100%% colspan=\"2\" bgcolor=\"#008000\"><font color=\"#FFFFFF\" size=2><b></b></font></td>"));
	  websWrite (wp,   T ("<tr bgcolor=%s> <td width=40%%><font size=2><b>״̬</b></td>"),bgColor[col_nums++%2]);
	#else
	 websWrite (wp,     T              ("<tr>\n <td width=100%% colspan=\"2\" bgcolor=\"#008000\"><font color=\"#FFFFFF\" size=2><b>Wireless Configuration</b></font></td>"));
	  websWrite (wp,
                 T
                 ("<tr bgcolor=%s> <td width=40%%><font size=2><b>Wireless</b></td>"),bgColor[col_nums++%2]);
		
	#endif

      //ramen--wireless enable??
    

      websWrite (wp, T ("<td width=60%%><font size=2>"));

      mib_get (MIB_WLAN_DISABLED, (void *) &vChar);

      if (!vChar)
        websWrite (wp, T (INFO_ENABLED));
      else
        {
          websWrite (wp, T (INFO_DISABLED));
          websWrite (wp, T ("\n</td>\n</tr>\n"));
          goto wlend;
        }

      websWrite (wp, T ("\n</td>\n</tr>\n"));

      //ramen--get the wireless band
      #if defined(ZTE_531B_BRIDGE_SC) || defined(ZTE_GENERAL_ROUTER_SC)
      websWrite (wp,
                 T
                 ("<tr bgcolor=%s> <td width=40%%><font size=2><b>Ƶ</b></td>"),bgColor[col_nums++%2]);
     #else
	   websWrite (wp,
                 T
                 ("<tr bgcolor=%s> <td width=40%%><font size=2><b>Ƶ</b></td>"),bgColor[col_nums++%2]);
     #endif
      websWrite (wp, T ("<td width=60%%><font size=2>"));
      mib_get (MIB_WLAN_BAND, (void *) &vChar);
      websWrite (wp, "%s", wlan_band[(BAND_TYPE_T) vChar]);
      websWrite (wp, T ("\n</td>\n</tr>\n"));
      //ramen--get wireless mode

      if (mib_get (MIB_WLAN_MODE, (void *) &vChar))
        {
        #if defined(ZTE_531B_BRIDGE_SC) || defined(ZTE_GENERAL_ROUTER_SC)
          websWrite (wp,  T ("<tr bgcolor=%s> <td width=40%%><font size=2><b>ģʽ</b></td>"),bgColor[col_nums++%2]);
	#else
	websWrite (wp,  T ("<tr bgcolor=%s> <td width=40%%><font size=2><b>Mode</b></td>"),bgColor[col_nums++%2]);
	#endif
          websWrite (wp, T ("<td width=60%%><font size=2>"));

          if (vChar == AP_MODE)
            websWrite (wp, T ("AP"));
          else if (vChar == CLIENT_MODE)
            websWrite (wp, T ("Client"));

          websWrite (wp, T ("\n</td>\n</tr>\n"));
        }

      //ramen---broadcast SSID
      #if defined(ZTE_531B_BRIDGE_SC) || defined(ZTE_GENERAL_ROUTER_SC)
      websWrite (wp,  T ("<tr bgcolor=%s> <td width=40%%><font size=2><b>㲥  SSID</b></td>"),bgColor[col_nums++%2]);
	  #else
	    websWrite (wp,  T ("<tr bgcolor=%s> <td width=40%%><font size=2><b>Broadcast  SSID</b></td>"),bgColor[col_nums++%2]);
	  #endif
      websWrite (wp, T ("<td width=60%%><font size=2>"));
        mib_get (MIB_WLAN_HIDDEN_SSID, (void *) &vChar);	
          websWrite (wp, T((vChar!=0)?INFO_DISABLED:INFO_ENABLED));
          websWrite (wp, T ("\n</td>\n</tr>\n"));
          //ramen--get authentication type
     /* if (mib_get (MIB_WLAN_AUTH_TYPE, (void *) &vChar))
        {
          char *authtype[] = { "Open System", "Shared Key", "Auto" };
	#ifdef ZTE_531B_BRIDGE_SC
          websWrite (wp,
                     T
                     ("<tr bgcolor=%s> <td width=40%%><font size=2><b>֤</b></td>"),bgColor[col_nums++%2]);
	#else
	 websWrite (wp,
                     T
                     ("<tr bgcolor=%s> <td width=40%%><font size=2><b>Authentication Type</b></td>"),bgColor[col_nums++%2]);
	#endif
          websWrite (wp, T ("<td width=60%%><font size=2>"));
          websWrite (wp, T ("%s"), authtype[vChar]);
          websWrite (wp, T ("\n</td>\n</tr>\n"));
        }*/

    
wlend:
      websWrite (wp, T ("</tr>"));
#endif
      return 0;

    }
  
 //<%checkWrite("wlanencryptioninfo");%>
 
 if(!strcmp(name,T("wlanencryptioninfo")))
  {
  #ifdef WLAN_SUPPORT
  #if defined(ZTE_531B_BRIDGE_SC) || defined(ZTE_GENERAL_ROUTER_SC)
   char *encryptMode[]={"None","WEP","WPA (TKIP)","","WPA2(AES)","","WPA2 Mixed"};
   char *wepEncryptLength[]={"Disabled","64bits","128bits"};
   char *encryptFormat[]={"Ascii","Hex"};
   char *wpaAuthMode[]={"ҵ(RADIUS)","(ԤԿ) "};
   char *preshareKeyMode[]={"Passphrase","Hex"};
   char *authtype[] = { "Open System", "Shared Key", "Auto" };
   const char *bgColor[]=    {"#EEEEEE","#DDDDDD"    };
   char tmpBuf[100];
   unsigned char vChar;
   unsigned char bgcolorIndex=0;
  #ifdef WLAN_MBSSID
	MIB_CE_MBSSIB_T Entry;
	MIB_CE_MBSSIB_WEP_T wepEntry;
	char *ssidMode[]={"root","vap0","vap1","vap2","vap3"};
	int i;
    #endif
#ifdef WLAN_MBSSID
for(i = 0; i < 5; i++)
{
if (!mib_chain_get(MIB_MBSSIB_TBL, i, (void *)&Entry)) {  		
  		strcpy(tmpBuf, T(strGetMBSSIBTBLErr));				
		goto read_mib_chain_error;		
	}
   websWrite(wp,T("<tr><td width=100%% colspan=\"2\" bgcolor=\"#008000\"><font size=2 color=\"#FFFFFF\"><b>%s</b></font></td></tr>\n"),ssidMode[Entry.idx]);
   if(i !=0 && Entry.wlanDisabled) 
   	{
   	 websWrite (wp, T("<tr bgcolor=%s> <td width=40%%><font size=2><b>״̬</b></td>\n"),bgColor[bgcolorIndex++%2]);
	  websWrite (wp, T ("<td width=60%%><font size=2>%s</font></td></tr>\n"),INFO_DISABLED);
   	continue;
   	}
   else
   	{
   	  websWrite (wp, T("<tr bgcolor=%s> <td width=40%%><font size=2><b>״̬</b></td>\n"),bgColor[bgcolorIndex++%2]);
	  websWrite (wp, T ("<td width=60%%><font size=2>%s</font></td></tr>\n"),INFO_ENABLED);
   	}
   if(i==0)//root mode
   	{
   	  //ramen--get wireless SSID
          char rootssid[100] = { 0 };
          if (mib_get (MIB_WLAN_SSID, (void *) rootssid))
            {
              websWrite (wp, T("<tr bgcolor=%s> <td width=40%%><font size=2><b>SSID</b></td>\n"),bgColor[bgcolorIndex++%2]);
              websWrite (wp, T ("<td width=60%%><font size=2>\n"));
              websWrite (wp, T ("%s"), rootssid);
              websWrite (wp, T ("\n</td>\n</tr>\n"));
	  }  	 
   	}
  else{
  	 websWrite(wp,T("<tr bgcolor=%s><td width=40%%><font size=2><b>SSID</b></font></td>\n"),bgColor[bgcolorIndex++%2]);
          websWrite(wp,T("<td width=60%%><font size=2>%s</font></td></tr>\n"),Entry.ssid);
  	}
   websWrite(wp,T("<tr bgcolor=%s><td width=40%%><font size=2><b>֤ʽ</b></font></td>\n"),bgColor[bgcolorIndex++%2]);
   if(i == 0) //root
   	{
   	mib_get (MIB_WLAN_AUTH_TYPE, (void *) &vChar);
	websWrite(wp,T("<td width=60%%><font size=2>%s</font></td></tr>\n"),authtype[vChar]);
   	}
   else
   websWrite(wp,T("<td width=60%%><font size=2>%s</font></td></tr>\n"),authtype[Entry.authType]);
   websWrite(wp,T("<tr bgcolor=%s><td width=40%%><font size=2><b>ܷʽ</b></font></td>\n"),bgColor[bgcolorIndex++%2]);
   websWrite(wp,T("<td width=60%%><font size=2>%s</font></td></tr>"),encryptMode[Entry.encrypt] );

   if(Entry.encrypt==ENCRYPT_WEP)
   	{
		   	    websWrite(wp,T("<tr bgcolor=%s><td width=40%%><font size=2><b>Կ</b></font></td>\n"),bgColor[bgcolorIndex++%2]);
			    websWrite(wp,T("<td width=60%%><font size=2>%s</font></td></tr>\n"),wepEncryptLength[Entry.wep]);
				if (!mib_chain_get(MIB_MBSSIB_WEP_TBL, i, (void *)&wepEntry)) {  		
		  		strcpy(tmpBuf, T(strGetMBSSIBWEPTBLErr));				
				goto read_mib_chain_error;		
			}
			    websWrite(wp,T("<tr bgcolor=%s><td width=40%%><font size=2><b>Կʽ</b></font></td>\n"),bgColor[bgcolorIndex++%2]);
			    websWrite(wp,T("<td width=60%%><font size=2>%s</font></td></tr>\n"),encryptFormat[wepEntry.wepKeyType]);
			    websWrite(wp,T("<tr bgcolor=%s><td width=40%%><font size=2><b>Կ%d</b></font></td>\n"),bgColor[bgcolorIndex++%2],wepEntry.wepDefaultKey+1);
			    if(Entry.wep==1)//64bits
			       {
			          switch(wepEntry.wepDefaultKey+1)
			          {
			             case 1:
				      if(wepEntry.wepKeyType ==0)//ascii	 	
			                   websWrite(wp,T("<td width=60%%><font size=2>%c%c%c%c%c</font></td></tr>\n"),
			 	wepEntry.wep64Key1[0],wepEntry.wep64Key1[1],wepEntry.wep64Key1[2],wepEntry.wep64Key1[3],
			 	wepEntry.wep64Key1[4]);
				      else
					  websWrite(wp,T("<td width=60%%><font size=2>%02x%02x%02x%02x%02x</font></td></tr>\n"),
			 	wepEntry.wep64Key1[0],wepEntry.wep64Key1[1],wepEntry.wep64Key1[2],wepEntry.wep64Key1[3],
			 	wepEntry.wep64Key1[4]);
			                break;
			             case 2:
			                   if(wepEntry.wepKeyType ==0)//ascii	 	
			                   websWrite(wp,T("<td width=60%%><font size=2>%c%c%c%c%c</font></td></tr>\n"),
			 	wepEntry.wep64Key2[0],wepEntry.wep64Key2[1],wepEntry.wep64Key2[2],wepEntry.wep64Key2[3],
			 	wepEntry.wep64Key2[4]);
				      else
					  websWrite(wp,T("<td width=60%%><font size=2>%02x%02x%02x%02x%02x</font></td></tr>\n"),
			 	wepEntry.wep64Key2[0],wepEntry.wep64Key2[1],wepEntry.wep64Key2[2],wepEntry.wep64Key2[3],
			 	wepEntry.wep64Key2[4]);
			                break;
			             case 3:
			                  if(wepEntry.wepKeyType ==0)//ascii	 	
			                   websWrite(wp,T("<td width=60%%><font size=2>%c%c%c%c%c</font></td></tr>\n"),
			 	wepEntry.wep64Key3[0],wepEntry.wep64Key3[1],wepEntry.wep64Key3[2],wepEntry.wep64Key3[3],
			 	wepEntry.wep64Key3[4]);
				      else
					  websWrite(wp,T("<td width=60%%><font size=2>%02x%02x%02x%02x%02x</font></td></tr>\n"),
			 	wepEntry.wep64Key3[0],wepEntry.wep64Key3[1],wepEntry.wep64Key3[2],wepEntry.wep64Key3[3],
			 	wepEntry.wep64Key3[4]);
			                break;
			             case 4:
			                  if(wepEntry.wepKeyType ==0)//ascii	 	
			                   websWrite(wp,T("<td width=60%%><font size=2>%c%c%c%c%c</font></td></tr>\n"),
			 	wepEntry.wep64Key4[0],wepEntry.wep64Key4[1],wepEntry.wep64Key4[2],wepEntry.wep64Key4[3],
			 	wepEntry.wep64Key4[4]);
				      else
					  websWrite(wp,T("<td width=60%%><font size=2>%02x%02x%02x%02x%02x</font></td></tr>\n"),
			 	wepEntry.wep64Key4[0],wepEntry.wep64Key4[1],wepEntry.wep64Key4[2],wepEntry.wep64Key4[3],
			 	wepEntry.wep64Key4[4]);
			                break;
			             default:
			                websWrite(wp,T("<td width=60%%><font size=2>&nbsp</font></td></tr>\n"));
			                break;
			             
		            }
		         }
		      else if(Entry.wep == 2)//128bits
		           {
		           switch(wepEntry.wepDefaultKey+1)
			          {
			             case 1:
				   if(wepEntry.wepKeyType ==0)//ascii	 
			                websWrite(wp,T("<td width=60%%><font size=2>%c%c%c%c%c%c%c%c%c%c%c%c%c</font></td></tr>\n"),
			 	wepEntry.wep128Key1[0],wepEntry.wep128Key1[1],wepEntry.wep128Key1[2],wepEntry.wep128Key1[3],
			 	wepEntry.wep128Key1[4],wepEntry.wep128Key1[5],wepEntry.wep128Key1[6],wepEntry.wep128Key1[7],
			 	wepEntry.wep128Key1[8],wepEntry.wep128Key1[9],wepEntry.wep128Key1[10],wepEntry.wep128Key1[11],
			 	wepEntry.wep128Key1[12]);
				   else
				        websWrite(wp,T("<td width=60%%><font size=2>%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x</font></td></tr>\n"),
			 	wepEntry.wep128Key1[0],wepEntry.wep128Key1[1],wepEntry.wep128Key1[2],wepEntry.wep128Key1[3],
			 	wepEntry.wep128Key1[4],wepEntry.wep128Key1[5],wepEntry.wep128Key1[6],wepEntry.wep128Key1[7],
			 	wepEntry.wep128Key1[8],wepEntry.wep128Key1[9],wepEntry.wep128Key1[10],wepEntry.wep128Key1[11],
			 	wepEntry.wep128Key1[12]);
			                break;
			             case 2:
			                 if(wepEntry.wepKeyType ==0)//ascii	 
			                websWrite(wp,T("<td width=60%%><font size=2>%c%c%c%c%c%c%c%c%c%c%c%c%c</font></td></tr>\n"),
			 	wepEntry.wep128Key2[0],wepEntry.wep128Key2[1],wepEntry.wep128Key2[2],wepEntry.wep128Key2[3],
			 	wepEntry.wep128Key2[4],wepEntry.wep128Key2[5],wepEntry.wep128Key2[6],wepEntry.wep128Key2[7],
			 	wepEntry.wep128Key2[8],wepEntry.wep128Key2[9],wepEntry.wep128Key2[10],wepEntry.wep128Key2[11],
			 	wepEntry.wep128Key2[12]);
				   else
				        websWrite(wp,T("<td width=60%%><font size=2>%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x</font></td></tr>\n"),
			 	wepEntry.wep128Key2[0],wepEntry.wep128Key2[1],wepEntry.wep128Key2[2],wepEntry.wep128Key2[3],
			 	wepEntry.wep128Key2[4],wepEntry.wep128Key2[5],wepEntry.wep128Key2[6],wepEntry.wep128Key2[7],
			 	wepEntry.wep128Key2[8],wepEntry.wep128Key2[9],wepEntry.wep128Key2[10],wepEntry.wep128Key2[11],
			 	wepEntry.wep128Key2[12]);
			                break;
			             case 3:
			               if(wepEntry.wepKeyType ==0)//ascii	 
			                websWrite(wp,T("<td width=60%%><font size=2>%c%c%c%c%c%c%c%c%c%c%c%c%c</font></td></tr>\n"),
			 	wepEntry.wep128Key3[0],wepEntry.wep128Key3[1],wepEntry.wep128Key3[2],wepEntry.wep128Key3[3],
			 	wepEntry.wep128Key3[4],wepEntry.wep128Key3[5],wepEntry.wep128Key3[6],wepEntry.wep128Key3[7],
			 	wepEntry.wep128Key3[8],wepEntry.wep128Key3[9],wepEntry.wep128Key3[10],wepEntry.wep128Key3[11],
			 	wepEntry.wep128Key3[12]);
				   else
				        websWrite(wp,T("<td width=60%%><font size=2>%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x</font></td></tr>\n"),
			 	wepEntry.wep128Key3[0],wepEntry.wep128Key3[1],wepEntry.wep128Key3[2],wepEntry.wep128Key3[3],
			 	wepEntry.wep128Key3[4],wepEntry.wep128Key3[5],wepEntry.wep128Key3[6],wepEntry.wep128Key3[7],
			 	wepEntry.wep128Key3[8],wepEntry.wep128Key3[9],wepEntry.wep128Key3[10],wepEntry.wep128Key3[11],
			 	wepEntry.wep128Key3[12]);
			                break;
			             case 4:
			                if(wepEntry.wepKeyType ==0)//ascii	 
			                websWrite(wp,T("<td width=60%%><font size=2>%c%c%c%c%c%c%c%c%c%c%c%c%c</font></td></tr>\n"),
			 	wepEntry.wep128Key4[0],wepEntry.wep128Key4[1],wepEntry.wep128Key4[2],wepEntry.wep128Key4[3],
			 	wepEntry.wep128Key4[4],wepEntry.wep128Key4[5],wepEntry.wep128Key4[6],wepEntry.wep128Key4[7],
			 	wepEntry.wep128Key4[8],wepEntry.wep128Key4[9],wepEntry.wep128Key4[10],wepEntry.wep128Key4[11],
			 	wepEntry.wep128Key4[12]);
				   else
				        websWrite(wp,T("<td width=60%%><font size=2>%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x</font></td></tr>\n"),
			 	wepEntry.wep128Key4[0],wepEntry.wep128Key4[1],wepEntry.wep128Key4[2],wepEntry.wep128Key4[3],
			 	wepEntry.wep128Key4[4],wepEntry.wep128Key4[5],wepEntry.wep128Key4[6],wepEntry.wep128Key4[7],
			 	wepEntry.wep128Key4[8],wepEntry.wep128Key4[9],wepEntry.wep128Key4[10],wepEntry.wep128Key4[11],
			 	wepEntry.wep128Key4[12]);
			                break;
			             default:
			                websWrite(wp,T("<td width=60%%><font size=2>&nbsp</font></td></tr>\n"));
			                break;
			             
		            }
		      }
		//enable 802.1x??
		#ifdef WLAN_1x
		if(Entry.enable1X)
		{
		websWrite(wp,T("<tr bgcolor=%s><td width=40%%><font size=2><b>802.1x֤ʽ</b></font></td>\n"),bgColor[bgcolorIndex++%2]);
		 websWrite(wp,T("<td width=60%%><font size=2>%s</font></td></tr>\n"),INFO_ENABLED);
		 
		     websWrite(wp,T("<tr bgcolor=%s><td width=40%%><font size=2><b>Radius Ip </b></font></td>\n"),bgColor[bgcolorIndex++%2]);
		 websWrite(wp,T("<td width=60%%><font size=2>%d.%d.%d.%d</font></td></tr>\n"),Entry.rsIpAddr[0],Entry.rsIpAddr[1],Entry.rsIpAddr[2],Entry.rsIpAddr[3]);

		  websWrite(wp,T("<tr bgcolor=%s><td width=40%%><font size=2><b>Radius Ip ˿ں</b></font></td>\n"),bgColor[bgcolorIndex++%2]);
		 websWrite(wp,T("<td width=60%%><font size=2>%d</font></td></tr>\n"),Entry.rsPort);

		  websWrite(wp,T("<tr bgcolor=%s><td width=40%%><font size=2><b>Radius </b></font></td>\n"),bgColor[bgcolorIndex++%2]);
		 websWrite(wp,T("<td width=60%%><font size=2>%s</font></td></tr>\n"),Entry.rsPassword);

		}else
		{
		 websWrite(wp,T("<tr bgcolor=%s><td width=40%%><font size=2><b>802.1x֤ʽ</b></font></td>\n"),bgColor[bgcolorIndex++%2]);
		 websWrite(wp,T("<td width=60%%><font size=2>Disabled</font></td></tr>\n"));
		}	    
	     #endif
   	}
   else//other enctyption mode except WEP
   	{
   	  if(Entry.encrypt==ENCRYPT_DISABLED) continue;
	  websWrite(wp,T("<tr bgcolor=%s><td width=40%%><font size=2><b>WPA֤ģʽ</b></font></td>\n"),bgColor[bgcolorIndex++%2]);
	  websWrite(wp,T("<td width=60%%><font size=2>%s</font></td></tr>\n"),wpaAuthMode[Entry.wpaAuth-1]);
	  if(Entry.wpaAuth==1)//radius server
	    {
	    websWrite(wp,T("<tr bgcolor=%s><td width=40%%><font size=2><b>Radius Ip</b></font></td>\n"),bgColor[bgcolorIndex++%2]);
		 websWrite(wp,T("<td width=60%%><font size=2>%d.%d.%d.%d</font></td></tr>\n"),Entry.rsIpAddr[0],Entry.rsIpAddr[1],Entry.rsIpAddr[2],Entry.rsIpAddr[3]);

		  websWrite(wp,T("<tr bgcolor=%s><td width=40%%><font size=2><b>Radius Ip ˿ں</b></font></td>\n"),bgColor[bgcolorIndex++%2]);
		 websWrite(wp,T("<td width=60%%><font size=2>%d</font></td></tr>\n"),Entry.rsPort);

		  websWrite(wp,T("<tr bgcolor=%s><td width=40%%><font size=2><b>Radius </b></font></td>\n"),bgColor[bgcolorIndex++%2]);
		 websWrite(wp,T("<td width=60%%><font size=2>%s</font></td></tr>\n"),Entry.rsPassword);
	  }
	  else {//presharekey
	             websWrite(wp,T("<tr bgcolor=%s><td width=40%%><font size=2><b>ԤԿʽ</b></font></td>\n"),bgColor[bgcolorIndex++%2]);
		    websWrite(wp,T("<td  width=60%%><font size=2>%s</font></td></tr>\n"),preshareKeyMode[Entry.wpaPSKFormat]);
		    websWrite(wp,T("<tr bgcolor=%s><td width=40%%><font size=2><b>ԤԿ</b></font></td>\n"),bgColor[bgcolorIndex++%2]);
		    websWrite(wp,T("<td width=60%%><font size=2>%s</font></td></tr>\n"),Entry.wpaPSK);
	  	}
   	}
	
}
 #endif
 read_mib_chain_error:
 #endif
 #endif
  	return 0;
  }

  if (!strcmp(name,T("wlanClient")))
    {
#ifdef WLAN_SUPPORT
#if defined(ZTE_531B_BRIDGE_SC) || defined(ZTE_GENERAL_ROUTER_SC)
      websWrite(wp,T("<P><table border=0 width=\"550\" bgcolor=\"#999999\">"
                     "<tr> <td width=100%% colspan=\"6\" bgcolor=\"#008000\"><font color=\"#FFFFFF\" size=2><b>Wireless ͻб</b></font></td> </tr>"
                     "<tr bgcolor=#7f7f7f><td width=\"25%%\"><font size=2><b>MAC ַ</b></td>"
                     "<td width=\"15%%\"><font size=2><b></b></td>"
                     "<td width=\"15%%\"><font size=2><b>հ</b></td>"
                     "<td width=\"15%%\"><font size=2><b> (Mbps)</b></td>"
                     "<td width=\"15%%\"><font size=2><b>Power Saving</b></td>"
                     "<td width=\"15%%\"><font size=2><b>ʱ(s)</b></td></tr>"));
#else
 websWrite(wp,T("<P><table border=0 width=\"550\">"
                     "<tr> <td width=100%% colspan=\"6\" bgcolor=\"#008000\"><font color=\"#FFFFFF\" size=2><b>Wireless Client List</b></font></td> </tr>"
                     "<tr bgcolor=#7f7f7f><td width=\"25%%\"><font size=2><b>MAC Address</b></td>"
                     "<td width=\"15%%\"><font size=2><b>Tx Packet</b></td>"
                     "<td width=\"15%%\"><font size=2><b>Rx Packet</b></td>"
                     "<td width=\"15%%\"><font size=2><b>Tx Rate (Mbps)</b></td>"
                     "<td width=\"15%%\"><font size=2><b>Power Saving</b></td>"
                     "<td width=\"15%%\"><font size=2><b>Expired Time (s)</b></td></tr>"));
#endif
      wirelessClientList(0,wp,1,0);
      websWrite(wp,T("</table>"));
#endif
      return 0;
    }

  if (!strcmp(name,"wlanAccessControl"))
    {
    #ifdef WLAN_SUPPORT
     #ifdef WLAN_ACL
      unsigned char vChar;
      char *acType[]={"","бʽ","ֹбʽ"};
      mib_get( MIB_WLAN_AC_ENABLED, (void *)&vChar);
  #if defined(ZTE_531B_BRIDGE_SC) || defined(ZTE_GENERAL_ROUTER_SC)
      websWrite(wp,T("<P><table border=0 width=550 bgcolor=\"#999999\">"
                     "<tr> <td width=100%%  bgcolor=\"#008000\" colspan=\"2\"><font color=\"#FFFFFF\" size=2><b>ǰ·б</b></font></td> </tr>"));
      websWrite(wp,T("<tr bgcolor=\"#EEEEEE\"><td><font size=2><b>бģʽ</b></font></td><td align=left><font size=2>%s</font></td></tr>"),acType[vChar]);
  #else
   websWrite(wp,T("<P><table border=0 width=550>"
                     "<tr> <td width=100%%  bgcolor=\"#008000\" colspan=\"2\"><font color=\"#FFFFFF\" size=2><b>Current Access Control List:</b></font></td> </tr>"));
      websWrite(wp,T("<tr bgcolor=\"#EEEEEE\"><td><font size=2><b>Mode</b></font></td><td align=left><font size=2>%s</font></td></tr>"),acType[vChar]);

  #endif

      if (vChar)
        {
          wlShowAcList(0,wp,1,0);
        }

      else
      {}

      websWrite(wp,T(" </table>"));
  #endif
  #endif
      return 0;
    }

  if (!strcmp (name, T ("showpvctable0")))
    {
#ifdef ZTE_531B_BRIDGE_SC
      websWrite (wp,
                 T
                 ("<table border=\"0\" width=700><tr><font size=2><b>ǰATM VC б</b></font></tr>"));
#else
   websWrite (wp,
                 T
                 ("<table border=\"0\" width=700><tr><font size=2><b>Current ATM VC Table:</b></font></tr>"));
#endif
      atmVcList2 (0, wp, 1, 0);
      websWrite (wp, T ("</table>"));
      return 0;
    }

  if (!strcmp (name, T ("showpvctable1")))
    {
#ifdef ZTE_531B_BRIDGE_SC
 websWrite (wp,
                 T
                 ("<table border=\"0\" width=700><tr><font size=2><b>ǰATM VC б</b></font></tr>"));
      atmVcList2 (0, wp, 1, 0);
      websWrite (wp, T ("</table>"));
#else
      websWrite (wp,
                 T
                 ("<table border=\"0\" width=700><tr><font size=2><b>Current ATM VC Table:</b></font></tr>"));
      atmVcList2 (0, wp, 1, 0);
      websWrite (wp, T ("</table>"));
#endif
      return 0;
    }
	if ( !strcmp(name, T("dhcpMode")) ) {
 		if ( !mib_get( MIB_DHCP_MODE, (void *)&vChar) )
			return -1;
		if (vChar == 0) {
			websWrite(wp, "<option selected value=\"0\">None</option>\n" );
			websWrite(wp, "<option value=\"1\">DHCP Relay</option>\n" );
			websWrite(wp, "<option value=\"2\">DHCP Server</option>\n" );
		}
		if (vChar == 1) {
			websWrite(wp, "<option selected value=\"1\">DHCP Relay</option>\n" );
			websWrite(wp, "<option value=\"0\">None</option>\n" );
			websWrite(wp, "<option value=\"2\">DHCP Server</option>\n" );
		}
		if (vChar == 2) {
			websWrite(wp, "<option selected value=\"2\">DHCP Server</option>\n" );
			websWrite(wp, "<option value=\"0\">None</option>\n" );
			websWrite(wp, "<option value=\"1\">DHCP Relay</option>\n" );
		}
		return 0;
	}
#ifdef WLAN_SUPPORT
	if ( !strcmp(name, T("wlmode")) ) {
 		if ( !mib_get( MIB_WLAN_MODE, (void *)&vChar) )
			return -1;
		if (vChar == AP_MODE) {
			websWrite(wp, "<option selected value=\"0\">AP</option>\n" );

#if defined(ZTE_531B_BRIDGE_SC) || defined(ZTE_GENERAL_ROUTER_SC)
//xl_yue: 531B bridge only need AP
#ifdef ZTE_GENERAL_ROUTER_SC
#ifdef WLAN_CLIENT
			websWrite(wp, "<option value=\"1\">Client</option>\n" );
#endif
#ifdef WLAN_WDS
			websWrite(wp, "<option value=\"3\">AP+WDS</option>\n" );
#endif
#endif

#else

#ifdef WLAN_CLIENT
			websWrite(wp, "<option value=\"1\">Client</option>\n" );
#endif
#ifdef WLAN_WDS
			websWrite(wp, "<option value=\"3\">AP+WDS</option>\n" );
#endif

#endif
		}
#ifdef WLAN_CLIENT
		if (vChar == CLIENT_MODE) {
			websWrite(wp, "<option selected value=\"1\">Clinet</option>\n" );
			websWrite(wp, "<option value=\"0\">AP</option>\n" );
#ifdef WLAN_WDS

			websWrite(wp, "<option value=\"3\">AP+WDS</option>\n" );
#endif
		}
#endif
#ifdef WLAN_WDS
		if (vChar == AP_WDS_MODE) {
			websWrite(wp, "<option selected value=\"3\">AP+WDS</option>\n" );
			websWrite(wp, "<option value=\"0\">AP</option>\n" );
#ifdef WLAN_CLIENT
			websWrite(wp, "<option value=\"1\">Client</option>\n" );
#endif
		}
#endif
		return 0;
	}	
#ifdef WLAN_WDS
	if ( !strcmp(name, T("wlanWdsEnabled")) ) {
		if ( !mib_get( MIB_WLAN_WDS_ENABLED, (void *)&vChar) )
			return -1;
		if (1 == vChar)
			websWrite(wp, "checked");
		return 0;
	}

#endif
	if ( !strcmp(name, T("wlband")) ) {
#ifdef WIFI_TEST
		websWrite(wp, "<option value=3>WiFi-G</option>\n" );
		websWrite(wp, "<option value=4>WiFi-BG</option>\n" );
#endif
	}
	// Added by Mason Yu for TxPower	
	if ( !strcmp(name, T("txpower")) ) {
//modified by xl_yue
#if defined(ZTE_531B_BRIDGE_SC) || defined(ZTE_GENERAL_ROUTER_SC)
			websWrite(wp, "<option value=\"0\">15 mW</option>\n" );
			websWrite(wp, "<option value=\"1\">30 mW</option>\n" );
			websWrite(wp, "<option value=\"2\">60 mW</option>\n" );
			websWrite(wp, "<option value=\"3\">80 mW</option>\n" );
			websWrite(wp, "<option value=\"4\">90 mW</option>\n" );
			websWrite(wp, "<option value=\"5\">100 mW</option>\n" );
#else
 		if ( !mib_get( MIB_TX_POWER, (void *)&vChar) )
			return -1;

		if (vChar == 0) {
			websWrite(wp, "<option selected value=\"0\">15 mW</option>\n" );
			websWrite(wp, "<option value=\"1\">30 mW</option>\n" );
			websWrite(wp, "<option value=\"2\">60 mW</option>\n" );
		}
		if (vChar == 1) {
			websWrite(wp, "<option selected value=\"1\">30 mW</option>\n" );
			websWrite(wp, "<option value=\"0\">15 mW</option>\n" );
			websWrite(wp, "<option value=\"2\">60 mW</option>\n" );
		}
		if (vChar == 2) {
			websWrite(wp, "<option selected value=\"2\">60 mW</option>\n" );
			websWrite(wp, "<option value=\"0\">15 mW</option>\n" );
			websWrite(wp, "<option value=\"1\">30 mW</option>\n" );
		}
#endif
		return 0;
	}	
//added by xl_yue
#if defined(ZTE_531B_BRIDGE_SC) || defined(ZTE_GENERAL_ROUTER_SC)
#ifdef WLAN_MBSSID
	if ( !strcmp(name, T("wpaVAPSSID")) ) {
		MIB_CE_MBSSIB_T Entry;
		char strbuf2[20];
		char vapName[10];
		int i;
		
		for(i=0;i<4;i++){
			if (!mib_chain_get(MIB_MBSSIB_TBL, i+1, (void *)&Entry)) {  		
				return 0;		
			}
			
			if ( ((struct in_addr *)Entry.rsIpAddr)->s_addr == INADDR_NONE ) {
				sprintf(strbuf2, "%s", "");				
			} else {	
				sprintf(strbuf2, "%s", inet_ntoa(*((struct in_addr *)Entry.rsIpAddr)));
			}

			sprintf(vapName,"vap%d",i);

			if(!Entry.wlanDisabled)
				websWrite(wp, "<input type=\"radio\" name=\"wpaSSID\" value=\"vap%d\" onClick=\"postSecurity(%d, %d, %d, %d, %d, '%s', %d, '%s', '%s', '%s')\">VAP%d&nbsp;\n",
							i,Entry.encrypt, Entry.enable1X, Entry.wep, Entry.wpaAuth, Entry.wpaPSKFormat, Entry.wpaPSK, Entry.rsPort, strbuf2, Entry.rsPassword,vapName, i );
		}
	}
	if ( !strcmp(name, T("wepVAPSSID")) ) {
		MIB_CE_MBSSIB_T Entry;
		MIB_CE_MBSSIB_WEP_T EntryWEP;
		int i;
		for(i=0;i<4;i++){
			if (!mib_chain_get(MIB_MBSSIB_TBL, i+1, (void *)& Entry)) {  		
				return 0;		
			}
			if (!mib_chain_get(MIB_MBSSIB_WEP_TBL, i+1, (void *)&EntryWEP)) {  		
  				return 0;					
			}	
	
			if (Entry.wep == 0)
				Entry.wep = 1;

			if(! Entry.wlanDisabled)
				websWrite(wp, "<input type=\"radio\" name=\"wepSSID\" value=\"vap%d\" onClick=\"postWEP(%d, %d, %d, %d)\">VAP%d&nbsp;\n",
							i, Entry.wep, EntryWEP.wepKeyType+1, EntryWEP.wepDefaultKey+1, i+1, i);
		}
	}
#endif
#endif
	
#endif
	// Added by Mason Yu for 2 level web page
	if ( !strcmp(name, T("userMode")) ) {	
		char suStr[100], usStr[100];
#ifdef ZTE_531B_BRIDGE_SC
		//jim luo support ZTE 531B bridge only one user.
		sprintf(suStr, "<option selected value=\"0\">%s</option>\n", suName);
		websWrite(wp, suStr );
#else
	if(!strcmp(wp->user,suName))
		{
		sprintf(suStr, "<option selected value=\"0\">%s</option>\n", suName);
		sprintf(usStr, "<option value=\"1\">%s</option>\n", usName);
		}
	else
		sprintf(usStr, "<option selected value=\"1\">%s</option>\n", usName);
		
		websWrite(wp, suStr );
		websWrite(wp, usStr );	
#endif		
		return 0;
	}	
	if ( !strcmp(name, T("lan-dhcp-st")) ) {
		if ( !mib_get( MIB_DHCP_MODE, (void *)&vChar) )
			return -1;
		if (DHCP_LAN_SERVER == vChar) 
			websWrite(wp, "Enabled");
		else
			websWrite(wp, "Disabled");
		return 0;
	}
	else if ( !strcmp(name, T("br-stp-0")) ) {     
		if ( !mib_get( MIB_BRCTL_STP, (void *)&vChar) )
			return -1;
		if (0 == vChar) 
			websWrite(wp, "checked");      
		return 0;
	}
	else if ( !strcmp(name, T("br-stp-1")) ) {       
		if ( !mib_get( MIB_BRCTL_STP, (void *)&vChar) )
			return -1;
		if (1 == vChar) 
			websWrite(wp, "checked");      
		return 0;
	}
#ifdef CONFIG_USER_IGMPPROXY
	else if ( !strcmp(name, T("igmpProxy0")) ) {
		if ( !mib_get( MIB_IGMP_PROXY, (void *)&vChar) )
			return -1;
		if (0 == vChar)
			websWrite(wp, "checked");
		if (ifWanNum("rt") ==0)
			websWrite(wp, " disabled");
		return 0;
	}
	else if ( !strcmp(name, T("igmpProxy1")) ) {
		if ( !mib_get( MIB_IGMP_PROXY, (void *)&vChar) )
			return -1;
		if (1 == vChar)
			websWrite(wp, "checked");
		if (ifWanNum("rt") ==0)
			websWrite(wp, " disabled");
		return 0;
	}
	else if ( !strcmp(name, T("igmpProxy0d")) ) {
		if ( !mib_get( MIB_IGMP_PROXY, (void *)&vChar) )
			return -1;
		if (0 == vChar)
			websWrite(wp, "disabled");
		return 0;
	}
#endif
//#ifdef CONFIG_USER_UPNPD	
#if defined(CONFIG_USER_UPNPD)||defined(CONFIG_USER_MINIUPNPD)
	else if ( !strcmp(name, T("upnp0")) ) {
		if ( !mib_get( MIB_UPNP_DAEMON, (void *)&vChar) )
			return -1;
		if (0 == vChar)
			websWrite(wp, "checked");
		if (ifWanNum("rt") ==0)
			websWrite(wp, " disabled");
		return 0;
	}
	else if ( !strcmp(name, T("upnp1")) ) {
		if ( !mib_get( MIB_UPNP_DAEMON, (void *)&vChar) )
			return -1;
		if (1 == vChar)
			websWrite(wp, "checked");
		if (ifWanNum("rt") ==0)
			websWrite(wp, " disabled");
		return 0;
	}
	else if ( !strcmp(name, T("upnp0d")) ) {
		if ( !mib_get( MIB_UPNP_DAEMON, (void *)&vChar) )
			return -1;
		if (0 == vChar)
			websWrite(wp, "disabled");
		return 0;
	}
#endif	
	else if ( !strcmp(name, T("acl-cap0")) ) {
   		if ( !mib_get( MIB_ACL_CAPABILITY, (void *)&vChar) )
			return -1;
		if (0 == vChar)
			websWrite(wp, "checked");
		return 0;
	}
	else if ( !strcmp(name, T("acl-cap1")) ) {
   		if ( !mib_get( MIB_ACL_CAPABILITY, (void *)&vChar) )
			return -1;
		if (1 == vChar)
			websWrite(wp, "checked");
		return 0;
	}
#ifdef URL_BLOCKING_SUPPORT
	else if ( !strcmp(name, T("url-cap0")) ) {
   		if ( !mib_get( MIB_URL_CAPABILITY, (void *)&vChar) )
			return -1;
		if (0 == vChar)
			websWrite(wp, "checked");
		return 0;
	}
	else if ( !strcmp(name, T("url-cap1")) ) {
   		if ( !mib_get( MIB_URL_CAPABILITY, (void *)&vChar) )
			return -1;
		if (1 == vChar)
			websWrite(wp, "checked");
		return 0;
	}
#endif
#ifdef DOMAIN_BLOCKING_SUPPORT
	else if ( !strcmp(name, T("domainblk-cap0")) ) {
   		if ( !mib_get( MIB_DOMAINBLK_CAPABILITY, (void *)&vChar) )
			return -1;
		if (0 == vChar)
			websWrite(wp, "checked");
		return 0;
	}
	else if ( !strcmp(name, T("domainblk-cap1")) ) {
   		if ( !mib_get( MIB_DOMAINBLK_CAPABILITY, (void *)&vChar) )
			return -1;
		if (1 == vChar)
			websWrite(wp, "checked");
		return 0;
	}
#endif	
	else if ( !strcmp(name, T("dns0")) ) {
		if ( !mib_get( MIB_ADSL_WAN_DNS_MODE, (void *)&vChar) )
			return -1;
		if (0 == vChar) 
			websWrite(wp, "checked");
		return 0;
	}
	else if ( !strcmp(name, T("dns1")) ) {
		if ( !mib_get( MIB_ADSL_WAN_DNS_MODE, (void *)&vChar) )
			return -1;
		if (1 == vChar) 
			websWrite(wp, "checked");
		return 0;
	}
	else if ( !strcmp(name, T("portFwEn"))) {
		if ( !mib_get( MIB_PORT_FW_ENABLE, (void *)&vChar) )
			return -1;
		if (1 == vChar) 
			websWrite(wp, "checked");
		return 0;
	}
	if ( !strcmp(name, T("portFwNum"))) {
		vUInt = mib_chain_total(MIB_PORT_FW_TBL);
		if (0 == vUInt)
			websWrite(wp, "disableDelButton();");
		return 0;
	}
#ifdef NATIP_FORWARDING
	else if ( !strcmp(name, T("ipFwEn"))) {
		if ( !mib_get( MIB_IP_FW_ENABLE, (void *)&vChar) )
			return -1;
		if (1 == vChar) 
			websWrite(wp, "checked");
		return 0;
	}
	if ( !strcmp(name, T("ipFwNum"))) {
		vUInt = mib_chain_total(MIB_IP_FW_TBL);
		if (0 == vUInt)
			websWrite(wp, "disableDelButton();");
		return 0;
	}
#endif
	else if ( !strcmp(name, T("ipf_out_act0"))) {
		if ( !mib_get( MIB_IPF_OUT_ACTION, (void *)&vChar) )
			return -1;
		if (0 == vChar) 
			websWrite(wp, "checked");
		return 0;
	}
	else if ( !strcmp(name, T("ipf_out_act1"))) {
		if ( !mib_get( MIB_IPF_OUT_ACTION, (void *)&vChar) )
			return -1;
		if (1 == vChar) 
			websWrite(wp, "checked");
		return 0;
	}
	else if ( !strcmp(name, T("ipf_in_act0"))) {
		if ( !mib_get( MIB_IPF_IN_ACTION, (void *)&vChar) )
			return -1;
		if (0 == vChar) 
			websWrite(wp, "checked");
		return 0;
	}
	else if ( !strcmp(name, T("ipf_in_act1"))) {
		if ( !mib_get( MIB_IPF_IN_ACTION, (void *)&vChar) )
			return -1;
		if (1 == vChar) 
			websWrite(wp, "checked");
		return 0;
	}
	else if ( !strcmp(name, T("macf_out_act0"))) {
		if ( !mib_get( MIB_MACF_OUT_ACTION, (void *)&vChar) )
			return -1;
		if (0 == vChar) 
			websWrite(wp, "checked");
		return 0;
	}
	else if ( !strcmp(name, T("macf_out_act1"))) {
		if ( !mib_get( MIB_MACF_OUT_ACTION, (void *)&vChar) )
			return -1;
		if (1 == vChar) 
			websWrite(wp, "checked");
		return 0;
	}
	else if ( !strcmp(name, T("macf_in_act0"))) {
		if ( !mib_get( MIB_MACF_IN_ACTION, (void *)&vChar) )
			return -1;
		if (0 == vChar) 
			websWrite(wp, "checked");
		return 0;
	}
	else if ( !strcmp(name, T("macf_in_act1"))) {
		if ( !mib_get( MIB_MACF_IN_ACTION, (void *)&vChar) )
			return -1;
		if (1 == vChar) 
			websWrite(wp, "checked");
		return 0;
	}
	else if ( !strcmp(name, T("dmzEn"))) {
		if ( !mib_get( MIB_DMZ_ENABLE, (void *)&vChar) )
			return -1;
		if (vChar)
			websWrite(wp, "checked");
		return 0;
	}
	else if ( !strcmp(name, T("ipFilterNum"))) {
		vUInt = mib_chain_total(MIB_IP_PORT_FILTER_TBL);
		if (0 == vUInt)
			websWrite(wp, "disableDelButton();");
		return 0;
	}
else if (!strcmp(name, T("fc_page2"))) {
#if defined(ZTE_531B_BRIDGE_SC) && !defined(ZTE_GENERAL_ROUTER_SC)
		websWrite(wp, "disabled");
#endif
		return 0;
	}
#if defined(ZTE_531B_BRIDGE_SC) || defined(ZTE_GENERAL_ROUTER_SC)
#ifdef ZTE_GENERAL_ROUTER_SC
	else if(!strcmp(name,"openedPort")) {
		MIB_CE_VTL_SVR_T vtlSvrEntry;
		int vtlSvrEntryNums=mib_chain_total(MIB_VIRTUAL_SVR_TBL);
		int vtlSvrEntryIndex = 0;

		for(vtlSvrEntryIndex=0;vtlSvrEntryIndex<vtlSvrEntryNums;vtlSvrEntryIndex++)
		{
			if(!mib_chain_get(MIB_VIRTUAL_SVR_TBL,vtlSvrEntryIndex,&vtlSvrEntry))
			{
				printf("get MIB_VIRTUAL_SVR_TBL error!\n");
				return 0;
			}
			websWrite(wp, T("openedPort[%d]=new portInfo(%d,%d,\"%s\");\n"), 
				vtlSvrEntryIndex,vtlSvrEntry.wanStartPort,vtlSvrEntry.wanEndPort,vtlSvrEntry.svrName);
		}
		return 0;
	}
	else if (!strcmp(name, "rmtAccPort")) {
		MIB_CE_ACC_T Entry;
		int totalEntry = mib_chain_total(MIB_ACC_TBL);
		int entryIdx;
		int portArray[21];
		int i, idx=0;

		memset(portArray, 0, sizeof(portArray));
		for (entryIdx=0; entryIdx<totalEntry; entryIdx++) {
			if(!mib_chain_get(MIB_ACC_TBL, entryIdx, &Entry))
			{
				printf("get MIB_ACC_TBL error!\n");
				return 0;
			}
			if (Entry.telnet==0x01)
				portArray[idx++] = Entry.telnet_port;
			if (Entry.ftp==0x01)
				portArray[idx++] = Entry.ftp_port;
			if (Entry.web==0x01)
				portArray[idx++] = Entry.web_port;
		}
		if (idx) {
			for(i=0; i<idx-1; i++)
				websWrite(wp,T("%d,"),portArray[i]);
			websWrite(wp, T("%d"), portArray[idx-1]);
		}
		
		return 0;
	}
#endif
	else if (!strcmp(name, T("wanip"))) {
		unsigned char tmpBuf[20];

		memcpy(tmpBuf, fcEntry2.wanIpAddr, IP_ADDR_LEN);
		if ( ((struct in_addr *)tmpBuf)->s_addr == 0 )
			websWrite(wp, "0.0.0.0");
		else
			websWrite(wp, T("%s"),  inet_ntoa(*((struct in_addr *)tmpBuf)));
		
		return 0;
	}
	else if (!strcmp(name, T("ip"))) {
		unsigned char tmpBuf[20];

		memcpy(tmpBuf, fcEntry2.ipAddr, IP_ADDR_LEN);
		if ( ((struct in_addr *)tmpBuf)->s_addr == 0 ) {
			getMIB2Str(MIB_ADSL_LAN_IP, tmpBuf);
			websWrite(wp, T("%s"),  tmpBuf);
		} else {
			websWrite(wp, T("%s"),  inet_ntoa(*((struct in_addr *)tmpBuf)));
		}
		return 0;
	}
	else if (!strcmp(name, T("mask"))) {
		unsigned char tmpBuf[20];

		memcpy(tmpBuf, fcEntry2.netMask, IP_ADDR_LEN);
		if ( ((struct in_addr *)tmpBuf)->s_addr == 0 ) {
			getMIB2Str(MIB_ADSL_LAN_SUBNET, tmpBuf);
			websWrite(wp, T("%s"),  tmpBuf);
		} else
			websWrite(wp, T("%s"),  inet_ntoa(*((struct in_addr *)tmpBuf)));
		return 0;
	}
	else if (!strcmp(name, T("IP2"))) {
		if (fcEntry2.enable_ip2)
			websWrite(wp, "checked");
		return 0;
	}
	else if (!strcmp(name, T("enblDhcps"))) {
		if (fcEntry2.enable_dhcp)
			websWrite(wp, "checked");
		return 0;
	}
	else if (!strcmp(name, T("vpi-vci"))) {
		char tmpBuf[10];
		
		sprintf(tmpBuf, "%d / %d", fcEntry.vpi, fcEntry.vci);

		websWrite(wp, T("%s"), tmpBuf);
		return 0;
	}
	else if (!strcmp(name, T("conn-type"))) {
		char tmpBuf[60];

		memset(tmpBuf, 0, sizeof(tmpBuf));
		/*if (fcEntry.cmode == 0)
			strcpy(tmpBuf, "bridge ");
		if (fcEntry.encap == 1)
			strcpy(tmpBuf+strlen(tmpBuf), "LLC/SNAP");
		else if (fcEntry.encap == 0)
			strcpy(tmpBuf+strlen(tmpBuf), "VC-Mux");
		websWrite(wp, T("%s"), tmpBuf);*/
		switch(fcEntry.cmode)
		{
			case ADSL_BR1483:
				strcpy(tmpBuf, "bridge ");
				break;
			case ADSL_PPPoE:
				strcpy(tmpBuf, "PPPoE ");
				break;
			case ADSL_PPPoA:
				strcpy(tmpBuf, "PPPoA ");
				break;
			case ADSL_MER1483:
				strcpy(tmpBuf, "MER ");
				break;
			case ADSL_RT1483:
				strcpy(tmpBuf, "IPoA ");
				break;
			default:
				break;
		}
		if (fcEntry.encap == 1)
			strcpy(tmpBuf+strlen(tmpBuf), "LLC/SNAP");
		else if (fcEntry.encap == 0)
			strcpy(tmpBuf+strlen(tmpBuf), "VC-Mux");

		if (fcEntry.cmode==2 || fcEntry.cmode==3)
		{//pppoe/ppp0a
			char tmpBuf2[20];
			switch(fcEntry.pppCtype)
			{
				case 0://connect forever
					strcpy(tmpBuf+strlen(tmpBuf), Tconn_forever);
					break;
				case 1://connect on demand
					sprintf(tmpBuf2, Tconn_on_dem, fcEntry.pppIdleTime);
					strcpy(tmpBuf+strlen(tmpBuf), tmpBuf2);
					break;
				case 2:
					strcpy(tmpBuf+strlen(tmpBuf), Tconn_manual);
					break;
			}
		}
		websWrite(wp, T("%s"), tmpBuf);
				
		return 0;
	}
	else if (!strcmp(name, T("lanip"))) {
		char tmpBuf[40];
		
		strcpy(tmpBuf, inet_ntoa(*((struct in_addr *)fcEntry2.ipAddr)));
		strcat(tmpBuf, " / ");
		strcat(tmpBuf, inet_ntoa(*((struct in_addr *)fcEntry2.netMask)));

		websWrite(wp, T("%s"), tmpBuf);
		return 0;
	}
	else if (!strcmp(name, T("seclanip"))) {
		char tmpBuf[40], strIp[IP_ADDR_LEN], strMask[IP_ADDR_LEN];;

		if (fcEntry2.enable_ip2) {
			memcpy(strIp, fcEntry2.lanIp2, IP_ADDR_LEN);
			memcpy(strMask, fcEntry2.subnetMask2, IP_ADDR_LEN);
			strcpy(tmpBuf, inet_ntoa(*((struct in_addr *)strIp)));
			strcat(tmpBuf+strlen(tmpBuf), " / ");
			strcat(tmpBuf+strlen(tmpBuf), inet_ntoa(*((struct in_addr *)strMask)));
		} else {
			strcpy(tmpBuf, "0.0.0.0 / 0.0.0.0");
		}

		websWrite(wp, T("%s"), tmpBuf);
		return 0;
	}
	else if (!strcmp(name, T("dhcps"))) {
		char tmpBuf[32];

		if (fcEntry2.enable_dhcp == 2)
			websWrite(wp, T("%s"), "");
		else
			websWrite(wp, T("%s"), "ͣ");
		
		return 0;
	}
#endif
	else if ( !strcmp(name, T("macFilterNum"))) {
		vUInt = mib_chain_total(MIB_MAC_FILTER_TBL);
		if (0 == vUInt)
			websWrite(wp, "disableDelButton();");
		return 0;
	}
	else if ( !strcmp(name, T("vcMax"))) {
		vUInt = mib_chain_total(MIB_ATM_VC_TBL);
		if (vUInt >= 16) {
			websWrite(wp, "alert(\"Max number of ATM VC Settings is 16!\");");
			websWrite(wp, "return false;");
		}
		return 0;
	}
	else if ( !strcmp(name, T("vcCount"))) {
		vUInt = mib_chain_total(MIB_ATM_VC_TBL);
		if (vUInt == 0) {
			websWrite(wp, "disableButton(document.adsl.delvc);");
			websWrite(wp, "disableButton(document.adsl.refresh);");
		}
		return 0;
	}
	else if ( !strcmp(name, T("pppoeStatus")) ) {
		if (0) {
			websWrite(wp, "\n<script> setPPPConnected(); </script>\n");
		}
		return 0;
	}
#ifdef CONFIG_USER_PPPOE_PROXY
  else if(!strcmp(name,T("pppoeProxy")))
  	{
  	websWrite(wp,T("<tr><td><font size=2><b>PPPoE Proxy:</b></td>"
         "<td><b><input type=\"radio\" value=1 name=\"pppEnable\" >Enable&nbsp;&nbsp;"
	"<input type=\"radio\" value=0 name=\"pppEnable\" checked>Disable</b></td></tr>"));
  	}
  else if(!strcmp(name,T("pppSettingsDisable")))
  	{
  	  websWrite(wp,T("{document.adsl.pppEnable[0].disabled=true;\n"
	  	"document.adsl.pppEnable[1].disabled=true;}"));
  	}
    else if(!strcmp(name,T("pppSettingsEnable")))
  	{
  	  websWrite(wp,T("{document.adsl.pppEnable[0].disabled=false;\n"
	  	"document.adsl.pppEnable[1].disabled=false;}else{document.adsl.pppEnable[0].disabled=true;\n"
	  	"document.adsl.pppEnable[1].disabled=true;}"
	  	"document.adsl.pppEnable[0].checked=false;"
	  	"document.adsl.pppEnable[1].checked=true;"));
  	} 

 #endif
  #ifdef CONFIG_USER_PPPOE_PROXY
     else if(!strcmp(name,T("PostVC")))
     	{
     	   websWrite(wp,T("function postVC(vpi,vci,encap,napt,mode,username,passwd,pppType,idletime,pppoeProxyEnable,ipunnum,ipmode,ipaddr,remoteip,netmask,droute,status,enable)"));
     	}
     else if(!strcmp(name,T("pppoeProxyEnable")))
     	{
	websWrite(wp,T("  if(mode==\"PPPoE\")"
		"{if(pppoeProxyEnable)"
		"{ document.adsl.pppEnable[0].checked=true;\n"
                  "document.adsl.pppEnable[1].checked=false;}\n"			
		"else {document.adsl.pppEnable[0].checked=false;"
		 " document.adsl.pppEnable[1].checked=true;}  "
		" document.adsl.pppEnable[0].disabled=false;"
			  " document.adsl.pppEnable[1].disabled=false;"));	
	websWrite(wp,T(" }else"
		"{"
		"	  document.adsl.pppEnable[0].checked=false;"
		"	   document.adsl.pppEnable[1].checked=true;"
		"	   document.adsl.pppEnable[0].disabled=true;"
		"	   document.adsl.pppEnable[1].disabled=true;}"
		));
     	}
  #else
   else if(!strcmp(name,T("PostVC")))
     	{
     	   websWrite(wp,T("function postVC(vpi,vci,encap,napt,mode,username,passwd,pppType,idletime,ipunnum,ipmode,ipaddr,remoteip,netmask,droute,status,enable)"));
     	}
     else if(!strcmp(name,T("pppoeProxyEnable")))
     	{
     	 
     	}
  #endif

	else if ( !strcmp(name, T("adsl-line-mode")) ) {
		if ( !mib_get( MIB_ADSL_MODE, (void *)&vChar) )
			return -1;
		if (1 == vChar) {
			websWrite(wp,"<option selected value=\"1\">T1.413</option>");
			websWrite(wp,"<option value=\"2\">G.dmt</option>");
			websWrite(wp,"<option value=\"3\">MultiMode</option>");
		} else if (2 == vChar) {
			websWrite(wp,"<option value=\"1\">T1.413</option>");
			websWrite(wp,"<option selected value=\"2\">G.dmt</option>");
			websWrite(wp,"<option value=\"3\">MultiMode</option>");
		} else if (3 == vChar) {
			websWrite(wp,"<option value=\"1\">T1.413</option>");
			websWrite(wp,"<option value=\"2\">G.dmt</option>");
			websWrite(wp,"<option selected value=\"3\">MultiMode</option>");
		}		
		return 0;
	}
#ifdef WLAN_SUPPORT
#ifdef WLAN_8185AG
	else if ( !strcmp(name, T("wl_txRate"))) {
		mib_get( MIB_WLAN_BAND, (void *)&vChar);
		websWrite(wp, "band=%d\n",vChar);
		mib_get( MIB_WLAN_FIX_RATE, (void *)&vUShort);
		websWrite(wp, "txrate=%d\n",vUShort);
		mib_get( MIB_WLAN_RATE_ADAPTIVE_ENABLED, (void *)&vChar);
		websWrite(wp, "auto=%d\n",vChar);
	}
#endif
//added by xl_yue:
#if defined(ZTE_531B_BRIDGE_SC) || defined(ZTE_GENERAL_ROUTER_SC)
	else if ( !strcmp(name, T("wl_domain"))) {
		struct wlCountryReg * pwlCountryReg = wlCountryRegList;
		//selIndex must start from 0, it should keep accordance with wlCountryRegList's index,
		//because in formWlanSetup(), the value of select option will be used as the array's index of wlCountryRegList
		//and in initWlBasicPage(), the regDomainList[] is start from 0
		int selIndex = 0;
		while(strlen(pwlCountryReg->country)){
			websWrite(wp, "<option value=\"%d\">%s</option>\n",selIndex,pwlCountryReg->country);
			pwlCountryReg++;
			selIndex++;
		}
	}
#endif
	else if ( !strcmp(name, T("wl_chno"))) {
//xl_yue:
#if defined(ZTE_531B_BRIDGE_SC) || defined(ZTE_GENERAL_ROUTER_SC)
		mib_get( MIB_WLAN_REG_DOMAIN, (void *)&vChar);
#else
		mib_get( MIB_HW_REG_DOMAIN, (void *)&vChar);
#endif
		websWrite(wp, "regDomain=%d\n",vChar);
		mib_get( MIB_WLAN_CHAN_NUM, (void *)&vChar);
		websWrite(wp, "defaultChan=%d\n",vChar);
	}
#endif

	//for web log
	else if ( !strcmp(name, T("log-cap0")) ) {
   		if ( !mib_get( MIB_SYSLOG, (void *)&vChar) )
			return -1;
		if (0 == vChar)
			websWrite(wp, "checked");
		return 0;
	}
	else if ( !strcmp(name, T("log-cap1")) ) {
   		if ( !mib_get( MIB_SYSLOG, (void *)&vChar) )
			return -1;
		if (1 == vChar)
			websWrite(wp, "checked");
		return 0;
	}

	//for adsl debug
	else if ( !strcmp(name, T("adsldbg-cap0")) ) {
   		if ( !mib_get( MIB_ADSL_DEBUG, (void *)&vChar) )
			return -1;
		if (0 == vChar)
			websWrite(wp, "checked");
		return 0;
	}
	else if ( !strcmp(name, T("adsldbg-cap1")) ) {
   		if ( !mib_get( MIB_ADSL_DEBUG, (void *)&vChar) )
			return -1;
		if (1 == vChar)
			websWrite(wp, "checked");
		return 0;
	}
#ifdef _CWMP_MIB_
	else if ( !strcmp(name, T("tr069-interval")) ) {
   		if ( !mib_get( CWMP_INFORM_ENABLE, (void *)&vChar) )
			return -1;
		if (0 == vChar)
			websWrite(wp, "disabled");
		return 0;
	}
	else if ( !strcmp(name, T("tr069-inform-0")) ) {
   		if ( !mib_get( CWMP_INFORM_ENABLE, (void *)&vChar) )
			return -1;
		if (0 == vChar)
			websWrite(wp, "checked");
		return 0;
	}
	else if ( !strcmp(name, T("tr069-inform-1")) ) {
   		if ( !mib_get( CWMP_INFORM_ENABLE, (void *)&vChar) )
			return -1;
		if (1 == vChar)
			websWrite(wp, "checked");
		return 0;
	}
	else if ( !strcmp(name, T("tr069-dbgmsg-0")) ) {
   		if ( !mib_get( CWMP_FLAG, (void *)&vChar) )
			return -1;
		if ( (vChar & CWMP_FLAG_DEBUG_MSG)==0 )
			websWrite(wp, "checked");
		return 0;
	}
	else if ( !strcmp(name, T("tr069-dbgmsg-1")) ) {
   		if ( !mib_get( CWMP_FLAG, (void *)&vChar) )
			return -1;
		if ( (vChar & CWMP_FLAG_DEBUG_MSG)!=0 )
			websWrite(wp, "checked");
		return 0;
	}
	else if ( !strcmp(name, T("tr069-certauth-0")) ) {
   		if ( !mib_get( CWMP_FLAG, (void *)&vChar) )
			return -1;
		if ( (vChar & CWMP_FLAG_CERT_AUTH)==0 )
			websWrite(wp, "checked");
		return 0;
	}
	else if ( !strcmp(name, T("tr069-certauth-1")) ) {
   		if ( !mib_get( CWMP_FLAG, (void *)&vChar) )
			return -1;
		if ( (vChar & CWMP_FLAG_CERT_AUTH)!=0 )
			websWrite(wp, "checked");
		return 0;
	}
	else if ( !strcmp(name, T("tr069-sendgetrpc-0")) ) {
   		if ( !mib_get( CWMP_FLAG, (void *)&vChar) )
			return -1;
		if ( (vChar & CWMP_FLAG_SENDGETRPC)==0 )
			websWrite(wp, "checked");
		return 0;
	}
	else if ( !strcmp(name, T("tr069-sendgetrpc-1")) ) {
   		if ( !mib_get( CWMP_FLAG, (void *)&vChar) )
			return -1;
		if ( (vChar & CWMP_FLAG_SENDGETRPC)!=0 )
			websWrite(wp, "checked");
		return 0;
	}
	else if ( !strcmp(name, T("tr069-skipmreboot-0")) ) {
   		if ( !mib_get( CWMP_FLAG, (void *)&vChar) )
			return -1;
		if ( (vChar & CWMP_FLAG_SKIPMREBOOT)==0 )
			websWrite(wp, "checked");
		return 0;
	}
	else if ( !strcmp(name, T("tr069-skipmreboot-1")) ) {
   		if ( !mib_get( CWMP_FLAG, (void *)&vChar) )
			return -1;
		if ( (vChar & CWMP_FLAG_SKIPMREBOOT)!=0 )
			websWrite(wp, "checked");
		return 0;
	}
	else if ( !strcmp(name, T("tr069-delay-0")) ) {
   		if ( !mib_get( CWMP_FLAG, (void *)&vChar) )
			return -1;
		if ( (vChar & CWMP_FLAG_DELAY)==0 )
			websWrite(wp, "checked");
		return 0;
	}
	else if ( !strcmp(name, T("tr069-delay-1")) ) {
   		if ( !mib_get( CWMP_FLAG, (void *)&vChar) )
			return -1;
		if ( (vChar & CWMP_FLAG_DELAY)!=0 )
			websWrite(wp, "checked");
		return 0;
	}
	else if ( !strcmp(name, T("tr069-autoexec-0")) ) {
   		if ( !mib_get( CWMP_FLAG, (void *)&vChar) )
			return -1;
		if ( (vChar & CWMP_FLAG_AUTORUN)==0 )
			websWrite(wp, "checked");
		return 0;
	}
	else if ( !strcmp(name, T("tr069-autoexec-1")) ) {
   		if ( !mib_get( CWMP_FLAG, (void *)&vChar) )
			return -1;
		if ( (vChar & CWMP_FLAG_AUTORUN)!=0 )
			websWrite(wp, "checked");
		return 0;
	}

#endif
#ifdef CONFIG_WIFI_SIMPLE_CONFIG // WPS
	else if (!strcmp(name, T("wscConfig-0")) ) {
		if (!mib_get(MIB_WSC_CONFIGURED, (void *)&vChar))
			return -1;
		if (!vChar) websWrite(wp, "checked");
		return 0;
	}
	else if (!strcmp(name, T("wscConfig-1"))) {
		if (!mib_get(MIB_WSC_CONFIGURED, (void *)&vChar))
			return -1;
		if (vChar)
			websWrite(wp, "checked");
		return 0;
	}
	else if (!strcmp(name, T("wscConfig-A"))) {
		if (!mib_get(MIB_WSC_CONFIGURED, (void *)&vChar))
			return -1;
		if (vChar)
			websWrite(wp, "isConfig=1;");
		else
			websWrite(wp, "isConfig=0;");
		return 0;
	}
	else if (!strcmp(name, T("wlanMode")))  {
		if (!mib_get(MIB_WLAN_MODE, (void *)&vChar))
			return -1;
		if (vChar)
			websWrite(wp, "isClient=1;");
		else
			websWrite(wp, "isClient=0;");
		return 0;
		
	}
	else if (!strcmp(name, T("wscDisable")))  {
		
		if (!mib_get(MIB_WSC_DISABLE, (void *)&vChar))
			return -1;
		if (vChar)
			websWrite(wp, "checked");
		return 0;		
	}
	else if (!strcmp(name, T("wps_auth")))  {
		
		if (!mib_get(MIB_WSC_AUTH, (void *)&vChar))
			return -1;
		switch(vChar) {
			case WSC_AUTH_OPEN: websWrite(wp, "Open"); break;
			case WSC_AUTH_WPAPSK: websWrite(wp, "WPA PSK"); break;
			case WSC_AUTH_SHARED: websWrite(wp, "WEP Shared"); break;
			case WSC_AUTH_WPA: websWrite(wp, "WPA Enterprise"); break;
			
			case WSC_AUTH_WPA2: websWrite(wp, "WPA2 Enterprise"); break;
			case WSC_AUTH_WPA2PSK: websWrite(wp, "WPA2 PSK"); break;
			case WSC_AUTH_WPA2PSKMIXED: websWrite(wp, "WPA2-Mixed PSK"); break;
			default:
				break;
		}
		return 0;
	}
	else if (!strcmp(name, T("wps_enc")))  {
		
		if (!mib_get(MIB_WSC_ENC, (void *)&vChar))
			return -1;
		switch(vChar) {
			case 0:
			case WSC_ENCRYPT_NONE: websWrite(wp, "None"); break;
			case WSC_ENCRYPT_WEP: websWrite(wp, "WEP"); break;
			case WSC_ENCRYPT_TKIP: websWrite(wp, "TKIP"); break;
			case WSC_ENCRYPT_AES: websWrite(wp, "AES"); break;			
			case WSC_ENCRYPT_TKIPAES: websWrite(wp, "TKIP+AES"); break;
			default:
				break;
		}
		return 0;
	}
#endif
	// Jenny, for RFC1577
	if ( !strcmp(name, T("adslcmode")) ) {
#ifdef CONFIG_ATM_CLIP
		websWrite(wp, "<option value=\"5\">1577 Routed</option>" );
#endif
		return 0;
	}
//add by ramen for create new menu 
       else if(!strcmp(name,T("CreateMenu"))){
	   
	   #if defined( ZTE_531B_BRIDGE_SC)||defined(ZTE_GENERAL_ROUTER_SC)	    
	   createMenu(0,wp,0,0);
	   #else
	   addMenuJavaScript( wp,0,0);
	   char *argv[]={"1",0};
	   if(!strcmp(argv[0],"1"))
	                     createMenu(0,wp,2,argv);
	   websWrite(wp,T("</script>\n</head>\n"));
	   websWrite(wp,T("<body onload=\"MTMStartMenu(true)\" bgcolor=\"#000033\" text=\"#ffffcc\" link=\"yellow\" vlink=\"lime\" alink=\"red\">\n"));
	   if(!strcmp(argv[0],"0"))
	                     {
	                      websWrite(wp,T("<table width=100%% border=0 cellpadding=0 cellspacing=0>\n<tr><td width=20%%>&nbsp&nbsp</td><td  align=left>\n"));
	                     createMenu(0,wp,2,argv);
			   websWrite(wp,T("</td></tr>\n</table>\n"));
	   	     }
	 
	   #endif	
	   websWrite(wp,T("</body>\n</html>\n"));
	   	return 0;
       	}else if(!strcmp(name,T("CreateMenu_user")))
       	
       		{
       		createMenu_user(0,wp,0,0);
			return 0;
       		}
	//add by ramen for zte acl default ip
	else if(!strcmp(name,T("remoteClientIp")))
		{
		  websWrite(wp,T("%s"),wp->remote_ip_addr);
			return 0;
		}
	   //add by ramen shen for the remote managment for zte
	   #ifdef ZTE_GENERAL_ROUTER_SC	
	   else if(!strcmp(name,T("WANInterfaceList")))
       		{
				MIB_CE_ATM_VC_T Entry;
				int entryNum;
				int mibcnt;		
				char interfacename[MAX_NAME_LEN];
				entryNum = mib_chain_total(MIB_ATM_VC_TBL);
				unsigned char forSelect=0;
		            for(mibcnt=0;mibcnt<entryNum;mibcnt++)
		            {
		            if (!mib_chain_get(MIB_ATM_VC_TBL, mibcnt, (void *)&Entry))
						{
		  					websError(wp, 400, T("Get chain record error!\n"));
							return -1;
						}
			      if(Entry.cmode!=ADSL_BR1483)// ADSL_BR1483 ADSL_MER1483 ADSL_PPPoE ADSL_PPPoA	ADSL_RT1483	ADSL_RT1577
			      	{
			      	  	#ifdef CTC_WAN_NAME
					sprintf(interfacename,"Internet_R_%u_%u",Entry.vpi,Entry.vci);
					#else
				  	if((Entry.cmode == ADSL_PPPoE) || (Entry.cmode == ADSL_PPPoA) ) {
							 sprintf(interfacename, "PPP%u", PPP_INDEX(Entry.ifIndex));
				  	}else
				  	{
				  	                     sprintf(interfacename, "vc%u", PPP_INDEX(Entry.ifIndex));
				  	}
		                           #endif
					if(forSelect==0)
						{
						  websWrite(wp,T("<tr>\n<td>ѡ˿</td>"
						  	"<td colspan=\"2\"><select size=\"1\" name=\"interfaceName\" onChange=\"interfaceNameSelection()\">"));
						  websWrite(wp,T("<option selected value=\"%d\">%s</option>"),mibcnt,interfacename);					
						}
					else {
						websWrite(wp,T("<option  value=\"%d\">%s</option>"),mibcnt,interfacename);						
						}
					forSelect++;
					 
		                    }
				 
		            }
		            if(forSelect==0) websWrite(wp,T("<tr><td>û·ģʽPVC<a href=\"wanadsl_sc.asp\">ӿ</a></td></tr>\n"));
				else websWrite(wp,T("</select></td></tr>\n"));
				
       		return 0;
       			}else if(!strcmp(name,"interfaceNameSelection"))
       				{
       				int accIndex=0,nBytesSent=0;
				char strDocAcc[]="document.acc.";								
				MIB_CE_ACC_T Entry;
				int accTotalNums=mib_chain_total(MIB_ACC_TBL);
				for(accIndex=0;accIndex<accTotalNums;accIndex++)
				{
				if(!mib_chain_get(MIB_ACC_TBL,accIndex,&Entry))
					{
					printf("get MIB_ACC_TBL enttry failed\n");
					return 0;
					}
				if(Entry.protocol != ADSL_BR1483)
					{
					nBytesSent+=websWrite(wp,T("if(document.acc.interfaceName.value ==\"%d\")\n{\n"),accIndex);
					#ifdef CONFIG_USER_TELNETD_TELNETD
						if (Entry.telnet & 0x01)
						nBytesSent += websWrite(wp, T("%sw_telnet.checked=true;\n"), strDocAcc);
						else
							nBytesSent += websWrite(wp, T("%sw_telnet.checked=false;\n"), strDocAcc);
					#endif
					//#ifdef CONFIG_USER_FTPD_FTPD
					#if defined(CONFIG_USER_BFTPD_BFTPD) || defined(CONFIG_USER_FTPD_FTPD)
						if (Entry.ftp & 0x01)
						nBytesSent += websWrite(wp, T("%sw_ftp.checked=true;\n"), strDocAcc);
						else
							nBytesSent += websWrite(wp, T("%sw_ftp.checked=false;\n"), strDocAcc);
					#endif
					#ifdef CONFIG_USER_TFTPD_TFTPD
						if (Entry.tftp & 0x01)
						nBytesSent += websWrite(wp, T("%sw_tftp.checked=true;\n"), strDocAcc);
						else
							nBytesSent += websWrite(wp, T("%sw_tftp.checked=false;\n"), strDocAcc);
					#endif
						if (Entry.web & 0x01)
						nBytesSent += websWrite(wp, T("%sw_web.checked=true;\n"), strDocAcc);
						else nBytesSent += websWrite(wp, T("%sw_web.checked=false;\n"), strDocAcc);
					#ifdef CONFIG_USER_SNMPD_SNMPD_V2CTRAP
						if (Entry.snmp & 0x01)
						nBytesSent += websWrite(wp, T("%sw_snmp.checked=true;\n"), strDocAcc);
						else
							nBytesSent += websWrite(wp, T("%sw_snmp.checked=false;\n"), strDocAcc);
					#endif
					#ifdef CONFIG_USER_SSH_DROPBEAR
						if (Entry.ssh & 0x01)
						nBytesSent += websWrite(wp, T("%sw_ssh.checked=true;\n"), strDocAcc);
						else nBytesSent += websWrite(wp, T("%sw_ssh.checked=false;\n"), strDocAcc);
					#endif
						if (Entry.icmp & 0x01)							
						nBytesSent += websWrite(wp, T("%sw_icmp.checked=true;\n"), strDocAcc);
						else nBytesSent += websWrite(wp, T("%sw_icmp.checked=false;\n"), strDocAcc);
					#ifdef CONFIG_USER_TELNETD_TELNETD	
						nBytesSent += websWrite(wp, T("%sw_telnet_port.value=%d;\n"), strDocAcc, Entry.telnet_port);
					#endif
					//#ifdef CONFIG_USER_FTPD_FTPD
					#if defined(CONFIG_USER_BFTPD_BFTPD) || defined(CONFIG_USER_FTPD_FTPD)
					         nBytesSent += websWrite(wp, T("%sw_ftp_port.value=%d;\n"), strDocAcc, Entry.ftp_port);	
					#endif
					         nBytesSent += websWrite(wp, T("%sw_web_port.value=%d;\n"), strDocAcc, Entry.web_port);
						nBytesSent +=websWrite(wp,T("document.getElementById(\"accSetting\").style.display=\"\";\n"));
						nBytesSent +=websWrite(wp,T("return true;\n}\n"));
					}	
				}
			       		return 0;
       			}else if(!strcmp(name,"pwdManagment"))
       				{
       				if(!strcmp(wp->user,suName))
       				websWrite(wp,T("<form action=/goform/formPasswordSetup method=POST name=\"password\">"));
				else
				websWrite(wp,T("<form action=/goform/admin/formPasswordSetup method=POST name=\"password\">"));
       				return 0;
       				}
	#endif
#ifdef DEFAULT_GATEWAY_V2
	// Jenny, for PPPoE auto route
	else if ( !strcmp(name, T("autort")) ) {
#ifdef AUTO_PPPOE_ROUTE
		websWrite(wp, "<option value=239>Auto</option>" );
#endif
		return 0;
	}
#endif
	
	else
		return -1;
	
	return 0;
}
// Kaohj
#if 0
int getIndex(int eid, webs_t wp, int argc, char_t **argv)
{
	char_t *name;
	char_t  buffer[100];

	unsigned char vChar;
	unsigned short vUShort;
	unsigned int vUInt;

   	if (ejArgs(argc, argv, T("%s"), &name) < 1) {
   		websError(wp, 400, T("Insufficient args\n"));
   		return -1;
   	}

	memset(buffer,0x00,100);
   	if ( !strcmp(name, T("device-type")) ) {
 		if ( !mib_get( MIB_DEVICE_TYPE, (void *)&vChar) )
			return -1;
#ifdef __uClinux__
		sprintf(buffer, "%u", vChar);
#else
		sprintf(buffer,"%u", 1);
#endif
		ejSetResult(eid, buffer);
		return 0;
	}
   	if ( !strcmp(name, T("boot-mode")) ) {
 		if ( !mib_get( MIB_BOOT_MODE, (void *)&vChar) )
			return -1;
		sprintf(buffer, "%u", vChar);
		ejSetResult(eid, buffer);
		return 0;
	}
	if ( !strcmp(name, T("dhcp-mode")) ) {
 		if ( !mib_get( MIB_DHCP_MODE, (void *)&vChar) )
			return -1;
		sprintf(buffer, "%u", vChar);
		ejSetResult(eid, buffer);
		return 0;
	}	
   	if ( !strcmp(name, T("adsl-line-mode")) ) {
 		if ( !mib_get( MIB_ADSL_MODE, (void *)&vChar) )
			return -1;
		sprintf(buffer, "%u", vChar);
		ejSetResult(eid, buffer);
		return 0;
	}
#ifdef CONFIG_USER_ROUTED_ROUTED
   	if ( !strcmp(name, T("rip-on")) ) {
 		if ( !mib_get( MIB_RIP_ENABLE, (void *)&vChar) )
			return -1;
		sprintf(buffer, "%u", vChar);
		ejSetResult(eid, buffer);
		return 0;
	}
   	if ( !strcmp(name, T("rip-ver")) ) {
 		if ( !mib_get( MIB_RIP_VERSION, (void *)&vChar) )
			return -1;
		sprintf(buffer, "%u", vChar);
		ejSetResult(eid, buffer);
		return 0;
	}
#endif
// Commented by Mason Yu for dhcpmode
#if 0	
   	if ( !strcmp(name, T("lan-dhcp")) ) {
 		if ( !mib_get( MIB_ADSL_LAN_DHCP, (void *)&vChar) )
			return -1;
		sprintf(buffer, "%u", vChar);
		ejSetResult(eid, buffer);
		return 0;
	}
#endif	
 	else if ( !strcmp(name, T("br-stp")) ) {
   		if ( !mib_get( MIB_BRCTL_STP, (void *)&vChar) )
			return -1;
		sprintf(buffer, "%u", vChar);
		ejSetResult(eid, buffer);
		return 0;
	}
#ifdef CONFIG_EXT_SWITCH
 	else if ( !strcmp(name, T("mp-mode")) ) {
   		if ( !mib_get( MIB_MPMODE, (void *)&vChar) )
			return -1;
		sprintf(buffer, "%u", vChar);
		ejSetResult(eid, buffer);
		return 0;
	}
#endif
#ifdef CONFIG_USER_IGMPPROXY
 	else if ( !strcmp(name, T("igmp-proxy")) ) {
   		if ( !mib_get( MIB_IGMP_PROXY, (void *)&vChar) )
			return -1;
		sprintf(buffer, "%u", vChar);
		ejSetResult(eid, buffer);
		return 0;
	}
#endif
	
	else if ( !strcmp(name, T("acl-cap")) ) {
   		if ( !mib_get( MIB_ACL_CAPABILITY, (void *)&vChar) )
			return -1;
		sprintf(buffer, "%u", vChar);
		ejSetResult(eid, buffer);
		return 0;
	}
	
   	else if ( !strcmp(name, T("wan-dns")) ) {
 		if ( !mib_get( MIB_ADSL_WAN_DNS_MODE, (void *)&vChar) )
			return -1;
		sprintf(buffer, "%u", vChar);
		ejSetResult(eid, buffer);
		return 0;
	}
	else if ( !strcmp(name, T("portFwEnabled"))) {
		if ( !mib_get( MIB_PORT_FW_ENABLE, (void *)&vChar) )
			return -1;
		sprintf(buffer, "%u", vChar);
		ejSetResult(eid, buffer);
		return 0;
	}
	else if ( !strcmp(name, T("defaultFilterAction"))) {
		if ( !mib_get( MIB_IPF_OUT_ACTION, (void *)&vChar) )
			return -1;
		sprintf(buffer, "%u", vChar);
		ejSetResult(eid, buffer);
		return 0;
	}
	else if ( !strcmp(name, T("dmzEnabled"))) {
		if ( !mib_get( MIB_DMZ_ENABLE, (void *)&vChar) )
			return -1;
		sprintf(buffer, "%u", vChar);
		ejSetResult(eid, buffer);
		return 0;
	}
	else if ( !strcmp(name, T("vc-auto"))) {
		if ( !mib_get( MIB_ATM_VC_AUTOSEARCH, (void *)&vChar) )
			return -1;
		sprintf(buffer, "%u", vChar);
		ejSetResult(eid, buffer);
		return 0;
	}
   	else if ( !strcmp(name, T("ippt-itf"))) {
		if( !mib_get( MIB_IPPT_ITF,  (void *)&vChar))
			return -1;
		sprintf(buffer, "%u", vChar);
		ejSetResult(eid, buffer);
		return 0;
	}
   	else if ( !strcmp(name, T("ippt-lanacc"))) {
		if( !mib_get( MIB_IPPT_LANACC,  (void *)&vChar))
			return -1;
		sprintf(buffer, "%u", vChar);
		ejSetResult(eid, buffer);
		return 0;
	}
	else if ( !strcmp(name, T("rconf-status"))) {
		sprintf(buffer, "%d", g_remoteConfig);
		ejSetResult(eid, buffer);
		return 0;
	}
	else if ( !strcmp(name, T("rconf-port"))) {
		sprintf(buffer, "%d", g_remoteAccessPort);
		ejSetResult(eid, buffer);
		return 0;
	}
   	else if ( !strcmp(name, T("spc-enable"))) {
		if( !mib_get( MIB_SPC_ENABLE,  (void *)&vChar))
			return -1;
		sprintf(buffer, "%u", vChar);
		ejSetResult(eid, buffer);
		return 0;
	}
   	else if ( !strcmp(name, T("spc-iptype"))) {
		if( !mib_get( MIB_SPC_IPTYPE,  (void *)&vChar))
			return -1;
		sprintf(buffer, "%u", vChar);
		ejSetResult(eid, buffer);
		return 0;
	}
	else if ( !strcmp(name, T("ipPortFilterNum"))) {
		vUInt = mib_chain_total(MIB_IP_PORT_FILTER_TBL);
		sprintf(buffer, "%u", vUInt);
		ejSetResult(eid, buffer);
		return 0;
	}
	else if ( !strcmp(name, T("macFilterNum"))) {
		vUInt = mib_chain_total(MIB_MAC_FILTER_TBL);
		sprintf(buffer, "%u", vUInt);
		ejSetResult(eid, buffer);
		return 0;
	}
	else if ( !strcmp(name, T("portFwNum"))) {
		vUInt = mib_chain_total(MIB_PORT_FW_TBL);
		sprintf(buffer, "%u", vUInt);
		ejSetResult(eid, buffer);
		return 0;
	}
	else if ( !strcmp(name, T("atmVcNum"))) {
		vUInt = mib_chain_total(MIB_ATM_VC_TBL);
		sprintf(buffer, "%u", vUInt);
		ejSetResult(eid, buffer);
		return 0;
	}
   	else if ( !strcmp(name, T("wan-pppoeConnectStatus")) ) {
////	check the pppoe status
		sprintf(buffer, "%d", 0);
		ejSetResult(eid, buffer);
		return 0;
	}
#ifdef WLAN_SUPPORT
	else if ( !strcmp(name, T("channel")) ) {
		if ( !mib_get( MIB_WLAN_CHAN_NUM, (void *)&vChar) )
			return -1;
		sprintf(buffer, "%d", vChar);
		ejSetResult(eid, buffer);
		return 0;
	}
	else if ( !strcmp(name, T("regDomain")) ) {
		if ( !mib_get( MIB_HW_REG_DOMAIN, (void *)&vChar) )
			return -1;
		sprintf(buffer, "%u", vChar);
		ejSetResult(eid, buffer);
		return 0;
	}
	else if ( !strcmp(name, T("wep")) ) {
		if ( !mib_get( MIB_WLAN_WEP, (void *)&vChar) )
			return -1;
		sprintf(buffer, "%u", vChar);
		ejSetResult(eid, buffer);
   	    	return 0;
	}
	else if ( !strcmp(name, T("defaultKeyId")) ) {
		if ( !mib_get( MIB_WLAN_WEP_DEFAULT_KEY, (void *)&vChar) )
			return -1;
		vChar++;
		sprintf(buffer, "%u", vChar) ;
		ejSetResult(eid, buffer);
		return 0;
	}
	else if ( !strcmp(name, T("keyType")) ) {
		if ( !mib_get( MIB_WLAN_WEP_KEY_TYPE, (void *)&vChar) )
			return -1;
		sprintf(buffer, "%u", vChar) ;
		ejSetResult(eid, buffer);
		return 0;
	}
  	else if ( !strcmp(name, T("authType"))) {
		if ( !mib_get( MIB_WLAN_AUTH_TYPE, (void *)&vChar) )
			return -1;
		sprintf(buffer, "%u", vChar) ;
		ejSetResult(eid, buffer);
		return 0;
	}
	else if ( !strcmp(name, T("operRate"))) {
		if ( !mib_get( MIB_WLAN_SUPPORTED_RATE, (void *)&vUShort) )
			return -1;
		sprintf(buffer, "%u", vUShort);
		ejSetResult(eid, buffer);
		return 0;
	}
	else if ( !strcmp(name, T("basicRate"))) {
		if ( !mib_get( MIB_WLAN_BASIC_RATE, (void *)&vUShort) )
			return -1;
		sprintf(buffer, "%u", vUShort);
		ejSetResult(eid, buffer);
		return 0;
	}
	else if ( !strcmp(name, T("preamble"))) {
		if ( !mib_get( MIB_WLAN_PREAMBLE_TYPE, (void *)&vChar) )
			return -1;
		sprintf(buffer, "%u", vChar);
		ejSetResult(eid, buffer);
		return 0;
	}
	else if ( !strcmp(name, T("hiddenSSID"))) {
		if ( !mib_get( MIB_WLAN_HIDDEN_SSID, (void *)&vChar) )
			return -1;
		sprintf(buffer, "%u", vChar);
		ejSetResult(eid, buffer);
		return 0;
	}
	else if ( !strcmp(name, T("wmFilterNum"))) {
		if ( !mib_get( MIB_WLAN_AC_NUM, (void *)&vChar) )
			return -1;
		sprintf(buffer, "%u", vChar);
		ejSetResult(eid, buffer);
		return 0;
	}
	else if ( !strcmp(name, T("wlanDisabled"))) {
		if ( !mib_get( MIB_WLAN_DISABLED, (void *)&vChar) )
			return -1;
		sprintf(buffer, "%u", vChar);
		ejSetResult(eid, buffer);
		return 0;
	}
	else if ( !strcmp(name, T("wlanAcNum")) ) {
		if ( !mib_get( MIB_WLAN_AC_NUM, (void *)&vChar) )
			return -1;
		sprintf(buffer, "%u", vChar);
		ejSetResult(eid, buffer);
		return 0;
	}
	else if ( !strcmp(name, T("wlanAcEnabled"))) {
		if ( !mib_get( MIB_WLAN_AC_ENABLED, (void *)&vChar) )
			return -1;
		sprintf(buffer, "%u", vChar);
		ejSetResult(eid, buffer);
		return 0;
	}
	else if ( !strcmp(name, T("rateAdaptiveEnabled"))) {
		if ( !mib_get( MIB_WLAN_RATE_ADAPTIVE_ENABLED, (void *)&vChar) )
			return -1;
		sprintf(buffer, "%u", vChar);
		ejSetResult(eid, buffer);
		return 0;
	}
	else if ( !strcmp(name, T("wlanMode"))) {
		if ( !mib_get( MIB_WLAN_MODE, (void *)&vChar) )
			return -1;
		sprintf(buffer, "%u", vChar);
		ejSetResult(eid, buffer);
		return 0;
	}
	else if ( !strcmp(name, T("networkType"))) {
		if ( !mib_get( MIB_WLAN_NETWORK_TYPE, (void *)&vChar) )
			return -1;
		sprintf(buffer, "%u", vChar);
		ejSetResult(eid, buffer);
		return 0;
	}
	else if ( !strcmp(name, T("iappDisabled"))) {
		if ( !mib_get( MIB_WLAN_IAPP_DISABLED, (void *)&vChar) )
			return -1;
		sprintf(buffer, "%u", vChar);
		ejSetResult(eid, buffer);
		return 0;
	}
#ifdef WLAN_WPA
	else if ( !strcmp(name, T("encrypt"))) {
		if ( !mib_get( MIB_WLAN_ENCRYPT, (void *)&vChar) )
			return -1;
		sprintf(buffer, "%u", vChar);
		ejSetResult(eid, buffer);
		return 0;
	}
	else if ( !strcmp(name, T("enable1X"))) {
		if ( !mib_get( MIB_WLAN_ENABLE_1X, (void *)&vChar) )
			return -1;
		sprintf(buffer, "%u", vChar);
		ejSetResult(eid, buffer);
		return 0;
	}
	else if ( !strcmp(name, T("enableSuppNonWpa"))) {
		if ( !mib_get( MIB_WLAN_ENABLE_SUPP_NONWPA, (void *)&vChar) )
			return -1;
		sprintf(buffer, "%u", vChar);
		ejSetResult(eid, buffer);
		return 0;
	}
	else if ( !strcmp(name, T("suppNonWpa"))) {
		if ( !mib_get( MIB_WLAN_SUPP_NONWPA, (void *)&vChar) )
			return -1;
		sprintf(buffer, "%u", vChar);
		ejSetResult(eid, buffer);
		return 0;
	}
	else if ( !strcmp(name, T("wpaAuth"))) {
		if ( !mib_get( MIB_WLAN_WPA_AUTH, (void *)&vChar) )
			return -1;
		sprintf(buffer, "%u", vChar);
		ejSetResult(eid, buffer);
		return 0;
	}
	else if ( !strcmp(name, T("wpaCipher"))) {
		if ( !mib_get( MIB_WLAN_WPA_CIPHER_SUITE, (void *)&vChar) )
			return -1;
		sprintf(buffer, "%u", vChar);
		ejSetResult(eid, buffer);
		return 0;
	}
	else if ( !strcmp(name, T("pskFormat"))) {
		if ( !mib_get( MIB_WLAN_WPA_PSK_FORMAT, (void *)&vChar) )
			return -1;
		sprintf(buffer, "%u", vChar);
		ejSetResult(eid, buffer);
		return 0;
	}
	else if ( !strcmp(name, T("enableMacAuth"))) {
		if ( !mib_get( MIB_WLAN_ENABLE_MAC_AUTH, (void *)&vChar) )
			return -1;
		sprintf(buffer, "%u", vChar);
		ejSetResult(eid, buffer);
		return 0;
	}
	else if ( !strcmp(name, T("rsRetry")) ) {
		if ( !mib_get( MIB_WLAN_RS_RETRY, (void *)&vChar) )
			return -1;
		sprintf(buffer, "%u", vChar);
		ejSetResult(eid, buffer);
		return 0;
	}
#endif

#ifdef WLAN_WDS
	else if ( !strcmp(name, T("wlanWdsEnabled"))) {
		if ( !mib_get( MIB_WLAN_WDS_ENABLED, (void *)&vChar) )
			return -1;
		sprintf(buffer, "%u", vChar);
		ejSetResult(eid, buffer);
		return 0;
	}
	else if ( !strcmp(name, T("wlanWdsNum"))) {
		if ( !mib_get( MIB_WLAN_WDS_NUM, (void *)&vChar) )
			return -1;
		sprintf(buffer, "%u", vChar);
		ejSetResult(eid, buffer);
		return 0;
	}
	else if ( !strcmp(name, T("wdsWep"))) {
		if ( !mib_get( MIB_WLAN_WDS_WEP, (void *)&vChar) )
			return -1;
		sprintf(buffer, "%u", vChar);
		ejSetResult(eid, buffer);
		return 0;
	}
	else if ( !strcmp(name, T("wdsDefaultKeyId"))) {
		if ( !mib_get( MIB_WLAN_WDS_WEP_DEFAULT_KEY, (void *)&vChar) )
			return -1;
		sprintf(buffer, "%u", ++vChar);
		ejSetResult(eid, buffer);
		return 0;
	}
	else if ( !strcmp(name, T("wdsKeyType")) ) {
		if ( !mib_get( MIB_WLAN_WDS_WEP_KEY_TYPE, (void *)&vChar) )
			return -1;
		sprintf(buffer, "%u", vChar) ;
		 ejSetResult(eid, buffer);
		return 0;
	}
#endif

#ifdef WLAN_8185AG
	else if ( !strcmp(name, T("RFType")) ) {
		if ( !mib_get( MIB_HW_RF_TYPE, (void *)&vChar) )
			return -1;
		sprintf(buffer, "%u", vChar) ;
		 ejSetResult(eid, buffer);
		return 0;
	}
	else if ( !strcmp(name, T("band")) ) {
		if ( !mib_get( MIB_WLAN_BAND, (void *)&vChar) )
			return -1;
		sprintf(buffer, "%u", (int)vChar) ;
		 ejSetResult(eid, buffer);
		return 0;
	}
	else if ( !strcmp(name, T("fixTxRate")) ) {
		if ( !mib_get( MIB_WLAN_FIX_RATE, (void *)&vUShort) )
			return -1;
		sprintf(buffer, "%u", vUShort) ;
		 ejSetResult(eid, buffer);
		return 0;
	}	
#endif

#endif // of WLAN_SUPPORT

	sprintf(buffer, "%d", -1);
	ejSetResult(eid, buffer);
	return 0;

//   	return -1;
}
#endif

int isConnectPPP(void)
{
	return 0;
}

#ifdef CONFIG_EXT_SWITCH
int show_lanport(int eid, webs_t wp, int argc, char_t **argv)
{
	char_t *name;
	int vport;
	
	if (ejArgs(argc, argv, T("%s"), &name) < 1) {
		websError(wp, 400, T("Insufficient args\n"));
		return -1;
	}

	vport = name[0]-'0';
	websWrite(wp,"LAN%d;\n", virt2user[vport]);
	return 0;
}
#endif


int getNameServer(int eid, webs_t wp, int argc, char_t **argv) {

	FILE *fp;
	char buffer[128], tmpbuf[64];
	int count = 0;
	//fprintf(stderr, "getNameServer %x\n", gResolvFile);
	//websWrite(wp, "[]", tmpbuf);
	//if ((gResolvFile == NULL) || 
	if ( (fp = fopen("/var/resolv.conf", "r")) == NULL ) {
		//printf("Unable to open resolver file\n");
		return -1;
	}

	while (fgets(buffer, sizeof(buffer), fp) != NULL) {
		if (sscanf(buffer, "nameserver %s", tmpbuf) != 1) {
			continue;
		}

		if (count == 0)
			websWrite(wp, "%s", tmpbuf);
		else
			websWrite(wp, ", %s", tmpbuf);
		count ++;
	}
	
	fclose(fp);
	return 0;
}

#ifndef RTF_UP
/* Keep this in sync with /usr/src/linux/include/linux/route.h */
#define RTF_UP          0x0001	/* route usable                 */
#define RTF_GATEWAY     0x0002	/* destination is a gateway     */
#define RTF_HOST        0x0004	/* host entry (net otherwise)   */
#define RTF_REINSTATE   0x0008	/* reinstate route after tmout  */
#define RTF_DYNAMIC     0x0010	/* created dyn. (by redirect)   */
#define RTF_MODIFIED    0x0020	/* modified dyn. (by redirect)  */
#define RTF_MTU         0x0040	/* specific MTU for this route  */
#ifndef RTF_MSS
#define RTF_MSS         RTF_MTU	/* Compatibility :-(            */
#endif
#define RTF_WINDOW      0x0080	/* per route window clamping    */
#define RTF_IRTT        0x0100	/* Initial round trip time      */
#define RTF_REJECT      0x0200	/* Reject route                 */
#endif

// Jenny, get default gateway information
int getDefaultGW(int eid, webs_t wp, int argc, char_t **argv)
{
	char buff[256];
	int flgs;
	//unsigned long int g;
	struct in_addr gw, dest, mask;
	char ifname[16], dgw[16];
	FILE *fp;
	
	if (!(fp=fopen("/proc/net/route", "r"))) {
		printf("Error: cannot open /proc/net/route - continuing...\n");
		return -1;
	}

	fgets(buff, sizeof(buff), fp);
	while (fgets(buff, sizeof(buff), fp) != NULL) {
		//if (sscanf(buff, "%*s%*lx%lx%X%", &g, &flgs) != 2) {
		if (sscanf(buff, "%s%x%x%x%*d%*d%*d%x", ifname, &dest, &gw, &flgs, &mask) != 5) {
			printf("Unsuported kernel route format\n");
			fclose(fp);
			return -1;
		}
		
		//printf("ifname=%s, dest=%x, gw=%x, flgs=%x, mask=%x\n", ifname, dest.s_addr, gw.s_addr, flgs, mask.s_addr);
		if(flgs & RTF_UP) {
			// default gateway
			if (dest.s_addr == 0 && mask.s_addr == 0) {
				if (gw.s_addr != 0) {
					strncpy(dgw,  inet_ntoa(gw), 16);
					websWrite(wp, "%s", dgw);
					fclose(fp);
					return 0;
				}
				else {
					websWrite(wp, "%s", ifname);
					fclose(fp);
					return 0;
				}
			}
					
		}
		/*
		if(flgs & RTF_UP) {
			gw.s_addr = g;
			if (gw.s_addr != 0) {
				strcpy(dgw,  inet_ntoa(gw));
				websWrite(wp, "%s", dgw);
				fclose(fp);
				return 0;
			}
		}
		*/
	}
	fclose(fp);
	return 0;
}

