/*
 *      Web server handler routines for IP QoS
 *
 */

/*-- System inlcude files --*/
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <sys/wait.h>

#include "../webs.h"
#include "webform.h"
#include "mib.h"
#include "utility.h"

#ifdef EMBED
#include <linux/config.h>
#else
#include "../../../uClibc/include/linux/autoconf.h"
#endif

#ifdef IP_QOS

/////////////////////////////////////////////////////////////////////////////
void formQos(webs_t wp, char_t *path, char_t *query)
{
	char_t *submitUrl, *strAddQos, *strDelQos, *strVal, *strDelAllQos;
	char_t *strSAddr, *strDAddr, *strSport, *strDport;
	char tmpBuf[100];
	int intVal;
	unsigned int entryNum, totalEntry, i;
	MIB_CE_IP_QOS_T entry;
	struct in_addr curIpAddr, curSubnet;
	unsigned long v1, v2, v3, mask, mbit;
	unsigned char vChar;
	int itfid, itfdomain;
#ifndef NO_ACTION
	int pid;
#endif
	strVal = websGetVar(wp, T("admin"), T(""));
	
	// enable/disable Qos
	if (strVal[0]) {
		// bitmap for virtual lan port function
		// Port Mapping: bit-0
		// QoS : bit-1
		mib_get(MIB_MPMODE, (void *)&vChar);
		strVal = websGetVar(wp, T("qosen"), T(""));
		if ( strVal[0] == '1' )
			vChar |= 0x02;
		else
			vChar &= 0xfd;
		mib_set(MIB_MPMODE, (void *)&vChar);
#ifndef IP_QOS_VPORT
		if ( strVal[0] == '1' ) {
			strVal = websGetVar(wp, T("qosdmn"), T(""));
			vChar = strVal[0] - '0';
			mib_set(MIB_QOS_DOMAIN, (void *)&vChar);
		}
#endif
		submitUrl = websGetVar(wp, T("submit-url"), T(""));
		OK_MSG(submitUrl);
	  	return;
	}
	
	strAddQos = websGetVar(wp, T("addqos"), T(""));
	strDelQos = websGetVar(wp, T("delSel"), T(""));
	strDelAllQos = websGetVar(wp, T("delAll"), T(""));
	
	memset(&entry, 0, sizeof(entry));
	totalEntry = mib_chain_total(MIB_IP_QOS_TBL); /* get chain record size */
	
	/* Add new qos entry */
	if (strAddQos[0]) {
		if (totalEntry >= MAX_QOS_RULE)
		{
			strcpy(tmpBuf, T(Texceed_max_rules));
			goto setErr_qos;
		}
		
#ifdef _CWMP_MIB_
		// enable
		entry.enable = 1;
#endif
		
		// protocol type
		strVal = websGetVar(wp, T("prot"), T(""));
		entry.protoType = strVal[0] - '0';

		strSAddr = websGetVar(wp, T("sip"), T(""));
		strDAddr = websGetVar(wp, T("dip"), T(""));
		
		// Source address
		if (strSAddr[0]) {
			inet_aton(strSAddr, (struct in_addr *)&entry.sip);
			if ((entry.sip[0]=='\0') &&(entry.sip[1]=='\0') && (entry.sip[2]=='\0') && (entry.sip[3]=='\0')) {
				strcpy(tmpBuf, T(Tinvalid_source_ip));
				goto setErr_qos;
			}
			
			strSAddr = websGetVar(wp, T("smask"), T(""));
			if (strSAddr[0]) {
				inet_aton(strSAddr, (struct in_addr *)&mask);
				if (mask==0) {
					strcpy(tmpBuf, T(Tinvalid_source_netmask));
					goto setErr_qos;
				}
				mask = htonl(mask);
				mbit=0; intVal=0;
				for (i=0; i<32; i++) {
					if (mask&0x80000000) {
						if (intVal) {
							strcpy(tmpBuf, T(Tinvalid_source_netmask));
							goto setErr_qos;
						}
						mbit++;
					}
					else
						intVal=1;
					mask <<= 1;
				}
				entry.smaskbit = mbit;
			}
			else
				entry.smaskbit = 32;
		}
		
		// source port
		if (entry.protoType == PROTO_TCP || entry.protoType == PROTO_UDP) {
			strSport = websGetVar(wp, T("sport"), T(""));
			
			if (strSport[0]) {
				if ( !string_to_dec(strSport, &intVal) || intVal<1 || intVal>65535) {
					strcpy(tmpBuf, T(Tinvalid_source_port));
					goto setErr_qos;
				}
				entry.sPort = (unsigned short)intVal;
			}
		}

		// Destination address
		if (strDAddr[0]) {
			inet_aton(strDAddr, (struct in_addr *)&entry.dip);
			if ((entry.dip[0]=='\0') && (entry.dip[1]=='\0') && (entry.dip[2]=='\0') && (entry.dip[3]=='\0')) {
				strcpy(tmpBuf, T(Tinvalid_destination_ip));
				goto setErr_qos;
			}
			
			strDAddr = websGetVar(wp, T("dmask"), T(""));
			if (strDAddr[0]) {
				inet_aton(strDAddr, (struct in_addr *)&mask);
				if (mask==0) {
					strcpy(tmpBuf, T(Tinvalid_destination_netmask));
					goto setErr_qos;
				}
				mask = htonl(mask);
				mbit=0; intVal=0;
				for (i=0; i<32; i++) {
					if (mask&0x80000000) {
						if (intVal) {
							strcpy(tmpBuf, T(Tinvalid_destination_netmask));
							goto setErr_qos;
						}
						mbit++;
					}
					else
						intVal=1;
					mask <<= 1;
				}
				entry.dmaskbit = mbit;
			}
			else
				entry.dmaskbit = 32;
		}
		
		// destination port
		if (entry.protoType == PROTO_TCP || entry.protoType == PROTO_UDP) {
			strDport = websGetVar(wp, T("dport"), T(""));
		
			if (strDport[0]) {
				if ( !string_to_dec(strDport, &intVal) || intVal<1 || intVal>65535) {
					strcpy(tmpBuf, T(Tinvalid_destination_port));
					goto setErr_qos;
				}
				entry.dPort = (unsigned short)intVal;
			}
		}
		
		// physical port
		strVal = websGetVar(wp, T("phyport"), T(""));
		entry.phyPort = 0xff;
		itfid = atoi(strVal);
		if (itfid != 0) {
			itfdomain = IF_DOMAIN(itfid);
			itfid = itfid&0x0ff;
			if (itfdomain == DOMAIN_ELAN)
				entry.phyPort = itfid;
#ifdef WLAN_SUPPORT
			else if (itfdomain == DOMAIN_WLAN)
#if defined(ZTE_GENERAL_ROUTER_SC) && defined(WLAN_MBSSID)
			{
				entry.phyPort=5+itfid;
			}
#else
				entry.phyPort = 5;	// wlan0
#endif				
#endif
		}
		
		// outbound interface
		strVal = websGetVar(wp, T("out_if"), T(""));
		if (strVal[0]) {
			entry.outif = (unsigned char)atoi(strVal);
		}
		
		/*
		 *	Assign/Mark priority
		 */
		// outbound priority
		strVal = websGetVar(wp, T("prio"), T(""));
		entry.prior = (unsigned char)(strVal[0]-'0');
		
		// IP precedence
		strVal = websGetVar(wp, T("ipprio"), T(""));
		entry.m_ipprio = (unsigned char)(strVal[0]-'0');
		
		// IP Type of Service
		strVal = websGetVar(wp, T("tos"), T(""));
		entry.m_iptos = atoi(strVal);
		
		// 802.1p
		strVal = websGetVar(wp, T("m1p"), T(""));
		entry.m_1p = (unsigned char)(strVal[0]-'0');
		
		// Check if this qos entry exists
		// ...

		if(mib_chain_add(MIB_IP_QOS_TBL, (unsigned char*)&entry) != 1){
			strcpy(tmpBuf, T(Tadd_chain_error));
			goto setErr_qos;
		}
	
	}




#ifdef ZTE_GENERAL_ROUTER_SC
		/* Delete entry */
	if (strDelQos[0])
	{
		unsigned int i;
		unsigned int idx;
		strVal=websGetVar(wp, T("select"), T("")); 
		if(strVal[0])
		{
			for (i=0; i<totalEntry; i++) {

				idx = totalEntry-i-1;			
				snprintf(tmpBuf, 20, "s%d", idx);
				if ( !gstrcmp(strVal, T(tmpBuf)) ) 
				{
					if(mib_chain_delete(MIB_IP_QOS_TBL, idx) != 1) {
						strcpy(tmpBuf, T(Tdelete_chain_error));
						goto setErr_qos;
					}
				}
			}
		}	
		goto setOk_qos;
	}
#else
	/* Delete entry */
	if (strDelQos[0])
	{
		unsigned int i;
		unsigned int idx;

		for (i=0; i<totalEntry; i++) {

			idx = totalEntry-i-1;			
			snprintf(tmpBuf, 20, "s%d", idx);
			strVal = websGetVar(wp, tmpBuf, T(""));
			
			if (strVal[0] == '1') {
				if(mib_chain_delete(MIB_IP_QOS_TBL, idx) != 1) {
					strcpy(tmpBuf, T(Tdelete_chain_error));
					goto setErr_qos;
				}
			}
		}
	
		goto setOk_qos;
	}
#endif //ZTE_GENERAL_ROUTER_SC
	/* Delete all entry */
	if ( strDelAllQos[0]) {
		mib_chain_clear(MIB_IP_QOS_TBL); /* clear chain record */
		goto setOk_qos;
	}

setOk_qos:
//	mib_update(CURRENT_SETTING);

#ifndef NO_ACTION
	pid = fork();
        if (pid) {
	      	waitpid(pid, NULL, 0);
	}
        else if (pid == 0) {
		snprintf(tmpBuf, 100, "%s/%s", _CONFIG_SCRIPT_PATH, _FIREWALL_SCRIPT_PROG);
		execl( tmpBuf, _FIREWALL_SCRIPT_PROG, NULL);
               	exit(1);
        }
#endif

	submitUrl = websGetVar(wp, T("submit-url"), T(""));   // hidden page
	if (submitUrl[0])
		websRedirect(wp, submitUrl);
	else
		websDone(wp, 200);
  	return;

setErr_qos:
	ERR_MSG(tmpBuf);
}
#endif	// of IP_QOS

// List all the available LAN side interface at web page.
// return: number of interface listed.
int iflanList(int eid, webs_t wp, int argc, char_t **argv)
{
	int nBytesSent=0;
	int ifnum=0;
	unsigned int i;
	char_t *name;
	struct itfInfo itfs[16];
	int type;
	
	if (ejArgs(argc, argv, T("%s"), &name) < 1) {
		websError(wp, 400, T("Insufficient args\n"));
		return -1;
	}
	
	type = 0;
	if ( !strcmp(name, T("all")) )
		type = (DOMAIN_ELAN|DOMAIN_WLAN);
	else if ( !strcmp(name, T("eth")) )
		type = DOMAIN_ELAN;	// eth interface
	else if ( !strcmp(name, T("wlan")) )
		type = DOMAIN_WLAN;	// wireless interface
	else
		type = DOMAIN_ELAN;	// default to eth
	
	ifnum = get_domain_ifinfo(itfs, 16, type);
	if (ifnum==0)
		return 0;
#ifdef ZTE_GENERAL_ROUTER_SC
	for (i=0; i<ifnum; i++) {
			nBytesSent += websWrite(wp, T("<option value=%u>%s</option>\n"),
			IF_ID(itfs[i].ifdomain, itfs[i].ifid), itfs[i].name);
	}
#else
	for (i=0; i<ifnum; i++) {
			nBytesSent += websWrite(wp, T("<option value=%u>%s</option>\n"),
			IF_ID(itfs[i].ifdomain, itfs[i].ifid), itfs[i].name);
	}
#endif	
	return nBytesSent;
}

#ifdef IP_QOS
/////////////////////////////////////////////////////////////////////////////
int default_qos(int eid, webs_t wp, int argc, char_t **argv)
{
	int nBytesSent=0;
#ifdef IP_QOS_VPORT
	return 0;
#else
	nBytesSent += websWrite(wp, T("<td><font size=2><b>Default QoS:</b></td>\n"));
	nBytesSent += websWrite(wp, T("<td><select name=qosdmn>\n"));
	nBytesSent += websWrite(wp, T("<option value=%d>IP Pred</option>"
			"<option value=%d>802.1p</option>\n</select>\n</td>\n"), PRIO_IP, PRIO_802_1p);
	return nBytesSent;
#endif
}

/////////////////////////////////////////////////////////////////////////////
int qosList(int eid, webs_t wp, int argc, char_t **argv)
{
	int nBytesSent=0;

	unsigned int entryNum, i, k;
	MIB_CE_IP_QOS_T Entry;
	const char *type;
	char	*psip, *pdip, sip[20], dip[20];
	char sport[10], dport[10];
	const char *pPrio, *pIPrio, *pTos, *p1p;
	unsigned int mask, smask, dmask;
	char strPhy[]="LAN0", *pPhy;
	char wanif[8];
	//char_t  buffer[3];

	entryNum = mib_chain_total(MIB_IP_QOS_TBL);
#ifdef ZTE_GENERAL_ROUTER_SC
	nBytesSent += websWrite(wp, T("<tr>"
	"<th bgcolor=\"#808080\" colspan=6><font size=\"2\">%s</font></th>\n"
	"<th bgcolor=\"#808080\" colspan=4><font size=\"2\">%s</font></th>\n"
	"<th bgcolor=\"#808080\" colspan=2><font size=\"2\"></font></th>\n"),
	Tqos_traffic_rule,Tqos_mark);
	nBytesSent += websWrite(wp, T("<tr>"
      	"<td align=center width=\"25%%\" bgcolor=\"#808080\"><font size=\"2\"><b>%s</b></font></td>\n"
      	"<td align=center width=\"20%%\" bgcolor=\"#808080\"><font size=\"2\"><b>%s</b></font></td>\n"
      	"<td align=center width=\"25%%\" bgcolor=\"#808080\"><font size=\"2\"><b>%s</b></font></td>\n"
      	"<td align=center width=\"20%%\" bgcolor=\"#808080\"><font size=\"2\"><b>%s</b></font></td>\n"
      	"<td align=center width=\"20%%\" bgcolor=\"#808080\"><font size=\"2\"><b>%s</b></font></td>\n"
      	"<td align=center width=\"20%%\" bgcolor=\"#808080\"><font size=\"2\"><b>%s</b></font></td>\n"),
      	Tqos_sIP,Tqos_sPort,Tqos_dIP,Tqos_dPort,Tqos_pro,Tqos_if);
      	nBytesSent += websWrite(wp, T(
      	"<td align=center width=\"20%%\" bgcolor=\"#808080\"><font size=\"2\"><b>%s</b></font></td>\n"
      	"<td align=center width=\"20%%\" bgcolor=\"#808080\"><font size=\"2\"><b>%s</b></font></td>\n"
      	"<td align=center width=\"20%%\" bgcolor=\"#808080\"><font size=\"2\"><b>%s</b></font></td>\n"
      	"<td align=center width=\"20%%\" bgcolor=\"#808080\"><font size=\"2\"><b>%s</b></font></td>\n"
#ifdef IP_POLICY_ROUTING
      	"<td align=center width=\"20%%\" bgcolor=\"#808080\"><font size=\"2\"><b>%s</b></font></td>\n"
#endif
#ifdef _CWMP_MIB_
      	"<td align=center width=\"20%%\" bgcolor=\"#808080\"><font size=\"2\"><b>%s</b></font></td>\n"
#endif
      	"<td align=center width=\"15%%\" bgcolor=\"#808080\"><font size=\"2\"><b>%s</b></font></td></tr>\n"),
      	Tqos_level,Tqos_precd,Tqos_tos,Tqos_802_1q,
#ifdef IP_POLICY_ROUTING      	
      	Tqos_wan_if,
#endif
#ifdef _CWMP_MIB_
      	INFO_ENABLED,
#endif
	Tselect);
#else //ZTE_GENERAL_ROUTER_SC
	nBytesSent += websWrite(wp, T("<tr>"
	"<th bgcolor=\"#808080\" colspan=6><font size=\"2\">%s</font></th>\n"
	"<th bgcolor=\"#808080\" colspan=4><font size=\"2\">%s</font></th>\n"
	"<th bgcolor=\"#808080\" colspan=2><font size=\"2\"></font></th>\n"),
	Tqos_traffic_rule,Tqos_mark);
	nBytesSent += websWrite(wp, T("<tr>"
      	"<td align=center width=\"25%%\" bgcolor=\"#808080\"><font size=\"2\"><b>%s</b></font></td>\n"
      	"<td align=center width=\"20%%\" bgcolor=\"#808080\"><font size=\"2\"><b>%s</b></font></td>\n"
      	"<td align=center width=\"25%%\" bgcolor=\"#808080\"><font size=\"2\"><b>%s</b></font></td>\n"
      	"<td align=center width=\"20%%\" bgcolor=\"#808080\"><font size=\"2\"><b>%s</b></font></td>\n"
      	"<td align=center width=\"20%%\" bgcolor=\"#808080\"><font size=\"2\"><b>%s</b></font></td>\n"
      	"<td align=center width=\"20%%\" bgcolor=\"#808080\"><font size=\"2\"><b>%s</b></font></td>\n"),
      	Tqos_sIP,Tqos_sPort,Tqos_dIP,Tqos_dPort,Tqos_pro,Tqos_if);
      	nBytesSent += websWrite(wp, T(
      	"<td align=center width=\"20%%\" bgcolor=\"#808080\"><font size=\"2\"><b>%s</b></font></td>\n"
      	"<td align=center width=\"20%%\" bgcolor=\"#808080\"><font size=\"2\"><b>%s</b></font></td>\n"
      	"<td align=center width=\"20%%\" bgcolor=\"#808080\"><font size=\"2\"><b>%s</b></font></td>\n"
      	"<td align=center width=\"20%%\" bgcolor=\"#808080\"><font size=\"2\"><b>%s</b></font></td>\n"
#ifdef IP_POLICY_ROUTING
      	"<td align=center width=\"20%%\" bgcolor=\"#808080\"><font size=\"2\"><b>%s</b></font></td>\n"
#endif
#ifdef _CWMP_MIB_
      	"<td align=center width=\"20%%\" bgcolor=\"#808080\"><font size=\"2\"><b>%s</b></font></td>\n"
#endif
      	"<td align=center width=\"15%%\" bgcolor=\"#808080\"><font size=\"2\"><b>%s</b></font></td></tr>\n"),
      	Tqos_level,Tqos_precd,Tqos_tos,Tqos_802_1q,
#ifdef IP_POLICY_ROUTING      	
      	Tqos_wan_if,
#endif
#ifdef _CWMP_MIB_
      	INFO_ENABLED,
#endif
	Tselect);

#endif //ZTE_GENERAL_ROUTER_SC
	for (i=0; i<entryNum; i++) {

		if (!mib_chain_get(MIB_IP_QOS_TBL, i, (void *)&Entry))
		{
  			websError(wp, 400, T("Get chain record error!\n"));
			return -1;
		}

		// source ip
		psip = inet_ntoa(*((struct in_addr *)Entry.sip));
		if ( !strcmp(psip, "0.0.0.0"))
			psip = (char *)BLANK;
		else {
			if (Entry.smaskbit==0)
				snprintf(sip, 20, "%s", psip);
			else
				snprintf(sip, 20, "%s/%d", psip, Entry.smaskbit);
			psip = sip;
		}
		//snprintf(sip, 20, "%s/%d", inet_ntoa(*((struct in_addr *)Entry.sip)), Entry.smaskbit);
		// source port
		if (Entry.sPort == 0)
			strcpy(sport, BLANK);
		else
			snprintf(sport, 10, "%d", Entry.sPort);
		
		// destination ip
		pdip = inet_ntoa(*((struct in_addr *)Entry.dip));
		if ( !strcmp(pdip, "0.0.0.0"))
			pdip = (char *)BLANK;
		else {
			if (Entry.dmaskbit==0)
				snprintf(dip, 20, "%s", pdip);
			else
				snprintf(dip, 20, "%s/%d", pdip, Entry.dmaskbit);
			pdip = dip;
		}
		//snprintf(dip, 20, "%s/%d", inet_ntoa(*((struct in_addr *)Entry.dip)), Entry.dmaskbit);
		// destination port
		if (Entry.dPort == 0)
			strcpy(dport, BLANK);
		else
			snprintf(dport, 10, "%d", Entry.dPort);
		
		
		if ( Entry.protoType == PROTO_TCP )
			type = ARG_TCP;
		else if ( Entry.protoType == PROTO_UDP )
			type = ARG_UDP;
		else if ( Entry.protoType == PROTO_ICMP )
			type = ARG_ICMP;
		else
			type = BLANK;
		
		// Assign Priority
		if (Entry.prior <= (IPQOS_NUM_PRIOQ-1))
			pPrio = prioLevel[Entry.prior];
		else
			// should not be here !!
			pPrio = prioLevel[2];
		
		if (Entry.m_ipprio == 0)
			pIPrio = BLANK;
		else
			pIPrio = n0to7[Entry.m_ipprio];
		
		if (Entry.m_iptos == 0xff)
			pTos = BLANK;
		else {
			int mask, i;
			
			mask = i = 1;
			while (i<=5) {
				if (Entry.m_iptos & mask)
					break;
				else {
					i++;
					mask<<=1;
				}
			}
			if (i>=6)
				i = 1;
			pTos = ipTos[i];
		}
		
		if (Entry.m_1p == 0)
			p1p = BLANK;
		else
			p1p = n0to7[Entry.m_1p];
		
		pPhy = strPhy;
		if (Entry.phyPort == 0xff)
			pPhy = (char *)BLANK;
#if (defined(CONFIG_EXT_SWITCH)  && defined(IP_QOS_VPORT))
//#ifdef CONFIG_EXT_SWITCH
		else if (Entry.phyPort < SW_PORT_NUM)
			strPhy[3] = '0' + virt2user[Entry.phyPort];
#else
		else if (Entry.phyPort == 0)
			strPhy[3] = '0';
#endif
#ifdef WLAN_SUPPORT
		else
#ifdef ZTE_GENERAL_ROUTER_SC
		{
			if(Entry.phyPort==5) //wlan0
				pPhy = (char *)WLANIF;
			else 	// 6,7,8,9 vap0,vap1 vap2 vap3
			{
				strncpy(strPhy, "vap0", 4);
				strPhy[3]='0'+ (Entry.phyPort-6);
			}
		}
#else
			pPhy = (char *)WLANIF;
#endif //ZTE_GENERAL_ROUTER_SC
		
#endif
		
		nBytesSent += websWrite(wp, T("<tr>"
			"<td align=center width=\"25%%\" bgcolor=\"#C0C0C0\"><font size=\"2\">%s</td>\n"
      			"<td align=center width=\"20%%\" bgcolor=\"#C0C0C0\"><font size=\"2\">%s</td>\n"
			"<td align=center width=\"25%%\" bgcolor=\"#C0C0C0\"><font size=\"2\">%s</td>\n"
      			"<td align=center width=\"20%%\" bgcolor=\"#C0C0C0\"><font size=\"2\">%s</td>\n"
      			"<td align=center width=\"20%%\" bgcolor=\"#C0C0C0\"><font size=\"2\">%s</td>\n"
     			"<td align=center width=\"20%%\" bgcolor=\"#C0C0C0\"><font size=\"2\">%s</td>\n"
     			"<td align=center width=\"20%%\" bgcolor=\"#C0C0C0\"><font size=\"2\">%s</td>\n"
     			"<td align=center width=\"20%%\" bgcolor=\"#C0C0C0\"><font size=\"2\">%s</td>\n"
     			"<td align=center width=\"20%%\" bgcolor=\"#C0C0C0\"><font size=\"2\">%s</td>\n"
     			"<td align=center width=\"20%%\" bgcolor=\"#C0C0C0\"><font size=\"2\">%s</td>\n"),
			psip, sport, pdip, dport, type, pPhy, pPrio, pIPrio, pTos, p1p);
		
#ifdef IP_POLICY_ROUTING
		if (Entry.outif == 0xff) {
			wanif[0] = ' ';
			wanif[1] = 0;
		}
		else {
			if (PPP_INDEX(Entry.outif) != 0x0f)
			{	// PPP interface
				snprintf(wanif, 8, "ppp%u", PPP_INDEX(Entry.outif));
     			}
     			else
			{	// vc interface
				snprintf(wanif, 8, "vc%u", VC_INDEX(Entry.outif));
     			}
     		}
     		
		nBytesSent += websWrite(wp, T(
			"<td align=center width=\"20%%\" bgcolor=\"#C0C0C0\"><font size=\"2\">%s</td>\n"),
			 wanif);
#endif
#ifdef _CWMP_MIB_
		nBytesSent += websWrite(wp, T(
			"<td align=center width=\"20%%\" bgcolor=\"#C0C0C0\"><font size=\"2\">%s</td>\n"),
			 (Entry.enable==0)?"Disable":"Enable" );
#endif
#ifndef ZTE_GENERAL_ROUTER_SC
		nBytesSent += websWrite(wp, T(
      			"<td align=center width=\"15%%\" bgcolor=\"#C0C0C0\"><input type=\"checkbox\" name=\"s%d\" value=1></td></tr>\n"),
      			i);
#else
      		nBytesSent += websWrite(wp, T(
      			"<td align=center width=\"15%%\" bgcolor=\"#C0C0C0\"><input type=\"radio\" name=\"select\" value=\"s%d\"></td></tr>\n"),
      			i);
#endif
      			
	}
#ifdef ZTE_GENERAL_ROUTER_SC
	nBytesSent += websWrite(wp, T("<input type=\"hidden\" value=%d name=\"listnumber\">"), i);
#endif
	//snprintf(buffer, 3, "%u", entryNum);
	//ejSetResult(eid, buffer);
	return nBytesSent;
}
#endif	// of IP_QOS

int policy_route_outif(int eid, webs_t wp, int argc, char_t **argv)
{
	int nBytesSent=0;
	unsigned int entryNum, i;
	MIB_CE_ATM_VC_T Entry;
	
#ifdef IP_POLICY_ROUTING
	#ifdef ZTE_GENERAL_ROUTER_SC
	nBytesSent += websWrite(wp, T(" <tr>\n"));
	nBytesSent += websWrite(wp, T(" \t<td align=left><font size=2><b>Outbound Iterface:</b></td>\n"));
	nBytesSent += websWrite(wp, T("  \t<td align=left>\n\t\t<select name=out_if>\n"));
	nBytesSent += websWrite(wp, T(" \t\t<option value=255></option>\n"));
	entryNum = mib_chain_total(MIB_ATM_VC_TBL);
	
	for (i=0; i<entryNum; i++) {
		if (!mib_chain_get(MIB_ATM_VC_TBL, i, (void *)&Entry))
		{
  			websError(wp, 400, T("Get chain record error!\n"));
			return -1;
		}
		
		if (Entry.enable == 0)
			continue;
		
		if (Entry.cmode == ADSL_BR1483)
			continue;
		
		if (PPP_INDEX(Entry.ifIndex) != 0x0f)
		{	// PPP interface
			nBytesSent += websWrite(wp, T(" \t\t<option value=%u>ppp%u</option>\n"),
				Entry.ifIndex, PPP_INDEX(Entry.ifIndex));
		}
		else
		{	// vc interface
			nBytesSent += websWrite(wp, T(" \t\t<option value=%u>vc%u</option>\n"),
				Entry.ifIndex, VC_INDEX(Entry.ifIndex));
		}
	}
	
	nBytesSent += websWrite(wp, T("\t\t</select>\n\t</td>\n</tr>\n"));	
	#else
	nBytesSent += websWrite(wp, T("<tr>\n"));
	nBytesSent += websWrite(wp, T("\t<td align=left><font size=2><b>Outbound Iterface:</b></td>\n"));
	nBytesSent += websWrite(wp, T("\t<td align=left>\n\t\t<select name=out_if>\n"));
	nBytesSent += websWrite(wp, T("\t\t<option value=255></option>\n"));
	entryNum = mib_chain_total(MIB_ATM_VC_TBL);
	
	for (i=0; i<entryNum; i++) {
		if (!mib_chain_get(MIB_ATM_VC_TBL, i, (void *)&Entry))
		{
  			websError(wp, 400, T("Get chain record error!\n"));
			return -1;
		}
		
		if (Entry.enable == 0)
			continue;
		
		if (Entry.cmode == ADSL_BR1483)
			continue;
		
		if (PPP_INDEX(Entry.ifIndex) != 0x0f)
		{	// PPP interface
			nBytesSent += websWrite(wp, T("\t\t<option value=%u>ppp%u</option>\n"),
				Entry.ifIndex, PPP_INDEX(Entry.ifIndex));
		}
		else
		{	// vc interface
			nBytesSent += websWrite(wp, T("\t\t<option value=%u>vc%u</option>\n"),
				Entry.ifIndex, VC_INDEX(Entry.ifIndex));
		}
	}
	
	nBytesSent += websWrite(wp, T("\t\t</select>\n\t</td>\n</tr>\n"));
	#endif
#endif
	return nBytesSent;
}

#ifdef IP_QOS
int priority_outif(int eid, webs_t wp, int argc, char_t **argv)
{
	int nBytesSent=0, i;
	const char *pPrio;
#ifdef ZTE_GENERAL_ROUTER_SC
	nBytesSent += websWrite(wp, T("  \t<select name=prio>\n"));
	pPrio = prioLevel[IPQOS_NUM_PRIOQ-1];
	nBytesSent += websWrite(wp, T(" \t\t<option value=%d>%s(%s)</option>\n"),  IPQOS_NUM_PRIOQ-1, pPrio,Tqos_level_lowest);
	for (i=IPQOS_NUM_PRIOQ-2; i>=1; i--) {
		pPrio = prioLevel[i];
		nBytesSent += websWrite(wp, T("  \t\t<option value=%d>%s</option>\n"), i, pPrio);
	}
	pPrio = prioLevel[0];
	nBytesSent += websWrite(wp, T("  \t\t<option value=0>%s(%s)</option>\n"), pPrio,Tqos_level_highest);
	nBytesSent += websWrite(wp, T("  '\t</select>\n"));
#else
	nBytesSent += websWrite(wp, T(" \t<select name=prio>\n"));
	pPrio = prioLevel[IPQOS_NUM_PRIOQ-1];
nBytesSent += websWrite(wp, T("\t\t<option value=%d>%s(%s)</option>\n"),  IPQOS_NUM_PRIOQ-1, pPrio,Tqos_level_lowest);
	for (i=IPQOS_NUM_PRIOQ-2; i>=1; i--) {
		pPrio = prioLevel[i];
		nBytesSent += websWrite(wp, T("\t\t<option value=%d>%s</option>\n"), i, pPrio);
	}
	pPrio = prioLevel[0];
	nBytesSent += websWrite(wp, T(" \t\t<option value=0>%s(%s)</option>\n"), pPrio,Tqos_level_highest);
	nBytesSent += websWrite(wp, T("  \t<select>\n"));
#endif
	return nBytesSent;
}
#endif	// of IP_QOS


