/*
 *      Web server handler routines for IP QoS
 *
 */

/*-- System inlcude files --*/
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <sys/wait.h>

#include "../webs.h"
#include "webform.h"
#include "mib.h"
#include "utility.h"

#ifdef EMBED
#include <linux/config.h>
#else
#include "../../../uClibc/include/linux/autoconf.h"
#endif

#undef QOS_DEBUG
static int function_print = 1;
static int debug_print    = 1;

//macros
#define QUEUE_NUM_MAX          256
#define QUEUE_RULE_NUM_MAX     256
#define TRAFFICTL_RULE_NUM_MAX 256
#define INF_NAME_MAX           16

#define DELIM      '&'
#define SUBDELIM   '|'
#define SUBDELIM1  "|"

//print debug information
#ifdef QOS_DEBUG
#define PRINT_FUNCTION do{ if(function_print) printf("%s: %s\n", __FILE__, __FUNCTION__);}while(0);
#define QOS_DEBUG(fmt, args...) do{if(debug_print) printf("QOS DEBUG: " fmt, ##args);}while(0)

#define  PRINT_QUEUE(pEntry)  \
    printf("[QUEUE]: ifIndex:%d, ifname:%s, desc:%s, prio:%d, queueKey:%d, enable:%d\n",  \
	    pEntry->ifIndex, pEntry->ifname, pEntry->desc, pEntry->prio, pEntry->queueKey, pEntry->enable);

#define PRINT_QUEUE_RULE(pEntry)  \
   printf("[QUEUE RULE]: idx:%d, ifidx:%d, name:%s, state:%d, prio:%d, mark dscp:%02x,\n"      \
	  "mark 8021p:%d, dscp:%02x, vlan 8021p:%d, phyPort:%d, prototype:%d, "    \
	  "srcip:%s, smaskbits:%d, dstip:%s, dmaskbits:%d, src port:%d-%d, dst port:%d-%d\n",               \
	   pEntry->index, pEntry->ifIndex, pEntry->RuleName, pEntry->state, pEntry->prio, \
	   pEntry->m_dscp, pEntry->m_1p, pEntry->dscp, pEntry->vlan1p, pEntry->phyPort, pEntry->protoType,  \
	   inet_ntoa(*((struct in_addr*)&pEntry->srcip)), pEntry->smaskbits, \
	   inet_ntoa(*((struct in_addr*)&pEntry->dstip)), pEntry->dmaskbits, pEntry->srcPortStart,          \
	   pEntry->srcPortEnd, pEntry->dstPortStart, pEntry->dstPortEnd); 

#define PRINT_TRAFFICTL_RULE(pEntry) \
    printf("[TRAFFIC CONTROL]: entryid:%d, ifIndex:%d, ifName:%s, srcip:%s, smaskbits:%d, dstip:%s, dmaskbits:%d,"  \
	   "sport:%d, dport%d, protoType:%d, limitspeed:%d\n",                                                      \
	    pEntry->entryid, pEntry->ifIndex, pEntry->ifname, inet_ntoa(*((struct in_addr*)&pEntry->srcip)),        \
	    pEntry->smaskbits, inet_ntoa(*((struct in_addr*)&pEntry->dstip)), pEntry->dmaskbits,                    \
	    pEntry->sport, pEntry->dport, pEntry->protoType, pEntry->limitSpeed);

#else

#define PRINT_FUNCTION do{}while(0);
#define QOS_DEBUG(fmt, args...) do{}while(0)
#define PRINT_QUEUE(pEntry)
#define PRINT_QUEUE_RULE(pEntry)
#define PRINT_TRAFFICTL_RULE(pEntry)

#endif

// List all the available LAN side interface at web page.
// return: number of interface listed.
int iflanList(int eid, webs_t wp, int argc, char_t **argv)
{
	int nBytesSent=0;
	int ifnum=0;
	unsigned int i;
	char_t *name;
	struct itfInfo itfs[16];
	int type;

	PRINT_FUNCTION
	
	if (ejArgs(argc, argv, T("%s"), &name) < 1) {
		websError(wp, 400, T("Insufficient args\n"));
		return -1;
	}
	
	type = 0;
	if ( !strcmp(name, T("all")) )
		type = (DOMAIN_ELAN|DOMAIN_WLAN);
	else if ( !strcmp(name, T("eth")) )
		type = DOMAIN_ELAN;	// eth interface
	else if ( !strcmp(name, T("wlan")) )
		type = DOMAIN_WLAN;	// wireless interface
	else
		type = DOMAIN_ELAN;	// default to eth
	
	ifnum = get_domain_ifinfo(itfs, 16, type);
	if (ifnum==0)
		return 0;
#ifdef ZTE_GENERAL_ROUTER_SC
	for (i=0; i<ifnum; i++) {
			nBytesSent += websWrite(wp, T("<option value=%u>%s</option>\n"),
			IF_ID(itfs[i].ifdomain, itfs[i].ifid), itfs[i].name);
	}
#else
	for (i=0; i<ifnum; i++) {
			nBytesSent += websWrite(wp, T("<option value=%u>%s</option>\n"),
			IF_ID(itfs[i].ifdomain, itfs[i].ifid), itfs[i].name);
	}
#endif	
	return nBytesSent;
}

//show the wan interface list, using js code, must have waniflist array in js code
int ifWanList(int eid, webs_t wp, int argc, char_t **argv)
{
    MIB_CE_ATM_VC_T entry;
    int entryNum = 0, i=0, nBytes = 0;
    char wanif[32] = {0}, *prefix = NULL, type='R';

    PRINT_FUNCTION

	entryNum = mib_chain_total(MIB_ATM_VC_TBL);
    QOS_DEBUG("Total entry num:%d\n", entryNum);

    //default
    nBytes += websWrite(wp, "waniflst.add(new it(\" \", \" \"));");

    for(i=0;i<entryNum;i++)
	{
		if(!mib_chain_get(MIB_ATM_VC_TBL, i, &entry)||!entry.qosenable)
		    continue;
		/*
		if(entry.cmode == 0) 
		    type ='B';
		else
		    type ='R';

		if(entry.cmode==2){
		    prefix = "ppp";
		}else{
		    prefix = "nas";
		}

		if(prefix)
		{
		    snprintf(wanif, 32, "%s_%c_0_%d_%d", prefix, type, entry.vpi, entry.vci);
		    QOS_DEBUG("WAN INTERFACE:%s\n", wanif);
		    nBytes += websWrite(wp, "waniflst.add(new it(\"%d|%s\", \"%s\"));", 
			    entry.ifIndex, wanif, wanif);
		}*/
		getWanName(&entry, wanif);
		nBytes += websWrite(wp, "waniflst.add(new it(\"%d|%s\", \"%s\"));", 
			entry.ifIndex, wanif, wanif);
	}

    return nBytes;
}

////////////////////////////////////////////////////////////////////////////////////////////
//Queue Show and edit code


//we don't write it, let RTL do it
static int showWirelessQueue(webs_t* wp)
{
    return 0;
}

static int showWanQueue(webs_t* wp)
{
    MIB_CE_IP_QUEUE_CFG_T entry;
    int i=0, entryNum=0, nBytes = 0;

    PRINT_FUNCTION

    entryNum = mib_chain_total(MIB_IP_QUEUE_CFG_TBL);
    for(;i<entryNum; i++)
    {
	char buff[512] = {0};
	if(!mib_chain_get(MIB_IP_QUEUE_CFG_TBL, i, &entry))
	    continue;
	nBytes += websWrite(*wp, "wanqueue.push(new it_nr(\"\",\n"
		"new it(\"inf\", \"%s\"),\n"
		"new it(\"desc\", \"\"),\n"
		"new it(\"prio\", %d),\n"
		"new it(\"queuekey\", %d),\n"
		"new it(\"queuestatus\", %d)));\n",
		entry.ifname, entry.prio, entry.queueKey, entry.enable);
    }

    return nBytes;
} 

int initQueuePage(int eid, webs_t wp, int argc, char_t **argv)
{
    int nBytes = 0;

    PRINT_FUNCTION

    nBytes += showWirelessQueue(&wp);
    nBytes += showWanQueue(&wp);

    return nBytes;
}

int initQueuePolicy(int eid, webs_t wp, int argc, char_t **argv)
{
	MIB_CE_IP_QUEUE_LEN_T qEntry;
	MIB_CE_ATM_VC_T vcEntry;
	int vcTotal, qTotal, i, j;
    unsigned char policy = 0;
	char wanname[32];
	int nBytes=0;
	char *prefix = NULL, type='R';

    PRINT_FUNCTION

    if(!mib_get(MIB_QOS_QUEUE_POLICY, &policy)|| (policy !=0&&policy!=1))
		policy = 0;
    nBytes = websWrite(wp, "policy=%d;\n", policy);

	///
	nBytes += websWrite(wp, T("rules.push(\n"
		"new it_nr(\"  \",	  \n"	//wan interface name
		"new it(\"ifIndex\", 255),\n"
		"new it(\"prece1\", 0),\n"
		"new it(\"unit1\", 0),\n"
		"new it(\"prece2\", 0),\n"
		"new it(\"unit2\", 0),\n"
		"new it(\"prece3\", 0),\n"
		"new it(\"unit3\", 0),\n"
		"new it(\"prece4\", 0),\n"
		"new it(\"unit4\", 0)));\n"));

	vcTotal = mib_chain_total(MIB_ATM_VC_TBL);
	qTotal = mib_chain_total(MIB_IP_QUEUE_LEN_TBL);
	for(i=0; i<vcTotal; i++)
	{
		int len=0;
		
		if(!mib_chain_get(MIB_ATM_VC_TBL, i, &vcEntry)||!vcEntry.qosenable)
			continue;

		/*if(vcEntry.cmode == 0) 
			type ='B';
		else
			type ='R';
	
		if(vcEntry.cmode==2){
			prefix = "ppp";
		}else{
			prefix = "nas";
		}
		snprintf(wanname, 32, "%s_%c_0_%d_%d", prefix, type, vcEntry.vpi, vcEntry.vci);*/
		getWanName(&vcEntry, wanname);
		//QOS_DEBUG("WAN INTERFACE:%s\n", wanname);

		for (j=0; j<qTotal; j++)
		{
			if (!mib_chain_get(MIB_IP_QUEUE_LEN_TBL, j, &qEntry))
				continue;
			if (qEntry.ifIndex == vcEntry.ifIndex)
				break;
		}

		if (j >= qTotal)//not found
			nBytes += websWrite(wp, T("rules.push(\n"
				"new it_nr(\"%s\",    \n"	//wan interface name
			    "new it(\"ifIndex\",%d),\n"
			    "new it(\"prece1\", %d),\n"
			    "new it(\"unit1\", 0),\n"
			    "new it(\"prece2\", %d),\n"
			    "new it(\"unit2\", 0),\n"
			    "new it(\"prece3\", %d),\n"
			    "new it(\"unit3\", 0),\n"
			    "new it(\"prece4\", %d),\n"
			    "new it(\"unit4\", 0)));\n"),
			    wanname, vcEntry.ifIndex, 0, 0, 0, 0);
		else
			nBytes += websWrite(wp, T("rules.push(\n"
				"new it_nr(\"%s\",    \n"	//wan interface name
			    "new it(\"ifIndex\",%d),\n"
			    "new it(\"prece1\", %d),\n"
			    "new it(\"unit1\", %d),\n"
			    "new it(\"prece2\", %d),\n"
			    "new it(\"unit2\", %d),\n"
			    "new it(\"prece3\", %d),\n"
			    "new it(\"unit3\", %d),\n"
			    "new it(\"prece4\", %d),\n"
			    "new it(\"unit4\", %d)));\n"),
			    wanname, vcEntry.ifIndex, qEntry.len[0], qEntry.unit[0], qEntry.len[1], 
			    qEntry.unit[1], qEntry.len[2], qEntry.unit[2], qEntry.len[3], qEntry.unit[3]);
	}

	return nBytes;
}

void formQosPolicy(webs_t wp, char_t *path, char_t *query)
{
	MIB_CE_IP_QUEUE_LEN_T entry, tmpEntry;
	int mibtotal, i;
    char *action=NULL,*url=NULL, *p=NULL, *tmp=NULL;
    unsigned char policy = 0;
    int num = 0;
     
    PRINT_FUNCTION

    action = websGetVar(wp, T("lst"), T(""));
	//printf("policy: %s\n", action);
    if(action[0])
    {
		//policy
		p = strstr(action, "policy=");
		if(p)
		{
		    p+=strlen("policy=");
			num = strtol(p, &tmp, 0);
	    	if(tmp && tmp !=p && *tmp == DELIM) {
				policy = !!num;
				if(!mib_set(MIB_QOS_QUEUE_POLICY, &policy)) {
				    ERR_MSG("Failed to set qos queue policy.\n");
				    return;
				}
		    }
		}

		//queue status
		p = strstr(action, "ifIndex=");
		if(p)
		{
		    p+=strlen("ifIndex=");
			entry.ifIndex = strtol(p, &tmp, 0);
			if(entry.ifIndex == 255)
				goto done;
		}

		//prece 1 len
		p = strstr(action, "prece1=");
		if(p)
		{
		    p+=strlen("prece1=");
			if(*p == DELIM) {
				entry.len[0] = 0;
		    } else {
				entry.len[0] = strtol(p, &tmp, 0);
				if(*tmp != DELIM)
					ERR_MSG("error queue 1 len.\n");;
		    }
		}

		//prece 1 unit
		p = strstr(action, "unit1=");
		if(p)
		{
		    p+=strlen("unit1=");
			if(*p == DELIM) {
				entry.unit[0] = 0;
		    } else {
				entry.unit[0] = strtol(p, &tmp, 0);
				if(*tmp != DELIM)
					ERR_MSG("error queue 1 unit.\n");;
		    }
		}
		
		//prece 2 len
		p = strstr(action, "prece2=");
		if(p)
		{
		    p+=strlen("prece2=");
			if(*p == DELIM) {
				entry.len[1] = 0;
		    } else {
				entry.len[1] = strtol(p, &tmp, 0);
				if(*tmp != DELIM)
					ERR_MSG("error queue 2 len.\n");;
		    }
		}

		//prece 2 unit
		p = strstr(action, "unit2=");
		if(p)
		{
		    p+=strlen("unit2=");
			if(*p == DELIM) {
				entry.unit[1] = 0;
		    } else {
				entry.unit[1] = strtol(p, &tmp, 0);
				if(*tmp != DELIM)
					ERR_MSG("error queue 2 unit.\n");;
		    }
		}

		//prece 3 len
		p = strstr(action, "prece3=");
		if(p)
		{
		    p+=strlen("prece3=");
			if(*p == DELIM) {
				entry.len[2] = 0;
		    } else {
				entry.len[2] = strtol(p, &tmp, 0);
				if(*tmp != DELIM)
					ERR_MSG("error queue 3 len.\n");;
		    }
		}

		//prece 1 unit
		p = strstr(action, "unit3=");
		if(p)
		{
		    p+=strlen("unit3=");
			if(*p == DELIM) {
				entry.unit[2] = 0;
		    } else {
				entry.unit[2] = strtol(p, &tmp, 0);
				if(*tmp != DELIM)
					ERR_MSG("error queue 3 unit.\n");;
		    }
		}

		//prece 4 len
		p = strstr(action, "prece4=");
		if(p)
		{
		    p+=strlen("prece4=");
			if(*p == DELIM) {
				entry.len[3] = 0;
		    } else {
				entry.len[3] = strtol(p, &tmp, 0);
		    }
		}

		//prece 1 unit
		p = strstr(action, "unit4=");
		if(p)
		{
		    p+=strlen("unit4=");
			if(*p == DELIM) {
				entry.unit[3] = 0;
		    } else {
				entry.unit[3] = strtol(p, &tmp, 0);
				if(*tmp != DELIM)
					ERR_MSG("error queue 4 unit.\n");;
		    }
		}

		printf("ifIndex:%d prece1:%d unit1:%d prece2:%d unit2:%d prece3:%d unit3:%d prece4:%d unit4:%d\n", 
			entry.ifIndex, entry.len[0], entry.unit[0], entry.len[1], entry.unit[1], entry.len[2], 
			entry.unit[2], entry.len[3], entry.unit[3]);
		
		mibtotal = mib_chain_total(MIB_IP_QUEUE_LEN_TBL);
		printf("MIB_IP_QUEUE_LEN_TBL len:%d\n", mibtotal);
		for (i=0; i<mibtotal; i++)
		{
			if (!mib_chain_get(MIB_IP_QUEUE_LEN_TBL, i, &tmpEntry))
				continue;
			if (tmpEntry.ifIndex != entry.ifIndex)
				continue;
			
			//update current mib entry
			if (!mib_chain_update(MIB_IP_QUEUE_LEN_TBL, (void *)&entry, i)) {
				ERR_MSG("update IP Queue error.\n");
				return;
			}
		}
		if (i >= mibtotal)
			if (!mib_chain_add(MIB_IP_QUEUE_LEN_TBL, (void *)&entry))
				printf("MIB_IP_QUEUE_LEN_TBL add failed!\n");
    }

done:
	//take effect
	take_qos_effect();

	mib_update(CURRENT_SETTING, CONFIG_MIB_ALL);

    //redirect web page
    url = websGetVar(wp, T("submit-url"), T(""));
    if(url[0])
    {
		websRedirect(wp, url);
    }
    return;
}

//qos queue editing main function
void formQosQueueEdit(webs_t wp, char_t *path, char_t *query)
{
    MIB_CE_IP_QUEUE_CFG_T entry;
    char args[512]={0}, *action=NULL,*url=NULL, *p=NULL, *tmp=NULL;
    unsigned char policy = 0;
    unsigned char map[QUEUE_NUM_MAX+1] = {0};
    char ifName[64] ={0}; 
    unsigned char ifIndex = 0, queuestatus = 1, queueprio = 0;
    int i=0, entryNum=0,num = 0, queueKey = 1;
     
    PRINT_FUNCTION

    if(entryNum>=QUEUE_NUM_MAX)
    {
	ERR_MSG("Qos queue is full, you must delete some one!");
	return;
    }

    action = websGetVar(wp, T("lst"), T(""));

    if(action[0])
    {
	//policy
	p = strstr(action, "policy=");
	if(p)
	{
	    p+=strlen("policy=");
	    num = strtol(p, &tmp, 0);
	    if(tmp && tmp !=p && *tmp == DELIM) {
		policy = !!num;
		if(!mib_set(MIB_QOS_QUEUE_POLICY, &policy)) {
		    printf("Failed to set qos queue policy.\n");
		    return;
		}
	    }
	}
    
	//queue status
	p = strstr(action, "queuestatus=");
	if(p)
	{
	    p+=strlen("queuestatus=");

	    num = strtol(p, &tmp, 0);
	    if(tmp && tmp !=p && *tmp == DELIM)
	    {
		    queuestatus = !!num;
	    }
	}

	//queue interface
	p = strstr(action, "queueinf=");
	if(p)
	{
	    p+=strlen("queueinf=");

	    num = strtol(p, &tmp, 0);
	    if(tmp && tmp !=p && *tmp == SUBDELIM)
	    {
		    //check its validity
		    ifIndex = num;
		
	    }
	    //get ifname, modified by penggenyao
	    if(*tmp == SUBDELIM)
	    {
		i=0;
		for(tmp++; *tmp != DELIM; tmp++)
		    ifName[i++] = *tmp;
	    }
	}

	//queue priority
	p = strstr(action, "queueprio=");
	if(p)
	{
	    p+=strlen("queueprio=");

	    num = strtol(p, &tmp, 0);
	    if(tmp != p&&num>=1&&num<=4 )
	    {
		    //check its validity
		    queueprio = num;
	    }
	}

    entryNum = mib_chain_total(MIB_IP_QUEUE_CFG_TBL);

	for(i=0;i<entryNum;i++)
	{
	    if(!mib_chain_get(MIB_IP_QUEUE_CFG_TBL, i, &entry))
		    continue;

        //The queue with the same interface and prio has existed
        if(queueprio == entry.prio&&ifIndex == entry.ifIndex)
        {
            ERR_MSG("The queue with the same interface and prio has existed!");
            return;
        }
	    map[entry.queueKey] = 1; 
	}
	for(i=1;i<=QUEUE_NUM_MAX;i++)
	{
	    if(!map[i])
	    {
		queueKey = i;
		break;
	    }
	    else if(i==QUEUE_NUM_MAX)
	    {
		ERR_MSG("Qos queue is full, you must delete some one!");
		return;
	    }
	}

	memset(&entry, 0, sizeof(MIB_CE_IP_QUEUE_CFG_T));

	
	entry.queueKey = queueKey;
    entry.ifIndex = ifIndex;
    entry.prio = queueprio;
    entry.enable = queuestatus;
    strncpy(entry.ifname, ifName, sizeof(entry.ifname));

	PRINT_QUEUE((&entry));

	if(!mib_chain_add(MIB_IP_QUEUE_CFG_TBL, &entry))
	{
	    ERR_MSG("Failed to create new qos queue!\n");
	    return;
	}
    }
       mib_update(CURRENT_SETTING, CONFIG_MIB_ALL);
    //redirect web page
    url = websGetVar(wp, T("submit-url"), T(""));
    if(url[0])
    {
	websRedirect(wp, url);
    }
    return;
}


////////////////////////////////////////////////////////////////////////////////////////////
//Qos Traffic control code

//now the problem is that the ifIndex is used in c code, and ifname is used for displaying,
//how can i get the ifname or ifIndex
int initTraffictlPage(int eid, webs_t wp, int argc, char_t **argv)
{
    MIB_CE_IP_TC_T entry;
    int entryNum = 0, i=0, nBytes = 0;
    char sip[20], dip[20], *p = NULL;
    unsigned short total_bandwidth = 0;
	unsigned char totalBandWidthEn=0;

    PRINT_FUNCTION

    entryNum = mib_chain_total(MIB_TRAFFICTL_TBL);
	
    if(mib_get(MIB_TOTAL_BANDWIDTH, &total_bandwidth))
    {
		nBytes += websWrite(wp, "totalBandwidth=%d;\n", total_bandwidth);
    }
    else 
    {
		nBytes += websWrite(wp, "totalBandwidth=1024;\n");
    }

	if (mib_get(MIB_TOTAL_BANDWIDTH_LIMIT_EN, &totalBandWidthEn))
		nBytes += websWrite(wp, "totalBandWidthEn=%d;\n", totalBandWidthEn);
	else
		nBytes += websWrite(wp, "totalBandWidthEn=0;\n");

    for(;i<entryNum; i++)
	{
		if(!mib_chain_get(MIB_TRAFFICTL_TBL, i, &entry))
		    continue;
		strncpy(sip, inet_ntoa(*((struct in_addr*)&entry.srcip)), 16);
		strncpy(dip, inet_ntoa(*((struct in_addr*)&entry.dstip)), 16);
		if(entry.smaskbits)
		{
		    p = sip + strlen(sip);
		    snprintf(p,sizeof(sip)-strlen(sip), "/%d", entry.smaskbits );
		}
		if(entry.dmaskbits)
		{
		    p = dip + strlen(dip);
		    snprintf(p,sizeof(dip)-strlen(dip), "/%d", entry.dmaskbits );
		}
		nBytes += websWrite(wp, "traffictlRules.push(new it_nr(\"\",\n"
		          "new it(\"id\",         %d),\n"	      
			      "new it(\"inf\",    \"%s\"),\n"
			      "new it(\"proto\",      %d),\n"
			      "new it(\"sport\",      %d),\n"
			      "new it(\"dport\",      %d),\n"
			      "new it(\"srcip\",  \"%s\"),\n"
			      "new it(\"dstip\",  \"%s\"),\n"
			      "new it(\"uprate\",   %d)));\n",
			      entry.entryid, entry.ifname, entry.protoType, entry.sport,
			      entry.dport, sip, dip, entry.limitSpeed);
	}
    return nBytes;
}

void formQosTraffictl(webs_t wp, char_t *path, char_t *query)
{
    char *action = NULL, *url = NULL;
    char *act1="applybandwidth", *act2 = "traffictldelete", *act3 = "applysetting";
	char *act4="cancelbandwidth";
    int entryNum = 0;
    unsigned char policy=0;
    
    PRINT_FUNCTION

    action = websGetVar(wp, T("lst"), T("")); 
    
    entryNum = mib_chain_total(MIB_TRAFFICTL_TBL);  
    mib_get(MIB_QOS_QUEUE_POLICY, (void*)&policy);
    
    if(action[0])
	{
		if( !strncmp(action, act1, strlen(act1)) )
		{//set total bandwidth
		    unsigned short totalbandwidth = 0;
			unsigned char totalbandwidthEn=0;
		    char* strbandwidth = NULL;
		    strbandwidth = strstr(action, "bandwidth=");
		    strbandwidth += strlen("bandwidth=");
		    if(strbandwidth)//found it
		    {
				totalbandwidth = atoi(strbandwidth);

				totalbandwidthEn = 1;
				if (!mib_set(MIB_TOTAL_BANDWIDTH_LIMIT_EN, (void *)&totalbandwidthEn)) {
					ERR_MSG("Cannot set total bandwidth Enable into mib setting!\n");
				    return;
				}
				
				if(!mib_set(MIB_TOTAL_BANDWIDTH, (void*)&totalbandwidth))
				{
				    ERR_MSG("Cannot set total bandwidth into mib setting!\n");
				    return;
				}

				//take effect
				take_qos_effect();
		    }
			else {
				totalbandwidthEn = 0;
				if (!mib_set(MIB_TOTAL_BANDWIDTH_LIMIT_EN, (void *)&totalbandwidthEn)) {
					ERR_MSG("Cannot set total bandwidth Disable into mib setting!\n");
				    return;
				}
			}
		}
		else if ( !strncmp(action, act4, strlen(act4)) )
		{//cancel total bandwidth restrict
			unsigned char totalbandwidthEn=0;
			if (!mib_set(MIB_TOTAL_BANDWIDTH_LIMIT_EN, (void *)&totalbandwidthEn)) {
				ERR_MSG("Cannot disable total bandwidth restrict into mib setting!\n");
				return;
			}

			//take effect
			take_qos_effect();
		}
		else if( !strncmp(action, act2, strlen(act2)) )
		{//delete some
		    int idlst[TRAFFICTL_RULE_NUM_MAX+1] = {0}; 
		    int  i=0, j=0, index=1;
		    char stridlst[256],err_msg[256], *p = NULL;
		    MIB_CE_IP_TC_T entry;

		    p = strstr(action, "id=");
		    p += strlen("id=");
		    if(*p == '\0') {//delete none
			goto done;
		    }

		    stridlst[0] = '\0';

		    strncpy(stridlst, p, 256);

		    //convert the id list, store them in idlst, 
		    //you can delete most 10 rules at one time
		    p = strtok(stridlst, SUBDELIM1);
		    if(p) index = atoi(p);
		    if(index>0&&index<=TRAFFICTL_RULE_NUM_MAX) idlst[index]=1;
		    while((p = strtok(NULL, SUBDELIM1)) != NULL)
		    {
			index = atoi(p);
			idlst[index]=1;
			if(index > TRAFFICTL_RULE_NUM_MAX )
			    break;
		    }

		    for(i=entryNum-1; i>=0; i--) 
		    {
			if(!mib_chain_get(MIB_TRAFFICTL_TBL, i, &entry))
			    continue;

			if( 1 == idlst[entry.entryid]) //delete it
			{
	            //delete rules of  tc and iptables
			    if(1 != mib_chain_delete(MIB_TRAFFICTL_TBL, i)) {
				snprintf(err_msg, 256, "Error happened when deleting rule %d", entry.entryid);
				ERR_MSG(err_msg);
				return;
			    }
			}
		    }
		}
		else if( !strncmp(action, act3, strlen(act3)) ) {
		    //apply setting into effect
		    take_qos_effect();
		}
	}
    mib_update(CURRENT_SETTING, CONFIG_MIB_ALL);
done:
    //well, go back
    url = websGetVar(wp, T("submit-url"), T(""));
    if(url[0])
	websRedirect(wp, url);
    return;
}

//Used to get subnet mask bit number 
static int getNetMaskBit(char* netmask)
{
    unsigned int bits = 0, mask = 0;
    int i=0, flag = 0;

    if(!netmask||strlen(netmask)>15)
	return 0;
    mask = inet_network(netmask);
    for(;i<32;i++)
    {
	if(mask&(0x80000000>>i)) {
	    if(flag)
		return 0;
	    else
		bits++;
	}
	else {
	    flag = 1;
	}
    }
    return bits;
}

//parse formated string recieved from web client:
//inf=VAL&proto=VAL&srcip=VAL&srcnetmask=VAL&dstip=VAL&dstnetmask=VAL&sport=VAL&dport=VAL&uprate=VAL(old).
//where VAL is the value of the corresponding item. if none, VAL is ignored but the item name cannot be
//The argument action is not modified by the function, so it can be a const string or non-const string.
static int parseArgs(char* action, MIB_CE_IP_TC_Tp pEntry)
{
    char* p = NULL, *tmp = NULL;
    int i = 0;
    
    PRINT_FUNCTION

    //ifIndex
    tmp = strstr(action, "inf=");
    tmp += strlen("inf=");
    pEntry->ifIndex = strtol(tmp, &p,0);
    if(*p != SUBDELIM)
	return 1;

    //ifname
    for(p++; *p != DELIM; p++)
    {
	pEntry->ifname[i++] = *p;
    }

    //protocol
    tmp =strstr(action, "proto=");
    tmp += strlen("proto=");
    if(!tmp||*tmp == DELIM)//not set protocol, set it to default,none
	pEntry->protoType = 0;
    else
    {
	pEntry->protoType = strtol(tmp, &p, 0);
	if(*p != DELIM)
	    return 1;
    }

    //source ip
    tmp = strstr(action, "srcip=");
    tmp += strlen("srcip=");
    if(!tmp||*tmp == DELIM)//noet set, set default
	pEntry->srcip = 0;
    else
    {
	char sip[16]={0};
	struct in_addr addr;
	p = strchr(tmp, DELIM);
	if(p&&p-tmp>15)
	    return 1;
	strncpy(sip, tmp, p-tmp);
	if(!inet_aton(sip, &addr))
	    return 1;
	pEntry->srcip = addr.s_addr;
    }

    //source ip address netmask
    tmp = strstr(action, "srcnetmask=");
    tmp += strlen("srcnetmask=");
    if(!tmp||*tmp==DELIM)
	pEntry->smaskbits = 0;
    else
    {
	char smask[16]={0};
	p = strchr(tmp, DELIM);
	if(p&&p-tmp>15) return 1;
	strncpy(smask, tmp, p-tmp);
	pEntry->smaskbits = getNetMaskBit(smask);
    }

    //destination ip
    tmp = strstr(action, "dstip=");
    tmp += strlen("dstip=");
    if(!tmp||*tmp == DELIM)//noet set, set default
	pEntry->dstip = 0;
    else
    {
	char dip[16]={0};
	struct in_addr addr;
	p = strchr(tmp, DELIM);
	if(p&&p-tmp>15)
	    return 1;
	strncpy(dip, tmp, p-tmp);
	if(!inet_aton(dip, &addr))
	    return 1;
	pEntry->dstip = addr.s_addr;
    }

    //destination ip address netmask
    tmp = strstr(action, "dstnetmask=");
    tmp += strlen("dstnetmask=");
    if(!tmp||*tmp==DELIM)
	pEntry->dmaskbits = 0;
    else
    {
	char dmask[16]={0};
	p = strchr(tmp, DELIM);
	if(p&&p-tmp>15)
	    return 1;
	strncpy(dmask, tmp, p-tmp);
	pEntry->dmaskbits = getNetMaskBit(dmask);

    }

    //source port
    tmp = strstr(action, "sport=");
    tmp += strlen("sport=");
    if(!tmp||*tmp==DELIM)
	pEntry->sport = 0;
    else
    {
	pEntry->sport = strtol(tmp, &p, 0);
	if(*p != DELIM)
	    return 1;
    }

    //destination port
    tmp = strstr(action, "dport=");
    tmp += strlen("dport=");
    if(!tmp||*tmp==DELIM)
	pEntry->dport = 0;
    else
    {
	pEntry->dport = strtol(tmp, &p, 0);
	if(*p != DELIM)
	    return 1;
    }

    //upstream rate
    tmp = strstr(action, "uprate=");
    tmp += strlen("uprate=");
    if(!tmp||*tmp=='\0')
	pEntry->limitSpeed = 0;
    else
    {
	pEntry->limitSpeed = strtol(tmp, &p, 0);
	if(*p != '\0')
	    return 1;
    }

    return 0;
}

//add traffic controlling rules's main function
void formQosTraffictlEdit(webs_t wp, char_t *path, char_t *query)
{
    MIB_CE_IP_TC_T entry;
    char* action=NULL, *url = NULL;
    int entryNum = 0, entryid = 1;
    unsigned char map[TRAFFICTL_RULE_NUM_MAX+1]={0};

    PRINT_FUNCTION

    entryNum = mib_chain_total(MIB_TRAFFICTL_TBL);
    
    //You are allowed to have TRAFFICTL_RULE_NUM_MAX rules
    if(entryNum>=TRAFFICTL_RULE_NUM_MAX)
    {
		ERR_MSG("Traffic controlling queue is full, you must delete some one!");
		return;
    }

    action = websGetVar(wp, T("lst"), T(""));

    if(action[0])
    {
		//allocate a free rule id for new entry
		{
		    int i = 0;
		    for(;i<entryNum;i++)
		    {
				if(!mib_chain_get(MIB_TRAFFICTL_TBL, i, &entry))
				    continue;

				map[entry.entryid] = 1; 
		    }
		    for(i=1;i<=TRAFFICTL_RULE_NUM_MAX;i++)
		    {
				if(!map[i])
			       	{
				    entryid = i;
				    break;
				}
			    else if(i==TRAFFICTL_RULE_NUM_MAX)
				{
				    ERR_MSG("Traffic controlling queue is full, you must delete some one!");
				    return;
				}
		    }
		}

		memset(&entry, 0, sizeof(MIB_CE_IP_TC_T));

		entry.entryid = entryid;

		if(parseArgs(action, &entry))
		{//some arguments are wrong
		    ERR_MSG("Wrong setting is found!");
		    return;
		}

		PRINT_TRAFFICTL_RULE((&entry));
		if(!mib_chain_add(MIB_TRAFFICTL_TBL, &entry))
		{//adding mib setting is wrong 
		    ERR_MSG("Cannot add setting into mib!");
		    return;
		}
    }
    mib_update(CURRENT_SETTING, CONFIG_MIB_ALL);
    //redirect
    url = websGetVar(wp, T("submit-url"), T(""));
    if(url[0])
	websRedirect(wp, url);
    return;
}


char* netmaskbits2str(const int netmaskbit, char* netmaskstr, int strlen)
{
    unsigned int netmaskaddr = 0, i=0;
    if(!netmaskstr || !netmaskbit)
	return NULL;
    for(i=0;i<netmaskbit;i++)
	netmaskaddr = netmaskaddr|(0x80000000>>i);
    netmaskaddr = htonl(netmaskaddr);//host byte order to network byte order
    strncpy(netmaskstr, inet_ntoa(*(struct in_addr*)&netmaskaddr), strlen );

    return netmaskstr;
}

/******************************************************************
 * NAME:    intPageQosRule
 * DESC:    initialize the qos rules by reading mib setting and 
 *          format them to send them to webs to dispaly
 * ARGS:
 * RETURN:  
 ******************************************************************/
int initQosRulePage(int eid, webs_t wp, int argc, char_t **argv)
{
    MIB_CE_IP_QUEUE_RULE_T qEntry;
    MIB_CE_ATM_VC_T       vcEntry;
    char saddr[16]={0}, daddr[16]={0}, smask[16]={0}, dmask[16]={0};
    int i=0, qEntryNum = 0, vcEntryNum = 0, nBytes = 0;

    PRINT_FUNCTION

    //get number of  ip qos queue rules, if none, or cannot get, return
    if((qEntryNum=mib_chain_total(MIB_IP_QUEUE_RULE_TBL)) <=0)
		return;

    if((vcEntryNum=mib_chain_total(MIB_ATM_VC_TBL)) <=0)
		return;

    for(i=0;i<qEntryNum; i++)
	{
		char phyportName[8] = {0};
		if(!mib_chain_get(MIB_IP_QUEUE_RULE_TBL, i, (void*)&qEntry))
		    continue;
		
		//src addr
		snprintf(saddr, 16, "%s", inet_ntoa(*((struct in_addr*)&(qEntry.srcip))));

		//src subnet mask
		if(!netmaskbits2str(qEntry.smaskbits, smask, 16))
		{
		    smask[0] = '\0';
		}

		//dst addr 
		snprintf(daddr, 16, "%s", inet_ntoa(*((struct in_addr*)&(qEntry.dstip))));
		//dst subnet mask
		if(!netmaskbits2str(qEntry.dmaskbits, dmask, 16))
		{
		    dmask[0] = '\0';
		}

		//now write into webs using websWrite function
		nBytes += websWrite(wp, T("rules.push(\n"
			    "new it_nr(\"%s\",    \n"  //qos queue rule name
			    "new it(\"index\",%d),\n"  //index of queue rule(identifier)
			    "new it(\"state\",%d),\n"  //enable or disable
			    "new it(\"prio\", \"%d|%d\"),\n"  //queue priority, queueKey 
			    "new it(\"mdscp\",%d),\n"   //dscp mark 
			    "new it(\"m1p\",  %d),\n"     //802.1p mark for wan interface
			    "new it(\"vlan1p\",%d),\n"    //802.1p match for packet 
			    "new it(\"phypt\", %d),\n"    //Lan phy port number
			    "new it(\"proto\",%d),\n"     //protocol index, reference to mib.h 
			    "new it(\"dscp\", %d),\n"     //dscp
			    "new it(\"sip\",  \"%s\"),\n" //source ip, if stype=1, it is DHCP OPT 60
			    "new it(\"smsk\", \"%s\"),\n" //source ip subnet mask 
			    "new it(\"dip\",  \"%s\"),\n"  
			    "new it(\"dmsk\", \"%s\"),\n"
			    "new it(\"spts\", %d),\n"     //source port start
			    "new it(\"spte\", %d),\n"     //source port end
			    "new it(\"dpts\", %d),\n"
			    "new it(\"dpte\", %d),\n"
			    "new it(\"smac\", \"%s\"), \n"  //source mac address,now supported now, always 00:00:00:00:00:00
			    "new it(\"smacw\",\"%s\"), \n"  //source mac address wildword
			    "new it(\"dmac\", \"%s\"), \n"
			    "new it(\"dmacw\",\"%s\")));\n"),
		    qEntry.RuleName, qEntry.index, !!(qEntry.state), qEntry.ifIndex, qEntry.prio,
		    qEntry.m_dscp, qEntry.m_1p, qEntry.vlan1p, qEntry.phyPort, qEntry.protoType, qEntry.dscp, 
		    saddr, smask, daddr, dmask, qEntry.srcPortStart,
		    qEntry.srcPortEnd, qEntry.dstPortStart, qEntry.dstPortEnd, 
		    "00:00:00:00:00:00", "00:00:00:00:00:00", "00:00:00:00:00:00", "00:00:00:00:00:00");
	}

    return nBytes;
}

/******************************************************************
 * NAME:     formQosRule
 * DESC:     main function deal with qos rules, delete and change
 * ARGS:
 * RETURN:   
 ******************************************************************/
void formQosRule(webs_t wp, char_t* path, char_t* query)
{
    MIB_CE_IP_QUEUE_RULE_T entry;
    int entryNum=0, i=0, id = 0;
    unsigned char statuslist[QUEUE_RULE_NUM_MAX+1] = {0}, dellist[QUEUE_RULE_NUM_MAX+1]= {0};
    char* action = NULL, *p=NULL, *url = NULL;

    PRINT_FUNCTION

    //abstract args from string
    action = websGetVar(wp, T("lst"), T(""));
    entryNum = mib_chain_total(MIB_IP_QUEUE_RULE_TBL);

    p = strtok(action, ",&");
    if(p) id = atoi(p);
    if(id<1||id>QUEUE_RULE_NUM_MAX)
	return;
    p = strtok(NULL, ",&");
    if(p) statuslist[id] = !!atoi(p);
    p = strtok(NULL, ",&");
    if(p) dellist[id] = !!atoi(p);
    while( (p = strtok(NULL, ",&")) != NULL )
    {
	id = atoi(p);
	if(id<1||id>QUEUE_RULE_NUM_MAX) continue;
	p = strtok(NULL, ",&");
	if(p) statuslist[id] = !!atoi(p);
	p = strtok(NULL, ",&");
	if(p) dellist[id] = !!atoi(p);
    }

    //change status of rules if neccessary
    for(i=0;i<entryNum;i++)
    {
	if(!mib_chain_get(MIB_IP_QUEUE_RULE_TBL, i, &entry))
	    continue;

	if(statuslist[entry.index] != entry.state)
	{
	    entry.state = statuslist[entry.index];
	    if(!mib_chain_update(MIB_IP_QUEUE_RULE_TBL, &entry, i))
	    {
		ERR_MSG("updating rule error!");
		return;
	    }
	}
    }

    //delete some one
    for(i=entryNum-1;i>=0;i--)
    {
        if(!mib_chain_get(MIB_IP_QUEUE_RULE_TBL, i, &entry))
            continue;
		if(i+1<=QUEUE_RULE_NUM_MAX&&dellist[i+1])
		{
		    if(1 != mib_chain_delete(MIB_IP_QUEUE_RULE_TBL, i))
		    {
			    ERR_MSG("Delete rule error!");
			    return;
		    }
		}
    }

    //take effect
    take_qos_effect();
//alex	
     mib_update(CURRENT_SETTING, CONFIG_MIB_ALL);

    //redirect to show page
    url = websGetVar(wp, T("submit-url"), T(""));
    if(url[0])
    {
		websRedirect(wp, url);
    }
    return;
}

int initRulePriority(int eid, webs_t wp, int argc, char_t **argv)
{
    MIB_CE_ATM_VC_T vcEntry;
    int entryNum, i, j, nBytes = 0;
	char wanif[32]={0};

    PRINT_FUNCTION
    
    entryNum = mib_chain_total(MIB_ATM_VC_TBL);

    for(i=0; i<entryNum; i++)
    {
		if(!mib_chain_get(MIB_ATM_VC_TBL, i, &vcEntry)||!vcEntry.qosenable)
		    continue;

		getWanName(&vcEntry, wanif);

		for (j=1; j<=4; j++) {
			nBytes += websWrite(wp, "quekeys.add(new it(\"%d|%d\", \"%s&Prece %d\"));\n",
				vcEntry.ifIndex, j, wanif, j);
		}
    }
    return nBytes;
} 

static int parseRuleArgs(char* args, MIB_CE_IP_QUEUE_RULE_Tp pEntry)
{
    char* p=NULL, *tmp=NULL, buff[32] = {0};
    int idx = 0;

    //get index
    p = strstr(args, "index=");
    p +=strlen("index=");
    if(*p == DELIM) {
		pEntry->index = 0;
    } else {
		pEntry->index = strtol(p, &tmp, 0);
		if(*tmp != DELIM) return 1;
    }
    
    //get RuleName
    p = strstr(args, "name=");
    for(p+=strlen("name="); *p!=DELIM&&idx<INF_NAME_MAX;p++)
		pEntry->RuleName[idx++] = *p;

    //state
    p = strstr(args, "state=");
    p += strlen("state=");
    if(*p != DELIM) {
		pEntry->state = strtol(p, &tmp, 0);
		if(*tmp != DELIM)
	    	pEntry->state = 1;//default
    }else {
		pEntry->state = 1;//default
    }

	//get ifIndex, prio
    p = strstr(args, "prio=");
    p += strlen("prio=");
    if(*p==SUBDELIM||*p==DELIM) return 1;
    //ifIndex
    pEntry->ifIndex = strtol(p, &tmp, 0);
    if(*tmp != SUBDELIM) return 1;
    //prio
    pEntry->prio = strtol(++tmp, &p, 0);
    if(*p != DELIM) return 1;

    //mark 802.1p
    p = strstr(args, "mark1p=");
    p += strlen("mark1p=");
    if(*p == DELIM)//default
		pEntry->m_1p = 0;
    else {
		pEntry->m_1p = strtol(p, &tmp, 16);
		if(*tmp != DELIM) return 1;//invalid 802.1p value
    }

    //mark dscp
    p = strstr(args, "markdscp=");
    p += strlen("markdscp=");
    if(*p == DELIM)//default
		pEntry->m_dscp = 0;
    else {
		pEntry->m_dscp = strtol(p, &tmp, 0);
		if(*tmp != DELIM) return 1;//invalid dscp value
    }

    //phy port num
    p = strstr(args, "phyport=");
    p +=strlen("phyport=");
    if(*p == DELIM) {
		pEntry->phyPort = 0;//default phy port, none 
    } else {
		pEntry->phyPort = strtol(p, &tmp, 0);
		if(*tmp != DELIM) return 1;
    }

    //protocol
    p = strstr(args, "proto=");
    p += strlen("proto=");
    if(*p == DELIM) {//default, none
		pEntry->protoType = 0;
    } else {
		pEntry->protoType = strtol(p, &tmp, 0);
		if(*tmp != DELIM) return 1;
    }

    //dscp match
    p = strstr(args, "matchdscp=");
    p += strlen("matchdscp=");
    if(*p == DELIM) {//default
		pEntry->dscp = 0;
    } else {
		pEntry->dscp = strtol(p, &tmp, 0);
		if(*tmp != DELIM) return 1;
    }

    //ip source address
    p = strstr(args, "sip=");
    p += strlen("sip=");
    if(*p==DELIM) {//default
		pEntry->srcip =0;
    } else {
		for(idx=0; *p != DELIM&&idx<32; ++p)
		    buff[idx++] = *p;
		buff[idx] = '\0';
		{//ip address
		    struct in_addr addr;
		    inet_aton(buff, &addr);
		    pEntry->srcip = addr.s_addr;
		}
    }

    //source ip netmaskbit
    p = strstr(args, "smask=");
    p += strlen("smask=");
    if(*p==DELIM) {//default
		pEntry->smaskbits =0;
    } else {
		for(idx=0; *p != DELIM&&idx<32; ++p)
		    buff[idx++] = *p;
		buff[idx] = '\0';
		pEntry->smaskbits = getNetMaskBit(buff);
    }

    //ip dest address
    p = strstr(args, "dip=");
    p += strlen("dip=");
    if(*p==DELIM) {//default
		pEntry->dstip =0;
    } else {
		for(idx=0; *p != DELIM&&idx<32; ++p)
		    buff[idx++] = *p;
		buff[idx] = '\0';
		{//ip address
		    struct in_addr addr;
		    inet_aton(buff, &addr);
		    pEntry->dstip = addr.s_addr;
		}
    }

    //destination ip netmaskbit
    p = strstr(args, "dmask=");
    p += strlen("dmask=");
    if(*p==DELIM) {//default
		pEntry->dmaskbits =0;
    } else {
		for(idx=0; *p != DELIM&&idx<32; ++p)
		    buff[idx++] = *p;
		buff[idx] = '\0';
		pEntry->dmaskbits = getNetMaskBit(buff);
    }

    //src port start
    p = strstr(args, "spts=");
    p += strlen("spts=");
    if(*p==DELIM) {//default
		pEntry->srcPortStart =0;
    } else {
		pEntry->srcPortStart = strtol(p, &tmp, 0);
		if(*tmp != DELIM) return 1;
    }

    //src port end
    p = strstr(args, "spte=");
    p += strlen("spte=");
    if(*p==DELIM) {//default
		pEntry->srcPortEnd =0;
    } else {
		pEntry->srcPortEnd = strtol(p, &tmp, 0);
		if(*tmp != DELIM) return 1;
    }

    //dst port start
    p = strstr(args, "dpts=");
    p += strlen("dpts=");
    if(*p==DELIM) {//default
		pEntry->dstPortStart =0;
    } else {
		pEntry->dstPortStart = strtol(p, &tmp, 0);
		if(*tmp != DELIM) return 1;
    }

    //dst port end
    p = strstr(args, "dpte=");
    p += strlen("dpte=");
    if(*p==DELIM) {//default
		pEntry->dstPortEnd =0;
    } else {
		pEntry->dstPortEnd = strtol(p, &tmp, 0);
		if(*tmp != DELIM) return 1;
    }

    //vlan 802.1p match
    p = strstr(args, "vlan1p=");
    p += strlen("vlan1p=");
    if(*p=='\0') {//default
		pEntry->vlan1p =0;
    } else {
		pEntry->vlan1p = strtol(p, &tmp, 0);
		if(*tmp != '\0') return 1;
    }
    return 0;
}

void formQosRuleEdit(webs_t wp, char_t* path, char_t* query)
{
    MIB_CE_IP_QUEUE_RULE_T entry;
    char* action = NULL, args[256]={0}, *p = NULL, *tmp=NULL;
    char* act1="addrule", *act2="editrule", *url = NULL;
    unsigned char map[QUEUE_RULE_NUM_MAX+1]={0};
    int entryNum = 0, index=0,i=0;

    PRINT_FUNCTION

    action = websGetVar(wp, T("lst"), T(""));

    if(action[0])
	{
		entryNum = mib_chain_total(MIB_IP_QUEUE_RULE_TBL);

		if(!strncmp(action, act1, strlen(act1))) 
		{//add new one
		    if( entryNum>=QUEUE_RULE_NUM_MAX)
		    {
				ERR_MSG("You cannot add one new rule when queue is full.");
				return;
		    }
		    //get index for this rule
		    for(i=0;i<entryNum; i++) {
				if(!mib_chain_get(MIB_IP_QUEUE_RULE_TBL, i, &entry))
				    continue;
				map[entry.index] = 1;
		    }
		    for(i=1; i<=QUEUE_RULE_NUM_MAX; i++)
		    {
				if(!map[i])
				{
				    index = i;
				    break;
				}
				else if(i == QUEUE_RULE_NUM_MAX)
				{
				    ERR_MSG("You cannot add queue rule any more, pls delte some!\n");
				    return;
				}
		    }

		    //reset to zero
		    bzero(&entry, sizeof(MIB_CE_IP_QUEUE_RULE_T));

		    if(!parseRuleArgs(action, &entry))
		    {
				//index
				entry.index = index;

				PRINT_QUEUE_RULE((&entry));

				if(!mib_chain_add(MIB_IP_QUEUE_RULE_TBL, &entry))
			       	{
				    ERR_MSG("mib Error: Cannot add new one qos rule.");
				    return;
				}
		    }
		    else
		    {
				ERR_MSG("Wrong argument format!");
				return;
		    }
		    
		} else if(!strncmp(action, act2, strlen(act2))) {//update old one
		    MIB_CE_IP_QUEUE_RULE_T oldEntry;

		    //reset to zero
		    bzero(&entry, sizeof(MIB_CE_IP_QUEUE_RULE_T));

		    if(parseRuleArgs(action, &entry))
		    {
				ERR_MSG("Wrong argument format!");
				return;
		    }

		    PRINT_QUEUE_RULE((&entry));

		    //find the mib item
		    for(i=0; i<entryNum; i++)
		    {
				if(!mib_chain_get(MIB_IP_QUEUE_RULE_TBL, i, &oldEntry))
				    continue;
				if(oldEntry.index == entry.index)
				{//found it, update it
				    entry.InstanceNum=oldEntry.InstanceNum;
				    entry.modeTr69=oldEntry.modeTr69;
				    entry.appindex=oldEntry.appindex;
				    if(!mib_chain_update(MIB_IP_QUEUE_RULE_TBL, &entry, i))
				    {
						ERR_MSG("Updating qos rule into mib is wrong!");
						return;
				    }
				}
		    }
		} else {//undefined operation
		    ERR_MSG("Wrong operation happened! you only and add or edit qos rule in this page!");
		    return;
		}

	}
     mib_update(CURRENT_SETTING, CONFIG_MIB_ALL);
    //redirect
    url = websGetVar(wp, T("submit-url"),T(""));
    if(url[0]) 
    {
	websRedirect(wp, url);
    }

    return;
}

