/*
 *      Web server handler routines for APP
 *      Authors: 
 *
 */

/*-- System inlcude files --*/
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <sys/wait.h>
#include <semaphore.h>
#ifdef EMBED
#include <linux/config.h>
#else
#include "../../../uClibc/include/linux/autoconf.h"
#endif

/* for ioctl */
#include <sys/ioctl.h>
#include <net/if.h>

#include "../webs.h"
#include "fmdefs.h"
#include "mib.h"
#include "utility.h"
#include "debug.h"
#include "../defs.h"
#include "adsl_drv.h"


/*for debug*/
#define MIB_WLAN_XXX 1

#define	_VAR_FLAG_MASK		0x0000F000	/*flag mask*/
#define	_VAR_FLAG_MIB		0x00000000	/*mib database*/
#define	_VAR_FLAG_CSM		0x00004000	/*customer*/
#define _VAR_FLAG_FNC		0x00008000	/*function*/
#define	_VAR_TYPE_MASK		0x0000000F	/*type mask*/
#define _VAR_TYPE_NIL		0x00000000	/*null*/
#define _VAR_TYPE_SWITCH	0x00000001	/*enable or disable*/
#define _VAR_TYPE_BOOL		0x00000002	/*true or false*/
#define _VAR_TYPE_METHOD	0x00000003	/*auto or manual*/
#define _VAR_TYPE_INT		0x00000004	/*integer*/
#define _VAR_TYPE_STR		0x00000005	/*string*/
#define _VAR_TYPE_IP		0x00000006	/*ip*/
#define _VAR_TYPE_MAC		0x00000007	/*mac*/

struct cgivarinfo;
typedef int (*_fnvar)(int eid, webs_t wp, int argc, char_t ** argv, struct cgivarinfo* pinfo, int iscgi);

typedef struct cgivarinfo
{
	char*		key;
	int			flag;
	union
	{
		int		id;
		int		val;
		char*	str;
		_fnvar	fn;
	};
	int			data;
}cgivarinfo;

int adslgetvar(int eid, webs_t wp, int argc, char_t ** argv, struct cgivarinfo* pinfo, int iscgi)
{
	int				len = 0;
	unsigned char	buffer[255];

	if(pinfo == NULL)return -1;
	
	memset(buffer, 0, sizeof(buffer));
	
	len = getAdslInfo(pinfo->data, buffer, sizeof(buffer) - 1);
	if(len <= 0)return -1;
	buffer[len] = 0;

	switch(pinfo->flag & _VAR_TYPE_MASK)
	{
	case _VAR_TYPE_SWITCH:
		{
			if(iscgi) return websWrite(wp, T("cgi.%s = \"%s\";"), pinfo->key, (*(char*)buffer) ? "enable" : "disable");
			else return websWrite(wp, T("%s"), (*(char*)buffer) ? "Disable" : "Enable");
		}break;
	case _VAR_TYPE_BOOL:
		{
			if(iscgi) return websWrite(wp, T("cgi.%s = %s;"), pinfo->key, (*(char*)buffer) ? "true" : "false");
			else return websWrite(wp, T("%s"), (*(char*)buffer) ? "Disable" : "Enable");
		}break;
	case _VAR_TYPE_METHOD:
		{
			if(iscgi) return websWrite(wp, T("cgi.%s = \"%s\";"), pinfo->key, (*(char*)buffer) ? "auto" : "manual");
			else return websWrite(wp, T("%s"), (*(char*)buffer) ? "auto" : "manual");
		}break;
	case _VAR_TYPE_INT:
		{
			if(iscgi) return websWrite(wp, T("cgi.%s = %u;"), pinfo->key, *(unsigned int *)buffer);
			else return websWrite(wp, T("%u"), *(unsigned int *)buffer);
		}break;
	case _VAR_TYPE_STR:
		{
			if(iscgi) return websWrite(wp, T("cgi.%s = \"%s\";"), pinfo->key, (char*)buffer);
			else return websWrite(wp, T("%s"), (char*)buffer);
		}break;
	case _VAR_TYPE_IP:
		{
			if(iscgi) return websWrite(wp, T("cgi.%s = \"%s\";"), pinfo->key, inet_ntoa(*(struct in_addr*)buffer));
			else return websWrite(wp, T("%s"), inet_ntoa(*(struct in_addr*)buffer));
		}break;
	case _VAR_TYPE_MAC:
		{
			if(iscgi) return websWrite(wp, T("cgi.%s = \"%02X-%02X-%02X-%02X-%02X-%02X\";"), pinfo->key, buffer[0], buffer[1], buffer[2], buffer[3], buffer[4], buffer[5]);
			else return websWrite(wp, T("%02X-%02X-%02X-%02X-%02X-%02X"), buffer[0], buffer[1], buffer[2], buffer[3], buffer[4], buffer[5]);
		}break;
	}

	return 0;
}

int adslgetvar_lineProto(int eid, webs_t wp, int argc, char_t ** argv, struct cgivarinfo* pinfo, int iscgi)
{
	int				len = 0;
	short			mode = 0;
	unsigned char	buffer[255];

	if(pinfo == NULL)return -1;
	
	memset(buffer, 0, sizeof(buffer));

    mib_get(MIB_ADSL_MODE, (void *)&mode );

    if ( mode & ADSL_MODE_GLITE )
	strcat(buffer, "G.Lite ");

    if ( mode & ADSL_MODE_T1413 )
	strcat(buffer, "T1.413 ");

    if ( mode & ADSL_MODE_GDMT )
	strcat(buffer, "G.Dmt ");

    if ( mode & ADSL_MODE_ADSL2 )
	strcat(buffer, "ADSL2 ");

    if ( mode & ADSL_MODE_ADSL2P )
	strcat(buffer, "ADSL2+ ");

    return iscgi ? websWrite(wp, T("cgi.%s = \"%s\";"), pinfo->key, (char*)buffer) : websWrite(wp, T("%s"), buffer);
}

int sysgetvar(int eid, webs_t wp, int argc, char_t ** argv, struct cgivarinfo* pinfo, int iscgi)
{
	int				len = 0;
	unsigned char	buffer[255];

	if(pinfo == NULL)return -1;
	
	memset(buffer, 0, sizeof(buffer));

	len = getSYS2Str(pinfo->data, buffer);
	if(len <= 0)return -1;
	buffer[len] = 0;

	return iscgi ? websWrite(wp, T("cgi.%s = \"%s\";"), pinfo->key, (char*)buffer) : websWrite(wp, T("%s"), buffer);
}

static cgivarinfo	map_vars[] = {
	{"mf",					_VAR_FLAG_CSM | _VAR_TYPE_STR,	(int)"ASB", 0},
	{"strModel",			_VAR_FLAG_CSM | _VAR_TYPE_STR,	(int)"RG100A", 0},
	{"strId",				_VAR_FLAG_CSM | _VAR_TYPE_STR,	(int)"03100200000100100000007404010203", 0},
	{"ip",					_VAR_FLAG_CSM | _VAR_TYPE_STR,	(int)"192.168.2.1", 0},
	{"strHdVer",			_VAR_FLAG_CSM | _VAR_TYPE_STR,	(int)"V1.0", 0},
	{"strStVer",			_VAR_FLAG_CSM | _VAR_TYPE_STR,	(int)"RG100A_V1.0", 0},
	{"dfGw",				_VAR_FLAG_CSM | _VAR_TYPE_IP,	MIB_WLAN_XXX, 0},

	//add by liuxiao 2008-01-16
	{"lanip",				_VAR_FLAG_MIB | _VAR_TYPE_IP,	MIB_ADSL_LAN_IP, 0},
	{"subnetMask",			_VAR_FLAG_MIB | _VAR_TYPE_IP,	MIB_ADSL_LAN_SUBNET, 0},
	{"lanmac",				_VAR_FLAG_MIB | _VAR_TYPE_MAC,	MIB_ELAN_MAC_ADDR, 0},
	{"dns1",				_VAR_FLAG_MIB | _VAR_TYPE_IP,	MIB_ADSL_WAN_DNS1, 0},
	{"dns2",				_VAR_FLAG_MIB | _VAR_TYPE_IP,	MIB_ADSL_WAN_DNS2, 0},
	//add end by liuxiao 2008-01-29

	//adsl status..............
	//{"linkState",	...... ??
	{"upRate",				_VAR_FLAG_FNC | _VAR_TYPE_STR,	(int)adslgetvar, ADSL_GET_RATE_US},
	{"downRate",			_VAR_FLAG_FNC | _VAR_TYPE_STR,	(int)adslgetvar, ADSL_GET_RATE_DS},
	{"syncState",			_VAR_FLAG_FNC | _VAR_TYPE_STR,	(int)adslgetvar, ADSL_GET_STATE},
	{"vsnrMargin_u",		_VAR_FLAG_FNC | _VAR_TYPE_STR,	(int)adslgetvar, ADSL_GET_SNR_US},
	{"vsnrMargin_d",		_VAR_FLAG_FNC | _VAR_TYPE_STR,	(int)adslgetvar, ADSL_GET_SNR_DS},
	{"vpmsPramD_u",			_VAR_FLAG_FNC | _VAR_TYPE_STR,	(int)adslgetvar, ADSL_GET_D_US},
	{"vpmsPramD_d",			_VAR_FLAG_FNC | _VAR_TYPE_STR,	(int)adslgetvar, ADSL_GET_D_DS},
	{"lineProto",			_VAR_FLAG_FNC | _VAR_TYPE_STR,	(int)adslgetvar_lineProto, MIB_ADSL_MODE},
	{"attenuation_u",		_VAR_FLAG_FNC | _VAR_TYPE_STR,	(int)adslgetvar, ADSL_GET_LPATT_US},
	{"attenuation_d",		_VAR_FLAG_FNC | _VAR_TYPE_STR,	(int)adslgetvar, ADSL_GET_LPATT_DS},
	{"outputPower_u",		_VAR_FLAG_FNC | _VAR_TYPE_STR,	(int)adslgetvar, ADSL_GET_POWER_US},
	{"outputPower_d",		_VAR_FLAG_FNC | _VAR_TYPE_STR,	(int)adslgetvar, ADSL_GET_POWER_DS},
	{"pmsLatency",			_VAR_FLAG_FNC | _VAR_TYPE_STR,	(int)adslgetvar, ADSL_GET_LATENCY},
	//add end by liuxiao 2008-01-26

	{"uptime",				_VAR_FLAG_FNC | _VAR_TYPE_STR,	(int)sysgetvar, SYS_UPTIME},
	{"date",				_VAR_FLAG_FNC | _VAR_TYPE_STR,	(int)sysgetvar, SYS_DATE},
	{"year",				_VAR_FLAG_FNC | _VAR_TYPE_STR,	(int)sysgetvar, SYS_YEAR},
	{"month", 				_VAR_FLAG_FNC | _VAR_TYPE_STR,	(int)sysgetvar, SYS_MONTH},
	{"day", 				_VAR_FLAG_FNC | _VAR_TYPE_STR,	(int)sysgetvar, SYS_DAY},
	{"hour", 				_VAR_FLAG_FNC | _VAR_TYPE_STR,	(int)sysgetvar, SYS_HOUR},
	{"minute", 				_VAR_FLAG_FNC | _VAR_TYPE_STR,	(int)sysgetvar, SYS_MINUTE},
	{"second", 				_VAR_FLAG_FNC | _VAR_TYPE_STR,	(int)sysgetvar, SYS_SECOND},
	{"fwVersion", 			_VAR_FLAG_FNC | _VAR_TYPE_STR,	(int)sysgetvar, SYS_FWVERSION},

	//wlan status..............
	{"wlanState",			_VAR_FLAG_MIB | _VAR_TYPE_BOOL,	MIB_WLAN_XXX, 0},			//״̬
	{"wlDefChannel",		_VAR_FLAG_MIB | _VAR_TYPE_INT,	MIB_WLAN_XXX, 0},			//ŵ
	{"wlTxRate",			_VAR_FLAG_MIB | _VAR_TYPE_INT,	MIB_WLAN_XXX, 0},			//54gTM
	{"wlMulticastRate",		_VAR_FLAG_MIB | _VAR_TYPE_INT,	MIB_WLAN_XXX, 0},			//ಥ
	{"wlanMode",			_VAR_FLAG_MIB | _VAR_TYPE_INT,	MIB_WLAN_XXX, 0},			//54gTMģʽ, //1- 54g Auto;  2- 54g Performance; 3- 54g LRS;  4- 802.11b Only
	{"wlTxPower",			_VAR_FLAG_MIB | _VAR_TYPE_INT,	MIB_WLAN_XXX, 0},			//͹ //0- 100%; 1- 80%; 2- 50%; 3- 25%; 4-10%
	{"wlanBssid",			_VAR_FLAG_MIB | _VAR_TYPE_STR,	MIB_WLAN_XXX, 0},			//BSSID
	{"wlanSsidAttr",		_VAR_FLAG_MIB | _VAR_TYPE_STR,	MIB_WLAN_XXX, 0},			//SSID
	{"ssidName",			_VAR_FLAG_MIB | _VAR_TYPE_STR,	MIB_WLAN_XXX, 0},			//SSID-1
	{"encryptState",		_VAR_FLAG_MIB | _VAR_TYPE_STR,	MIB_WLAN_XXX, 0},			//SSID-1״̬:

	//wlan status..............
	{"usbstate",			_VAR_FLAG_MIB | _VAR_TYPE_BOOL,	MIB_WLAN_XXX, 0},			//USB״̬


	{"lan-ip",				_VAR_FLAG_MIB | _VAR_TYPE_IP,	MIB_ADSL_LAN_IP, 0},
	{"lan-subnet",			_VAR_FLAG_MIB | _VAR_TYPE_IP,	MIB_ADSL_LAN_SUBNET, 0},
	{"lan-dhcpRangeStart",	_VAR_FLAG_MIB | _VAR_TYPE_IP,	MIB_ADSL_LAN_CLIENT_START, 0},
	{"lan-dhcpRangeEnd",	_VAR_FLAG_MIB | _VAR_TYPE_IP,	MIB_ADSL_LAN_CLIENT_END, 0},
	{"lan-dhcpLTime",		_VAR_FLAG_MIB | _VAR_TYPE_INT,	MIB_ADSL_LAN_DHCP_LEASE, 0},
	{"lan-dhcpDName",		_VAR_FLAG_MIB | _VAR_TYPE_STR,	MIB_ADSL_LAN_DHCP_DOMAIN, 0},
	{"elan-Mac",			_VAR_FLAG_MIB | _VAR_TYPE_MAC,	MIB_ELAN_MAC_ADDR, 0},
	{"wlan-Mac",			_VAR_FLAG_MIB | _VAR_TYPE_MAC,	MIB_WLAN_MAC_ADDR, 0},
	{"wan-dns1",			_VAR_FLAG_MIB | _VAR_TYPE_IP,	MIB_ADSL_WAN_DNS1, 0},
	{"wan-dns2",			_VAR_FLAG_MIB | _VAR_TYPE_IP,	MIB_ADSL_WAN_DNS2, 0},
	{"wan-dns3",			_VAR_FLAG_MIB | _VAR_TYPE_IP,	MIB_ADSL_WAN_DNS3, 0},
	{"wan-dhcps",			_VAR_FLAG_MIB | _VAR_TYPE_IP,	MIB_ADSL_WAN_DHCPS, 0},
	{"dmzHost",				_VAR_FLAG_MIB | _VAR_TYPE_IP,	MIB_DMZ_IP, 0},
	{"snmpSysDescr",		_VAR_FLAG_MIB | _VAR_TYPE_STR,	MIB_SNMP_SYS_DESCR, 0},
	{"snmpSysContact",		_VAR_FLAG_MIB | _VAR_TYPE_STR,	MIB_SNMP_SYS_CONTACT, 0},
	{"snmpSysName",			_VAR_FLAG_MIB | _VAR_TYPE_STR,	MIB_SNMP_SYS_NAME, 0},
	{"snmpSysLocation",		_VAR_FLAG_MIB | _VAR_TYPE_STR,	MIB_SNMP_SYS_LOCATION, 0},
	{"snmpSysObjectID",		_VAR_FLAG_MIB | _VAR_TYPE_STR,	MIB_SNMP_SYS_OID, 0},
	{"snmpTrapIpAddr",		_VAR_FLAG_MIB | _VAR_TYPE_IP,	MIB_SNMP_TRAP_IP, 0},
	{"snmpCommunityRO",		_VAR_FLAG_MIB | _VAR_TYPE_STR,	MIB_SNMP_COMM_RO, 0},
	{"snmpCommunityRW",		_VAR_FLAG_MIB | _VAR_TYPE_STR,	MIB_SNMP_COMM_RW, 0},
	{"name",				_VAR_FLAG_MIB | _VAR_TYPE_STR,	MIB_SNMP_SYS_NAME, 0},

	{"bridge-ageingTime",	_VAR_FLAG_MIB | _VAR_TYPE_STR,	MIB_BRCTL_AGEINGTIME, 0},
#ifdef CONFIG_USER_IGMPPROXY
	{"igmp-proxy-itf",		_VAR_FLAG_MIB | _VAR_TYPE_STR,	MIB_IGMP_PROXY_ITF, 0},
#endif
//#ifdef CONFIG_USER_UPNPD	
#if defined(CONFIG_USER_UPNPD)||defined(CONFIG_USER_MINIUPNPD)
	{"upnp-ext-itf",		_VAR_FLAG_MIB | _VAR_TYPE_STR,	MIB_UPNP_EXT_ITF, 0},
#endif	
#ifdef AUTO_PROVISIONING
	{"http-ip",				_VAR_FLAG_MIB | _VAR_TYPE_STR,	MIB_HTTP_SERVER_IP, 0},
#endif
#ifdef IP_PASSTHROUGH
	{"ippt-itf",  			_VAR_FLAG_MIB | _VAR_TYPE_STR,	MIB_IPPT_ITF, 0},
	{"ippt-lease",			_VAR_FLAG_MIB | _VAR_TYPE_STR,	MIB_IPPT_LEASE, 0},
	{"ippt-lanacc",			_VAR_FLAG_MIB | _VAR_TYPE_STR,	MIB_IPPT_LANACC, 0},
#endif
#ifdef WLAN_SUPPORT
	{"ssid",  				_VAR_FLAG_MIB | _VAR_TYPE_STR,	MIB_WLAN_SSID, 0},
	{"channel",  			_VAR_FLAG_MIB | _VAR_TYPE_STR,	MIB_WLAN_CHAN_NUM, 0},
	{"fragThreshold",		_VAR_FLAG_MIB | _VAR_TYPE_STR,	MIB_WLAN_FRAG_THRESHOLD, 0},
	{"rtsThreshold",		_VAR_FLAG_MIB | _VAR_TYPE_STR,	MIB_WLAN_RTS_THRESHOLD, 0},
	{"beaconInterval",		_VAR_FLAG_MIB | _VAR_TYPE_STR,	MIB_WLAN_BEACON_INTERVAL, 0},
	{"wlanDisabled",		_VAR_FLAG_MIB | _VAR_TYPE_STR,	MIB_WLAN_DISABLED, 0},

	{"pskValue", 			_VAR_FLAG_MIB | _VAR_TYPE_STR,	SYS_WLAN_PSKVAL, 0},
#ifdef WLAN_1x
	{"rsPort",				_VAR_FLAG_MIB | _VAR_TYPE_STR,	MIB_WLAN_RS_PORT, 0},
	{"rsIp",				_VAR_FLAG_MIB | _VAR_TYPE_STR,	MIB_WLAN_RS_IP, 0},
	{"rsPassword",			_VAR_FLAG_MIB | _VAR_TYPE_STR,	MIB_WLAN_RS_PASSWORD, 0},
	{"enable1X", 			_VAR_FLAG_MIB | _VAR_TYPE_STR,	MIB_WLAN_ENABLE_1X, 0},
#endif
	{"wlanMode",			_VAR_FLAG_MIB | _VAR_TYPE_STR,	MIB_WLAN_MODE, 0},
	{"encrypt",				_VAR_FLAG_MIB | _VAR_TYPE_STR,	MIB_WLAN_ENCRYPT, 0},
	{"wpaAuth",				_VAR_FLAG_MIB | _VAR_TYPE_STR,	MIB_WLAN_WPA_AUTH, 0},
	{"networkType",			_VAR_FLAG_MIB | _VAR_TYPE_STR,	MIB_WLAN_NETWORK_TYPE, 0},

#ifdef CONFIG_WIFI_SIMPLE_CONFIG // WPS
	{"wscDisable",			_VAR_FLAG_MIB | _VAR_TYPE_STR,	MIB_WSC_DISABLE, 0},
	//{"wscConfig",			_VAR_FLAG_MIB | _VAR_TYPE_STR,	MIB_WSC_CONFIGURED, 0},
	{"wps_auth",			_VAR_FLAG_MIB | _VAR_TYPE_STR,	MIB_WSC_AUTH, 0},
	{"wps_enc",				_VAR_FLAG_MIB | _VAR_TYPE_STR,	MIB_WSC_ENC, 0},
	{"wscLoocalPin", 		_VAR_FLAG_MIB | _VAR_TYPE_STR,	MIB_WSC_PIN, 0},

#endif

#ifdef WLAN_WDS
	{"wlanWdsEnabled",		_VAR_FLAG_MIB | _VAR_TYPE_STR,	MIB_WLAN_WDS_ENABLED, 0},
#endif
#endif
#ifdef _CWMP_MIB_
	{"acs-url", 			_VAR_FLAG_MIB | _VAR_TYPE_STR,	CWMP_ACS_URL, 0},
	{"acs-username",		_VAR_FLAG_MIB | _VAR_TYPE_STR,	CWMP_ACS_USERNAME, 0},
	{"acs-password",		_VAR_FLAG_MIB | _VAR_TYPE_STR,	CWMP_ACS_PASSWORD, 0},
	{"inform-interval",		_VAR_FLAG_MIB | _VAR_TYPE_STR,	CWMP_INFORM_INTERVAL, 0},
	{"conreq-name",			_VAR_FLAG_MIB | _VAR_TYPE_STR,	CWMP_CONREQ_USERNAME, 0},
	{"conreq-pw", 			_VAR_FLAG_MIB | _VAR_TYPE_STR,	CWMP_CONREQ_PASSWORD, 0},
	{"cert-pw", 			_VAR_FLAG_MIB | _VAR_TYPE_STR,	CWMP_CERT_PASSWORD, 0},
#endif
#ifdef DOS_SUPPORT
	{"syssynFlood",			_VAR_FLAG_MIB | _VAR_TYPE_STR,	MIB_DOS_SYSSYN_FLOOD, 0},
	{"sysfinFlood",			_VAR_FLAG_MIB | _VAR_TYPE_STR,	MIB_DOS_SYSFIN_FLOOD, 0},
	{"sysudpFlood",			_VAR_FLAG_MIB | _VAR_TYPE_STR,	MIB_DOS_SYSUDP_FLOOD, 0},
	{"sysicmpFlood",		_VAR_FLAG_MIB | _VAR_TYPE_STR,	MIB_DOS_SYSICMP_FLOOD, 0},
	{"pipsynFlood",			_VAR_FLAG_MIB | _VAR_TYPE_STR,	MIB_DOS_PIPSYN_FLOOD, 0},
	{"pipfinFlood",			_VAR_FLAG_MIB | _VAR_TYPE_STR,	MIB_DOS_PIPFIN_FLOOD, 0},
	{"pipudpFlood",			_VAR_FLAG_MIB | _VAR_TYPE_STR,	MIB_DOS_PIPUDP_FLOOD, 0},
	{"pipicmpFlood",		_VAR_FLAG_MIB | _VAR_TYPE_STR,	MIB_DOS_PIPICMP_FLOOD, 0},
	{"blockTime",			_VAR_FLAG_MIB | _VAR_TYPE_STR,	MIB_DOS_BLOCK_TIME, 0},
#endif
	{NULL, 0, 0, 0}
};

int getvar(int eid, webs_t wp, int argc, char_t ** argv)
{
	cgivarinfo*	pinfo = NULL;
	char*		key = NULL;
	int			index = 0;
	unsigned char buffer[255];

   	if (ejArgs(argc, argv, T("%s"), &key) < 1 || key == NULL || *key == 0) 
	{
		return -1;
   	}

	for(index = 0; map_vars[index].key; index++)
	{
		if(strcmp(key, map_vars[index].key) == 0)
		{
			pinfo = &(map_vars[index]);
			break;
		}
	}

	if(pinfo == NULL)return -1;

	if(pinfo->flag & _VAR_FLAG_FNC)
	{
		return pinfo->fn(eid, wp, argc - 1, &(argv[1]), pinfo, 0);
	}

	switch(pinfo->flag & _VAR_FLAG_MASK)
	{
	case _VAR_FLAG_MIB:
		{
			memset(buffer, 0, sizeof(buffer));
			if(!mib_get(pinfo->id, (void *)buffer))
				return -1;

			switch(pinfo->flag & _VAR_TYPE_MASK)
			{
			case _VAR_TYPE_SWITCH:
				{
					return websWrite(wp, T("%s"), (*(char*)buffer) ? "enable" : "disable");
				}break;
			case _VAR_TYPE_BOOL:
				{
					return websWrite(wp, T("%s"), (*(char*)buffer) ? "true" : "false");
				}break;
			case _VAR_TYPE_METHOD:
				{
					return websWrite(wp, T("%s"), (*(char*)buffer) ? "auto" : "manual");
				}break;
			case _VAR_TYPE_INT:
				{
					return websWrite(wp, T("%u"), *(unsigned int *)buffer);
				}break;
			case _VAR_TYPE_STR:
				{
					return websWrite(wp, T("%s"), (char*)buffer);
				}break;
			case _VAR_TYPE_IP:
				{
					return websWrite(wp, T("%s"), inet_ntoa(*(struct in_addr*)buffer));
				}break;
			case _VAR_TYPE_MAC:
				{
					return websWrite(wp, T("%02X-%02X-%02X-%02X-%02X-%02X"), buffer[0], buffer[1], buffer[2], buffer[3], buffer[4], buffer[5]);
				}break;
			}
		}break;
	case _VAR_FLAG_CSM:
		{
			switch(pinfo->flag & _VAR_TYPE_MASK)
			{
			case _VAR_TYPE_SWITCH:
				{
					return websWrite(wp, T("%s"), pinfo->val ? "enable" : "disable");
				}break;
			case _VAR_TYPE_BOOL:
				{
					return websWrite(wp, T("%s"), pinfo->val ? "true" : "false");
				}break;
			case _VAR_TYPE_METHOD:
				{
					return websWrite(wp, T("%s"), pinfo->val ? "auto" : "manual");
				}break;
			case _VAR_TYPE_INT:
				{
					return websWrite(wp, T("%u"), pinfo->val);
				}break;
			case _VAR_TYPE_STR:
				{
					return websWrite(wp, T("%s"), pinfo->str);
				}break;
			case _VAR_TYPE_IP:
				{
					return websWrite(wp, T("%s"), inet_ntoa(*(struct in_addr*)&(pinfo->val)));
				}break;
			case _VAR_TYPE_MAC:
				{
					return websWrite(wp, T("%s"), pinfo->str);
				}break;
			}
		}break;
	}

	return 0;
}

int cgivar(int eid, webs_t wp, int argc, char_t ** argv)
{
	cgivarinfo*	pinfo = NULL;
	char*		key = NULL;
	int			index = 0;
	unsigned char buffer[255];

   	if (ejArgs(argc, argv, T("%s"), &key) < 1 || key == NULL || *key == 0) 
	{
		return -1;
   	}

	for(index = 0; map_vars[index].key; index++)
	{
		if(strcmp(key, map_vars[index].key) == 0)
		{
			pinfo = &(map_vars[index]);
			break;
		}
	}

	if(pinfo == NULL)return -1;

	if(pinfo->flag & _VAR_FLAG_FNC)
	{
		return pinfo->fn(eid, wp, argc - 1, &(argv[1]), pinfo, 1);
	}

	switch(pinfo->flag & _VAR_FLAG_MASK)
	{
	case _VAR_FLAG_MIB:
		{
			memset(buffer, 0, sizeof(buffer));
			if(!mib_get(pinfo->id, (void *)buffer))
				return -1;

			switch(pinfo->flag & _VAR_TYPE_MASK)
			{
			case _VAR_TYPE_SWITCH:
				{
					return websWrite(wp, T("cgi.%s = \"%s\";"), pinfo->key, (*(char*)buffer) ? "enable" : "disable");
				}break;
			case _VAR_TYPE_BOOL:
				{
					return websWrite(wp, T("cgi.%s = %s;"), pinfo->key, (*(char*)buffer) ? "true" : "false");
				}break;
			case _VAR_TYPE_METHOD:
				{
					return websWrite(wp, T("cgi.%s = \"%s\";"), pinfo->key, (*(char*)buffer) ? "auto" : "manual");
				}break;
			case _VAR_TYPE_INT:
				{
					return websWrite(wp, T("cgi.%s = %u;"), pinfo->key, *(unsigned int *)buffer);
				}break;
			case _VAR_TYPE_STR:
				{
					return websWrite(wp, T("cgi.%s = \"%s\";"), pinfo->key, (char*)buffer);
				}break;
			case _VAR_TYPE_IP:
				{
					return websWrite(wp, T("cgi.%s = \"%s\";"), pinfo->key, inet_ntoa(*(struct in_addr*)buffer));
				}break;
			case _VAR_TYPE_MAC:
				{
					return websWrite(wp, T("cgi.%s = \"%02X-%02X-%02X-%02X-%02X-%02X\";"), pinfo->key, buffer[0], buffer[1], buffer[2], buffer[3], buffer[4], buffer[5]);
				}break;
			}
		}break;
	case _VAR_FLAG_CSM:
		{
			switch(pinfo->flag & _VAR_TYPE_MASK)
			{
			case _VAR_TYPE_SWITCH:
				{
					return websWrite(wp, T("cgi.%s = \"%s\";"), pinfo->key, pinfo->val ? "enable" : "disable");
				}break;
			case _VAR_TYPE_BOOL:
				{
					return websWrite(wp, T("cgi.%s = \"%s\";"), pinfo->key, pinfo->val ? "true" : "false");
				}break;
			case _VAR_TYPE_METHOD:
				{
					return websWrite(wp, T("cgi.%s = \"%s\";"), pinfo->key, pinfo->val ? "auto" : "manual");
				}break;
			case _VAR_TYPE_INT:
				{
					return websWrite(wp, T("cgi.%s = %u;"), pinfo->key, pinfo->val);
				}break;
			case _VAR_TYPE_STR:
				{
					return websWrite(wp, T("cgi.%s = \"%s\";"), pinfo->key, pinfo->str);
				}break;
			}
			case _VAR_TYPE_IP:
				{
					return websWrite(wp, T("cgi.%s = \"%s\";"), pinfo->key, inet_ntoa(*(struct in_addr*)&(pinfo->val)));
				}break;
			case _VAR_TYPE_MAC:
				{
					return websWrite(wp, T("cgi.%s = \"%s\";"), pinfo->key, (char*)buffer);
				}break;
		}break;
	}

	return 0;
}
