/*
 *      System routines for Port-mapping
 *
 */

#include <string.h>
#include "debug.h"
#include "utility.h"
#include "wireless.h"

/*
 * The bitmap for interface should follow this convention commonly shared
 * by sar driver (sar_send())
 * Bit-map:
 *  (bit11) | bit10 | bit9  | bit8  | bit7 | bit6  | bit5   |  bit4    |  bit3  |  bit2  |  bit1  |  bit0
 *  (usb0)  | vap3  | vap2  | vap1  | vap0 | wlan  | resvd  | device   |  lan3  |  lan2  |  lan1  |  lan0
 * If usb(switch port) add:
 *  (bit11) | bit10 | bit9  | bit8  | bit7 | bit6  | bit5   |   bit4   |  bit3  |  bit2  |  bit1  |  bit0
 *  (usb0)  | vap3  | vap2  | vap1  | vap0 | wlan  | device | lan4(usb)|  lan3  |  lan2  |  lan1  |  lan0
 */

#ifdef CONFIG_EXT_SWITCH
// enable rtl-8305 VLAN
static void setup_8305_vlan_enable()
{
	struct ifreq ifr;
	struct ifvlan ifvl;

	strcpy(ifr.ifr_name, "eth0");
	ifr.ifr_data = (char *)&ifvl;
	// enable rtl-8305 VLAN
	ifvl.cmd=VLAN_ENABLE;
	ifvl.enable=1;
	setVlan(&ifr);
	TRACE(STA_SCRIPT, "RTL8305 VLAN enabled\n");
}


// disable rtl-8305 VLAN
static void setup_8305_vlan_disable()
{
	struct ifreq ifr;
	struct ifvlan ifvl;

	strcpy(ifr.ifr_name, "eth0");
	ifr.ifr_data = (char *)&ifvl;
	// disable rtl-8305 VLAN
	ifvl.cmd=VLAN_ENABLE;
	ifvl.enable=0;
	setVlan(&ifr);
	TRACE(STA_SCRIPT, "RTL8305 VLAN disabled\n");
}
#endif // of CONFIG_EXT_SWITCH

#ifdef ITF_GROUP
#ifdef CONFIG_USB_ETH
#define SIOCSITFGROUP	0x89b6      // Mason Yu 090903

static void setup_usb_group_member(int mbr)
{
	struct ifreq ifr;
	printf( "setup_usb_group_member> mbr=0x%x\n", mbr );
	strcpy(ifr.ifr_name, USBETHIF);
	ifr.ifr_ifru.ifru_ivalue = mbr;
	do_ioctl( SIOCSITFGROUP, &ifr );
	return;
}
#endif

#ifdef WLAN_SUPPORT
#define SIOSIWRTLITFGROUP		0x8B90

static void setup_wlan_group_member(int itf, int mbr)
{
	int skfd;
	struct iwreq wrq;
	int k;

	skfd = socket(AF_INET, SOCK_DGRAM, 0);

#ifndef WLAN_MBSSID
	/* Set device name */
	strncpy(wrq.ifr_name, (char *)WLANIF, IFNAMSIZ);
	wrq.u.data.flags = 0;
#else
	strncpy(wrq.ifr_name, (char *)WLANAPIF[itf], IFNAMSIZ);
	wrq.u.data.flags = itf;

#endif
	wrq.u.data.pointer = (caddr_t)&mbr;
	wrq.u.data.length = sizeof(mbr);

	ioctl(skfd, SIOSIWRTLITFGROUP, &wrq);
	close( skfd );
}
#endif // of WLAN_SUPPORT

/*------------------------------------------------------------------------
 *	Setup RTL8305 Vlan membership.
 *	Enable port-based Vlan and set the vlan membership with
 *	the following pre-requisite:
 *
 *	port		vlanIdx		Vlan	VID
 *	0		0		A	0
 *	1		1		B	1
 *	2		2		C	2
 *	3		3		D	3
 *
 *------------------------------------------------------------------------*/
static void setup_8305_vlan_eth_member(int port, int mbr)
{
	int k;
	struct ifreq ifr;
	struct ifvlan ifvl;

	strcpy(ifr.ifr_name, "eth0");
	ifr.ifr_data = (char *)&ifvl;

	// set 8305 port-based vlan members
	// port		vlanIdx		Vlan	VID
	// 0		0		A	0
	// 1		1		B	1
	// 2		2		C	2
	// 3		3		D	3
	ifvl.cmd=VLAN_SETINFO;

	// This port is a member ?
	if (mbr & (1<<port)) {
		// set the membership
		ifvl.vlanIdx=port;
		// VID 0 is used to identify priority frames and VID 4095(0xFFF) is reserved
		ifvl.vid=port+1;
		ifvl.member=mbr;

#ifdef CONFIG_USB_ETH
		ifvl.member|=( mbr&(1<<IFUSBETH_SHIFT) );
#endif //CONFIG_USB_ETH

		setVlan(&ifr);
		TRACE(STA_SCRIPT, "RTL8305 VLAN: vlanIdx=%d, vid=%d, member=0x%.2x\n", ifvl.vlanIdx, ifvl.vid, ifvl.member);
	} else {
		// Added by Mason Yu
		// set the membership
		ifvl.vlanIdx=port;
		// VID 0 is used to identify priority frames and VID 4095(0xFFF) is reserved
		ifvl.vid=port+1;

#ifdef CONFIG_USBCLIENT
		ifvl.member=0x3f;
#else
		ifvl.member=0x1f;
#endif
		setVlan(&ifr);
	}
}

void __dev_setupPortMapping(int flag)
{
	struct ifreq ifr;
	struct ifvlan ifvl;

	strcpy(ifr.ifr_name, "eth0");
	ifr.ifr_data = (char *)&ifvl;
	ifvl.cmd=VLAN_SETPORTMAPPING;
	ifvl.enable=flag;
	setVlan(&ifr);
	//printf("Port Mapping: %s\n", flag?"enabled":"disabled");
}

void setgroup(char *list, int grpnum)
{
	int itfid, itfdomain;
	int i, num;
	char *arg0, *token;

	arg0=list;
	while ((token=strtok(arg0,","))!=NULL) {
		itfid = atoi(token);
#if defined(ZTE_531B_BRIDGE_SC) || defined(ZTE_GENERAL_ROUTER_SC)
		setitfPrio(itfid, priority );
#endif
		itfdomain = IF_DOMAIN(itfid);
		itfid = itfid&0x0ff;

#ifdef ITF_GROUP_4P
		if (itfdomain == DOMAIN_ELAN) {
			MIB_CE_SW_PORT_T Entry;

			num = mib_chain_total(MIB_SW_PORT_TBL);
			if (itfid < num) {
				if (!mib_chain_get(MIB_SW_PORT_TBL, itfid, (void *)&Entry))
					return;
				Entry.itfGroup = grpnum;
				mib_chain_update(MIB_SW_PORT_TBL, (void *)&Entry, itfid);
			}
		}
#elif defined(ITF_GROUP_1P)
		if (itfdomain == DOMAIN_ELAN) {
			char mygroup;
			mygroup = (char)grpnum;
			if (itfid == 0)
				mib_set(MIB_ETH_ITF_GROUP, (void *)&mygroup);
		}
#endif

#ifdef WLAN_SUPPORT
		else if (itfdomain == DOMAIN_WLAN) {
			char mygroup;

			mygroup = (char)grpnum;
			if (itfid == 0)
				mib_set(MIB_WLAN_ITF_GROUP, (void *)&mygroup);

			// Added by Mason Yu
#ifdef WLAN_MBSSID
			for ( i=1; i<5; i++) {
				if (itfid == i) {// vap0
					mib_set(MIB_WLAN_VAP0_ITF_GROUP+i-1, (void *)&mygroup);
				}
			}

#endif
		}
#endif

#ifdef CONFIG_USB_ETH
		else if (itfdomain == DOMAIN_ULAN) {
			char mygroup;
			mygroup = (char)grpnum;
			if (itfid == 0)
				mib_set(MIB_USBETH_ITF_GROUP, (void *)&mygroup);
		}
#endif

		else if (itfdomain == DOMAIN_WAN) {
			MIB_CE_ATM_VC_T Entry;

			num = mib_chain_total(MIB_ATM_VC_TBL);
			for (i=0; i<num; i++) {
				if (!mib_chain_get(MIB_ATM_VC_TBL, i, (void *)&Entry))
					return;
				if (Entry.enable && Entry.ifIndex==itfid) {
					Entry.itfGroup = grpnum;
					mib_chain_update(MIB_ATM_VC_TBL, (void *)&Entry, i);
				}
			}
		}
		arg0=0;
	}
}

/*
 * Get interface group membership(bit mapped)
 */
static void get_group_membership(int *mbr)
{
	MIB_CE_SW_PORT_T Port;
	int i, k, total, num;
	char grpid;
	
	// flush membership
	for (k=0; k<IFGROUP_NUM; k++)
		mbr[k] = 0;
	
	// Ethernet LAN
	#ifdef ITF_GROUP_4P
	num = mib_chain_total(MIB_SW_PORT_TBL);
	for (k=0; k<num; k++) {
		if (!mib_chain_get(MIB_SW_PORT_TBL, k, (void *)&Port))
			continue;
		if (Port.itfGroup < IFGROUP_NUM)
			mbr[Port.itfGroup] |= (1<<k);
	}
	#else
	mib_get(MIB_ETH_ITF_GROUP, (void *)&grpid);
	if (grpid < IFGROUP_NUM)
		mbr[grpid] |= 1;
	#endif
	
	// Wireless
	#ifdef WLAN_SUPPORT
	mib_get(MIB_WLAN_ITF_GROUP, (void *)&grpid);
	if (grpid < IFGROUP_NUM)
		mbr[grpid] |= (1<<IFWLAN_SHIFT);
	#ifdef WLAN_MBSSID
	for ( k=1; k<5; k++ ) {
		mib_get(MIB_WLAN_VAP0_ITF_GROUP+k-1, (void *)&grpid);
		if (grpid < IFGROUP_NUM)
			mbr[grpid] |= (1<<(IFWLAN_SHIFT+k));
	}
	#endif
	#endif
	
	// USB-Eth
	#ifdef CONFIG_USB_ETH
	mib_get(MIB_USBETH_ITF_GROUP, (void *)&grpid);
	if (grpid < IFGROUP_NUM) {
		mbr[grpid] |= (1<<IFUSBETH_SHIFT);
	}  // Mason Yu 090903
	#endif
	
	// device is mandatory
	for (k=0; k<IFGROUP_NUM; k++)
		mbr[k] |= (1<<DEVICE_SHIFT);
}

static void set_group_membership(int *mbr)
{
	MIB_CE_ATM_VC_T pvcEntry;
	struct data_to_pass_st msg;
	char wanif[5];
	int i, k, total;
	
	/* set membership into vc */
	total = mib_chain_total(MIB_ATM_VC_TBL);
	for (i=0; i<total; i++) {
		int vcIdx;

		if (!mib_chain_get(MIB_ATM_VC_TBL, i, (void *)&pvcEntry))
			continue;
		if (!pvcEntry.enable)
			continue;
		
		vcIdx = VC_INDEX(pvcEntry.ifIndex);
		snprintf(wanif, 5, "vc%d", vcIdx);
		// mpoactl set vc0 group 1 member 3
		snprintf(msg.data, BUF_SIZE,
			"mpoactl set %s group 1 member %d",
			wanif, mbr[pvcEntry.itfGroup]);
		TRACE(STA_SCRIPT, "%s\n", msg.data);
		write_to_mpoad(&msg);
	}
	
	#ifdef ITF_GROUP_4P
	total = mib_chain_total(MIB_SW_PORT_TBL);
	#else
	total = 1;
	#endif
	for (i=0; i<IFGROUP_NUM; i++) {
		for (k=0; k<32; k++) {
			if (mbr[i]&(1<<k)) { // port k in group i
				if (k >= 0 && k < total) // eth LAN port
					setup_8305_vlan_eth_member(k, mbr[i]);
				#ifdef WLAN_SUPPORT
				if (k >= IFWLAN_SHIFT && k <= (IFWLAN_SHIFT+WLAN_MBSSID_NUM)) // wlan port
					setup_wlan_group_member(k-IFWLAN_SHIFT, mbr[i]);
				#endif
				#ifdef CONFIG_USB_ETH
				if (k == IFUSBETH_SHIFT) // usb-eth port
					setup_usb_group_member( mbr[i] );
				#endif
			}
		}
	}
	
}

/*
 * Setup interface group membership(bit mapped)
 * The bitmap for interface should follow this convention commonly shared
 * by sar driver (sar_send())
 * Bit-map:
 *  (bit11) | bit10 | bit9  | bit8  | bit7 | bit6  | bit5   |  bit4    |  bit3  |  bit2  |  bit1  |  bit0
 *  (usb0)  | vap3  | vap2  | vap1  | vap0 | wlan  | resvd  | device   |  lan3  |  lan2  |  lan1  |  lan0
 * If usb(switch port) add:
 *  (bit11) | bit10 | bit9  | bit8  | bit7 | bit6  | bit5   |   bit4   |  bit3  |  bit2  |  bit1  |  bit0
 *  (usb0)  | vap3  | vap2  | vap1  | vap0 | wlan  | device | lan4(usb)|  lan3  |  lan2  |  lan1  |  lan0
 */
void setupEth2pvc()
{
	int mbr[IFGROUP_NUM];
	
	#ifdef CONFIG_EXT_SWITCH
	// Enable virtual port
	if (!virtual_port_enabled)
		__dev_register_swport();
	#endif
	get_group_membership(mbr);
	set_group_membership(&mbr[0]);
	// enable Port Mapping
	__dev_setupPortMapping(1);
	// enable switch vlan
	#ifdef ITF_GROUP_4P
	setup_8305_vlan_enable();
	#endif
	printf("Port Mapping: Ethernet to pvc mapping started\n");
}

void setupEth2pvc_disable()
{
	MIB_CE_ATM_VC_T pvcEntry;
	int i, k, total, num, member;
	struct data_to_pass_st msg;
	char wanif[5];
	unsigned char vChar=0;

	// Mason Yu
	// If we disable Port Mapping, we should set all interface's membership to '0'.
	member=0;

	total = mib_chain_total(MIB_ATM_VC_TBL);
	for (i=0; i<total; i++) {
		int vcIdx;

		if (!mib_chain_get(MIB_ATM_VC_TBL, i, (void *)&pvcEntry))
			continue;
		if (!pvcEntry.enable)
			continue;

		vcIdx = VC_INDEX(pvcEntry.ifIndex);

		/* set membership into vc */
		snprintf(wanif, 5, "vc%d", vcIdx);
		// mpoactl set vc0 group 1 member 3
		snprintf(msg.data, BUF_SIZE,
			"mpoactl set %s group 1 member %d",
			wanif, member);
		TRACE(STA_SCRIPT, "%s\n", msg.data);
		write_to_mpoad(&msg);
	}

	/* set membership 0 into LAN ports */
	// Mason Yu. combine_1p_4p_PortMapping
	#ifdef ITF_GROUP_4P	
	num = mib_chain_total(MIB_SW_PORT_TBL);
	for (k=0; k<num; k++) {
		setup_8305_vlan_eth_member(k, member);
	}
	#elif defined(ITF_GROUP_1P)	
	setup_8305_vlan_eth_member(0, member);
	#endif

	#ifdef WLAN_SUPPORT
	for (k=0; k<=WLAN_MBSSID_NUM; k++)
		setup_wlan_group_member(k, member);
	#endif

	#ifdef CONFIG_USB_ETH
	/* set membership 0 into usb0 */
	setup_usb_group_member( member );
	#endif //CONFIG_USB_ETH

	// disable Port Mapping
	__dev_setupPortMapping(0);


	// disable 8305 vlan
	// Mason Yu. combine_1p_4p_PortMapping
	#ifdef ITF_GROUP_4P
	mib_get( MIB_MPMODE, (void *)&vChar);
	// If one the IGMP Snooping, IPQoS and Port Mapping is enabled, we  can not disable VLAN.
	if ( vChar == 0) {
		setup_8305_vlan_disable();
		__dev_setupVirtualPort(0);
	}
	#endif
	printf("Port Mapping: Ethernet to pvc mapping stopped\n");
}

/*------------------------------------------------------------------
 * Get a list of interface info. (itfInfo) of group.
 * where,
 * info: a list of interface info entries
 * len: max length of the info list
 * grpnum: group id
 *-----------------------------------------------------------------*/
int get_group_ifinfo(struct itfInfo *info, int len, int grpnum)
{
	unsigned int swNum, vcNum;
	int i, num;
	char mygroup;
	char strlan[]="LAN0", strPPP[]="ppp0", strvc[]="vc0";
	MIB_CE_SW_PORT_T Entry;
	MIB_CE_ATM_VC_T pvcEntry;
#ifdef CTC_WAN_NAME
	char wanname[100];
#endif

#ifdef ITF_GROUP_4P
	// LAN ports
	swNum = mib_chain_total(MIB_SW_PORT_TBL);
	num=0;
	for (i=swNum; i>0; i--) {
		if (!mib_chain_get(MIB_SW_PORT_TBL, i-1, (void *)&Entry))
			return -1;
		if (Entry.itfGroup == grpnum) {
			strlan[3] = '0'+virt2user[i-1];
			info[num].ifdomain = DOMAIN_ELAN;
			info[num].ifid=i-1;
			strncpy(info[num].name, strlan, 8);
			num++;
		}
		if (num > len)
			break;
	}
#elif defined(ITF_GROUP_1P)
        //eth0
        // Mason Yu
        num=0;
	mib_get(MIB_ETH_ITF_GROUP, (void *)&mygroup);
	if (mygroup == grpnum) {
		info[num].ifdomain = DOMAIN_ELAN;
		info[num].ifid=0;
		strcpy(info[num].name, (char *)ELANIF);
		num++;
	}
#endif

#ifdef WLAN_SUPPORT
	// wlan0
	mib_get(MIB_WLAN_ITF_GROUP, (void *)&mygroup);
	if (mygroup == grpnum) {
		info[num].ifdomain = DOMAIN_WLAN;
		info[num].ifid=0;
#if defined(ZTE_531B_BRIDGE_SC) || defined(ZTE_GENERAL_ROUTER_SC)
		sprintf(info[num].name, "SSID1(root)");
#else
		strncpy(info[num].name, (char *)WLANIF, 8);
#endif
		num++;
	}
#endif

#ifdef WLAN_MBSSID
	for (i=1; i<5; i++) {
		mib_get(MIB_WLAN_VAP0_ITF_GROUP+i-1, (void *)&mygroup);
		if (mygroup == grpnum) {
			info[num].ifdomain = DOMAIN_WLAN;
			info[num].ifid=i;
			//strncpy(info[num].name, "vap0", 8);
#if defined(ZTE_531B_BRIDGE_SC) || defined(ZTE_GENERAL_ROUTER_SC)
			sprintf(info[num].name, "SSID%d", i+1);
#else
			sprintf(info[num].name, "vap%d", i-1);
#endif
			num++;
		}
	}

#endif

#ifdef CONFIG_USB_ETH
	// usb0
	mib_get(MIB_USBETH_ITF_GROUP, (void *)&mygroup);
	if (mygroup == grpnum) {
		info[num].ifdomain = DOMAIN_ULAN;
		info[num].ifid=0;
		strcpy(info[num].name, (char *)USBETHIF);
		num++;
	}
#endif

	// vc
	vcNum = mib_chain_total(MIB_ATM_VC_TBL);

	for (i=0; i<vcNum; i++) {
		if (!mib_chain_get(MIB_ATM_VC_TBL, i, (void *)&pvcEntry))
		{
  			//websError(wp, 400, T("Get chain record error!\n"));
  			printf("Get chain record error!\n");
			return -1;
		}

		if (pvcEntry.enable == 0 || pvcEntry.itfGroup!=grpnum)
			continue;

#ifdef CTC_WAN_NAME
		getWanName(&pvcEntry, wanname);
		info[num].ifdomain = DOMAIN_WAN;
		info[num].ifid=pvcEntry.ifIndex;
		strncpy(info[num].name, wanname, 30);
		num++;
#else
		if (PPP_INDEX(pvcEntry.ifIndex) != 0x0f)
		{	// PPP interface
			strPPP[3] = '0'+PPP_INDEX(pvcEntry.ifIndex);
			info[num].ifdomain = DOMAIN_WAN;
			info[num].ifid=pvcEntry.ifIndex;
			strncpy(info[num].name, strPPP, 8);
			num++;
		}
		else
		{	// vc interface
			strvc[2] = '0'+VC_INDEX(pvcEntry.ifIndex);
			info[num].ifdomain = DOMAIN_WAN;
			info[num].ifid=pvcEntry.ifIndex;
			strncpy(info[num].name, strvc, 8);
			num++;
		}
#endif
		if (num > len)
			break;
	}

	return num;
}
#endif // of ITF_GROUP

/* =============================================================================================================*/

#if defined(CONFIG_EXT_SWITCH) && defined(VLAN_GROUP)
// enable/disable VLAN-Grouping
void __dev_setupVlanGrouping(int flag)
{
	struct ifreq ifr;
	struct ifvlan ifvl;

	strcpy(ifr.ifr_name, "eth0");
	ifr.ifr_data = (char *)&ifvl;
	ifvl.cmd=VLAN_SETVLANGROUPING;
	ifvl.enable=flag;
	setVlan(&ifr);
	printf("Port/VLAN mapping: %s\n", flag?"enabled":"disabled");
}

void setupEth2pvc_vlan()
{
#ifdef EMBED
	struct ifreq ifr;
	struct ifvlan ifvl;
	int i, k, mbr, maskbit;
	MIB_CE_VLAN_T vlan_entry;
	MIB_CE_SW_PORT_T sw_entry;
	unsigned char mode;

	mib_get(MIB_MPMODE, (void *)&mode);
#ifdef IP_QOS_VPORT
	if (mode&0x07) // IPQ, PortMapping and IGMP-snooping need to register virtual LAN ports by only once
#else
	if (mode&0x05)	// PortMapping and IGMP-snooping need to register virtual LAN ports
#endif
	if (!virtual_port_enabled)
	__dev_register_swport();

	if (!(mode&0x01))
		return;
	strcpy(ifr.ifr_name, "eth0");
	// set VLAN G(group1), H(group2), I(group3), J(group4)
	// VLANF(group0) is default group
	// VLAN A, B, C, D, E are used for LAN port identification
	ifvl.cmd=VLAN_SETINFO;
	//------------ set default group ---------------
	ifvl.vlanIdx=SW_PORT_NUM+1; // don't touch sw_port and cpu port
	ifvl.vid=0;
	ifvl.member = 0xffffffff;
	ifr.ifr_data = (char *)&ifvl;
	setVlan(&ifr);
	//------------ set user-defined group ----------
	for (i=1; i<VLAN_NUM; i++) {
		mib_chain_get(MIB_VLAN_TBL, i, (void *)&vlan_entry);
		ifvl.vlanIdx=SW_PORT_NUM+1+i; // don't touch sw_port and cpu port
		// VID 0 is used to identify priority frames and VID 4095(0xFFF) is reserved
		ifvl.vid=vlan_entry.tag;
		ifvl.member = vlan_entry.member; //mbr; // member ports:
		ifvl.member |= 0x10; // Set CPU as a member of each vlan.
		ifr.ifr_data = (char *)&ifvl;
		//printf("vlanidx: %d, vid: %d, mbr: 0x%x\n", ifvl.vlanIdx, ifvl.vid, ifvl.member);
		setVlan(&ifr);
	}

	//------------ Set LAN pvid ---------------
	ifvl.cmd=VLAN_SETPVIDX;
	for (i=0; i<SW_PORT_NUM; i++) {
		// set sw-port 0~3 vlan idex.
		mib_chain_get(MIB_SW_PORT_TBL, i, (void *)&sw_entry);
		ifvl.port=SW_PORT_NUM+1+i; // don't touch sw_port and cpu port
		ifvl.vlanIdx=SW_PORT_NUM+1+sw_entry.pvid;
		ifr.ifr_data = (char *)&ifvl;
		//printf("port: %d, pvid: %d\n", ifvl.port, ifvl.vlanIdx);
		setVlan(&ifr);
	}

	//------------ enable vlan-grouping -----------
	__dev_setupVlanGrouping(1);
	//------------ enable 8305 vlan -----------
	setup_8305_vlan_enable();
#endif	// of EMBED
}
void setupEth2pvc_vlan_disable()
{
	unsigned char vChar=0;

	// disable VLAN groupping
	__dev_setupVlanGrouping(0);

	// disable 8305 vlan
	mib_get( MIB_MPMODE, (void *)&vChar);
	// If one the IGMP Snooping, IPQoS and Port Mapping is enabled, we  can not disable VLAN.
	if ( vChar == 0) {
		printf("Disable switch VLAN.\n");
		setup_8305_vlan_disable();
		__dev_setupVirtualPort(0);
	}

	printf("Port/VLAN mapping stopped\n");
}

// membership bit-map:
// | resvd | vc7 vc6 vc5 vc4 | vc3 vc2 vc1 vc0 |
//  vap3 | vap2 vap1 vap0 wlan0 | resvd | LAN4 LAN3 LAN2 LAN1
void setgroup(char *list, int grpnum, int vlan_vid, int vlan_dhcps, int vlan_igmp)
{
	int itfid, itfdomain;
	int i, vlan_num, vc_num;
	char *arg0, *token;
	unsigned int member, maskbit;
	MIB_CE_VLAN_T vlan_entry;
	MIB_CE_ATM_VC_T vc_entry;
	MIB_CE_SW_PORT_T sw_entry;

	vlan_num = mib_chain_total(MIB_VLAN_TBL);
	if (grpnum <= 0 && grpnum >=vlan_num)
		return;
	mib_chain_get(MIB_VLAN_TBL, grpnum, (void *)&vlan_entry);
	// reset pvid
	maskbit = 1;
	for (i=0; i<4; i++) {
		if (vlan_entry.member & maskbit) {
			mib_chain_get(MIB_SW_PORT_TBL, i, (void *)&sw_entry);
			sw_entry.pvid = 0;
			mib_chain_update(MIB_SW_PORT_TBL, (void *)&sw_entry, i);
		}
		maskbit<<=1;
	}
	vlan_entry.member = 0;
	vlan_entry.tag = vlan_vid;
	vlan_entry.dhcps = vlan_dhcps;
	vlan_entry.igmp = vlan_igmp;
	vc_num = mib_chain_total(MIB_ATM_VC_TBL);

	member = 0;
	arg0=list;
	while ((token=strtok(arg0,","))!=NULL) {
		itfid = atoi(token);
		itfdomain = IF_DOMAIN(itfid);
		itfid = itfid&0x0ff;

		if (itfdomain == DOMAIN_ELAN) {
			// VLAN membershipt
			vlan_entry.member |= 1<<itfid;
			// set pvid
			mib_chain_get(MIB_SW_PORT_TBL, itfid, (void *)&sw_entry);
			sw_entry.pvid = grpnum;
			mib_chain_update(MIB_SW_PORT_TBL, (void *)&sw_entry, itfid);
		}
		else if (itfdomain == DOMAIN_WAN) {
			for (i=0; i<vc_num; i++) {
				mib_chain_get(MIB_ATM_VC_TBL, i, (void *)&vc_entry);
				if (vc_entry.enable && VC_INDEX(vc_entry.ifIndex)==VC_INDEX(itfid)) {
					if (vc_entry.cmode == ADSL_BR1483 ||
						vc_entry.cmode == ADSL_MER1483 ||
						vc_entry.cmode == ADSL_PPPoE)
						vlan_entry.member |= (1<<16)<<VC_INDEX(itfid);
				}
			}
		}
#ifdef WLAN_SUPPORT
		else if (itfdomain == DOMAIN_WLAN) {
			vlan_entry.member |= 0x0100<<itfid;
		}
#endif
		arg0=0;
	}
	//printf("vlan_entry %d, member=0x%x, tag=%d\n", grpnum, vlan_entry.member, vlan_entry.tag);
	mib_chain_update(MIB_VLAN_TBL, (void *)&vlan_entry, grpnum);
}

/*------------------------------------------------------------------
 * Get a list of interface info. (itfInfo) of group.
 * where,
 * info: a list of interface info entries
 * len: max length of the info list
 * grpnum: group id
 *-----------------------------------------------------------------*/
int get_group_ifinfo(struct itfInfo *info, int len, int grpnum)
{
	unsigned int swNum, vcNum;
	int i, k, num;
	unsigned int dft_member, maskbit;
	char mygroup;
	char strlan[]="LAN0", strPPP[]="ppp0", strvc[]="vc0";
	MIB_CE_ATM_VC_T pvcEntry;
	MIB_CE_VLAN_T vlan_entry;

	if (grpnum == 0) { // default group
		#ifdef WLAN_SUPPORT
		dft_member = 0x010f;
		#else
		dft_member = 0x0f;
		#endif
		// LAN -- get default LAN members
		for (k=1; k<VLAN_NUM; k++) {
			maskbit = 1;
			mib_chain_get(MIB_VLAN_TBL, k, (void *)&vlan_entry);
			// ELAN
			for (i=0; i<4; i++) {
				if (vlan_entry.member & maskbit) {
					dft_member &= (~maskbit);
				}
				maskbit<<=1;
			}
			#ifdef WLAN_SUPPORT
			// WLAN
			maskbit = 0x0100;
			if (vlan_entry.member & maskbit)
				dft_member &= (~maskbit);
			#endif
		}
		// assign default LAN info.
		num = 0;
		// ELAN
		maskbit = 1;
		for (i=0; i<4; i++) {
			if (num > len)
				break;
			if (dft_member & maskbit) {
				info[num].ifdomain = DOMAIN_ELAN;
				info[num].ifid=i;
				strlan[3] = '0'+virt2user[i];
				strncpy(info[num].name, strlan, 8);
				num++;
			}
			maskbit<<=1;
		}
		#ifdef WLAN_SUPPORT
		// WLAN
		maskbit = 0x0100;
		if (dft_member & maskbit) {
			info[num].ifdomain = DOMAIN_WLAN;
			info[num].ifid=0;
			strncpy(info[num].name, (char *)WLANIF, 8);
			num++;
		}
		#endif

		// WAN -- get default WAN members
		vcNum = mib_chain_total(MIB_ATM_VC_TBL);
		for (i=0; i<vcNum; i++) {
			mib_chain_get(MIB_ATM_VC_TBL, i, (void *)&pvcEntry);
			if (pvcEntry.enable) {
				if (pvcEntry.cmode == ADSL_BR1483 ||
					pvcEntry.cmode == ADSL_MER1483 ||
					pvcEntry.cmode == ADSL_PPPoE)
					dft_member |= (1<<16)<<VC_INDEX(pvcEntry.ifIndex);
			}
		}
		// assign default WAN info.
		maskbit = 0x10000;
		for (i=0; i<8; i++) {
			if (num > len)
				break;
			if (dft_member & maskbit) {
				info[num].ifdomain = DOMAIN_WAN;
				info[num].ifid=(0xf0|i);
				strvc[2] = '0'+i;
				strncpy(info[num].name, strvc, 8);
				num++;
			}
			maskbit<<=1;
		}
		
		return num;
	}
	if (grpnum <=0 && grpnum >=VLAN_NUM)
		return 0;

	mib_chain_get(MIB_VLAN_TBL, grpnum, (void *)&vlan_entry);
	num = 0;
	// LAN
	maskbit = 1;
	for (i=0; i<4; i++) {
		if (num > len)
			break;
		if (vlan_entry.member & maskbit) {
			info[num].ifdomain = DOMAIN_ELAN;
			info[num].ifid=i;
			strlan[3] = '0'+virt2user[i];
			strncpy(info[num].name, strlan, 8);
			num++;
		}
		maskbit<<=1;
	}
	#ifdef WLAN_SUPPORT
	// WLAN
	maskbit = 0x0100;
	if (vlan_entry.member & maskbit) {
		info[num].ifdomain = DOMAIN_WLAN;
		info[num].ifid=0;
		strncpy(info[num].name, (char *)WLANIF, 8);
		num++;
	}
	#endif
	#ifdef WLAN_MBSSID
	#endif

	// WAN
	maskbit = 0x10000;
	for (i=0; i<8; i++) {
		if (num > len)
			break;
		if (vlan_entry.member & maskbit) {
			info[num].ifdomain = DOMAIN_WAN;
			info[num].ifid=(0xf0|i);
			strvc[2] = '0'+i;
			strncpy(info[num].name, strvc, 8);
			num++;
		}
		maskbit<<=1;
	}

	return num;
}

void vg_setup_iptables()
{
	MIB_CE_VLAN_T vlan_entry;
	int i, k;
	unsigned int maskbit;
	char ifname[] = "eth0_sw0";
	unsigned char mode;

	va_cmd(IPTABLES, 2, 1, "-F", "itfgroup");
	va_cmd(EBTABLES, 2, 1, "-F", "itfgroup_in");
	va_cmd(EBTABLES, 2, 1, "-F", "itfgroup_fw");
	mib_get(MIB_MPMODE, (void *)&mode);
	if (!(mode&0x01))
		return;

	for (i=1; i<VLAN_NUM; i++) {
		mib_chain_get(MIB_VLAN_TBL, i, (void *)&vlan_entry);
		if (!vlan_entry.dhcps) {
			// Disable dhcp service for LAN port in this group.
			maskbit = 1;
			for (k=0; k<4; k++) {
				// find the virtual LAN ports in this group
				if (vlan_entry.member & maskbit) {
					ifname[7] = '0'+ k;
					// iptables -A itfgroup -i eth0_sw0 -p udp --dport 67 -j DROP
					va_cmd(IPTABLES, 10, 1, (char *)FW_ADD, "itfgroup", (char *)ARG_I, (char *)ifname, "-p", "udp", (char *)FW_DPORT, (char *)PORT_DHCP, "-j", (char *)FW_DROP);
				}
				maskbit<<=1;
			}
			// Disable dhcp service for Wlan port in this group.
			#ifdef WLAN_SUPPORT
			maskbit = 0x0100;
			if (vlan_entry.member & maskbit) {
				// iptables -A itfgroup -i wlan0 -p udp --dport 67 -j DROP
				va_cmd(IPTABLES, 10, 1, (char *)FW_ADD, "itfgroup", (char *)ARG_I, "wlan0", "-p", "udp", (char *)FW_DPORT, (char *)PORT_DHCP, "-j", (char *)FW_DROP);
			}
			#endif
		}
		else { // just in case
			// Device support dhcp service, so block dhcp packet from bridging.
			maskbit = 1;
			for (k=0; k<4; k++) {
				// find the virtual LAN ports in this group
				if (vlan_entry.member & maskbit) {
					ifname[7] = '0'+ k;
					// ebtables -A itfgroup_fw -i eth0_sw0 -p IPv4 --ip-proto 17 --ip-dport 67 -j DROP
					va_cmd(EBTABLES, 12, 1, (char *)FW_ADD, "itfgroup_fw", (char *)ARG_I, (char *)ifname, "-p", "IPv4", "--ip-proto", "17", "--ip-dport", "67", "-j", (char *)FW_DROP);
				}
				maskbit<<=1;
			}
			#ifdef WLAN_SUPPORT
			maskbit = 0x0100;
			if (vlan_entry.member & maskbit) {
				// ebtables -A itfgroup_fw -i wlan0 -p IPv4 --ip-proto 17 --ip-dport 67 -j DROP
				va_cmd(EBTABLES, 12, 1, (char *)FW_ADD, "itfgroup_fw", (char *)ARG_I, "wlan0", "-p", "IPv4", "--ip-proto", "17", "--ip-dport", "67", "-j", (char *)FW_DROP);
			}
			#endif
		}
		if (!vlan_entry.igmp) {
			// Block IGMP packet for LAN port in the group.
			maskbit = 1;
			for (k=0; k<4; k++) {
				// find the virtual LAN ports in this group
				if (vlan_entry.member & maskbit) {
					ifname[7] = '0'+ k;
					// ebtables -A itfgroup_in -i eth0_sw0 -p IPv4 --ip-proto 2 -j DROP
					va_cmd(EBTABLES, 10, 1, (char *)FW_ADD, "itfgroup_in", (char *)ARG_I, (char *)ifname, "-p", "IPv4", "--ip-proto", "2", "-j", (char *)FW_DROP);
					// ebtables -A itfgroup_fw -i eth0_sw0 -p IPv4 --ip-proto 2 -j DROP
					va_cmd(EBTABLES, 10, 1, (char *)FW_ADD, "itfgroup_fw", (char *)ARG_I, (char *)ifname, "-p", "IPv4", "--ip-proto", "2", "-j", (char *)FW_DROP);
				}
				maskbit<<=1;
			}
			// Block IGMP packet for WLAN port in the group.
			#ifdef WLAN_SUPPORT
			maskbit = 0x0100;
			if (vlan_entry.member & maskbit) {
				// ebtables -A itfgroup_in -i wlan0 -p IPv4 --ip-proto 2 -j DROP
				va_cmd(EBTABLES, 10, 1, (char *)FW_ADD, "itfgroup_in", (char *)ARG_I, "wlan0", "-p", "IPv4", "--ip-proto", "2", "-j", (char *)FW_DROP);
				// ebtables -A itfgroup_fw -i wlan0 -p IPv4 --ip-proto 2 -j DROP
				va_cmd(EBTABLES, 10, 1, (char *)FW_ADD, "itfgroup_fw", (char *)ARG_I, "wlan0", "-p", "IPv4", "--ip-proto", "2", "-j", (char *)FW_DROP);
			}
			#endif
		}
	}
}
#endif // of CONFIG_EXT_SWITCH && VLAN_GROUP


